@@ -12,6 +12,7 @@ const CatalogServiceListName = "catalog-services-list"
 
 // CatalogServiceList supports fetching service names via the catalog.
 type CatalogServiceList struct {
+	RegisterOptionsBlockingRefresh
 	RPC RPC
 }
 
@@ -49,7 +50,3 @@ func (c *CatalogServiceList) Fetch(opts cache.FetchOptions, req cache.Request) (
 	result.Index = reply.QueryMeta.Index
 	return result, nil
 }
-
-func (c *CatalogServiceList) SupportsBlocking() bool {
-	return true
-}
