@@ -1,7 +1,10 @@
 package cachetype
 
 import (
+	"crypto/x509"
+	"encoding/pem"
 	"fmt"
+	"net"
 	"strings"
 	"sync/atomic"
 	"testing"
@@ -965,6 +968,68 @@ func TestConnectCALeaf_expiringLeaf(t *testing.T) {
 	}
 }
 
+func TestConnectCALeaf_DNSSANForService(t *testing.T) {
+	t.Parallel()
+
+	require := require.New(t)
+	rpc := TestRPC(t)
+	defer rpc.AssertExpectations(t)
+
+	typ, rootsCh := testCALeafType(t, rpc)
+	defer close(rootsCh)
+
+	caRoot := connect.TestCA(t, nil)
+	caRoot.Active = true
+	rootsCh <- structs.IndexedCARoots{
+		ActiveRootID: caRoot.ID,
+		TrustDomain:  "fake-trust-domain.consul",
+		Roots: []*structs.CARoot{
+			caRoot,
+		},
+		QueryMeta: structs.QueryMeta{Index: 1},
+	}
+
+	// Instrument ConnectCA.Sign to
+	var caReq *structs.CASignRequest
+	rpc.On("RPC", "ConnectCA.Sign", mock.Anything, mock.Anything).Return(nil).
+		Run(func(args mock.Arguments) {
+			reply := args.Get(2).(*structs.IssuedCert)
+			leaf, _ := connect.TestLeaf(t, "web", caRoot)
+			reply.CertPEM = leaf
+
+			caReq = args.Get(1).(*structs.CASignRequest)
+		})
+
+	opts := cache.FetchOptions{MinIndex: 0, Timeout: 10 * time.Second}
+	req := &ConnectCALeafRequest{
+		Datacenter: "dc1",
+		Service:    "web",
+		DNSSAN:     []string{"test.example.com"},
+	}
+	_, err := typ.Fetch(opts, req)
+	require.NoError(err)
+
+	pemBlock, _ := pem.Decode([]byte(caReq.CSR))
+	csr, err := x509.ParseCertificateRequest(pemBlock.Bytes)
+	require.NoError(err)
+	require.Equal(csr.DNSNames, []string{"test.example.com"})
+}
+
+// testConnectCaRoot wraps ConnectCARoot to disable refresh so that the gated
+// channel controls the request directly. Otherwise, we get background refreshes and
+// it screws up the ordering of the channel reads of the testGatedRootsRPC
+// implementation.
+type testConnectCaRoot struct {
+	ConnectCARoot
+}
+
+func (r testConnectCaRoot) RegisterOptions() cache.RegisterOptions {
+	return cache.RegisterOptions{
+		Refresh:          false,
+		SupportsBlocking: true,
+	}
+}
+
 // testCALeafType returns a *ConnectCALeaf that is pre-configured to
 // use the given RPC implementation for "ConnectCA.Sign" operations.
 func testCALeafType(t *testing.T, rpc RPC) (*ConnectCALeaf, chan structs.IndexedCARoots) {
@@ -975,15 +1040,10 @@ func testCALeafType(t *testing.T, rpc RPC) (*ConnectCALeaf, chan structs.Indexed
 	rootsRPC := &testGatedRootsRPC{ValueCh: rootsCh}
 
 	// Create a cache
-	c := cache.TestCache(t)
-	c.RegisterType(ConnectCARootName, &ConnectCARoot{RPC: rootsRPC}, &cache.RegisterOptions{
-		// Disable refresh so that the gated channel controls the
-		// request directly. Otherwise, we get background refreshes and
-		// it screws up the ordering of the channel reads of the
-		// testGatedRootsRPC implementation.
-		Refresh: false,
+	c := cache.New(cache.Options{})
+	c.RegisterType(ConnectCARootName, &testConnectCaRoot{
+		ConnectCARoot: ConnectCARoot{RPC: rootsRPC},
 	})
-
 	// Create the leaf type
 	return &ConnectCALeaf{
 		RPC:        rpc,
@@ -1022,12 +1082,26 @@ func TestConnectCALeaf_Key(t *testing.T) {
 	r1 := ConnectCALeafRequest{Service: "web"}
 	r2 := ConnectCALeafRequest{Service: "api"}
 
+	r3 := ConnectCALeafRequest{DNSSAN: []string{"a.com"}}
+	r4 := ConnectCALeafRequest{DNSSAN: []string{"b.com"}}
+
+	r5 := ConnectCALeafRequest{IPSAN: []net.IP{net.ParseIP("192.168.4.139")}}
+	r6 := ConnectCALeafRequest{IPSAN: []net.IP{net.ParseIP("192.168.4.140")}}
 	// hashstructure will hash the service name + ent meta to produce this key
 	r1Key := r1.Key()
 	r2Key := r2.Key()
+
+	r3Key := r3.Key()
+	r4Key := r4.Key()
+
+	r5Key := r5.Key()
+	r6Key := r6.Key()
+
 	require.True(t, strings.HasPrefix(r1Key, "service:"), "Key %s does not start with service:", r1Key)
 	require.True(t, strings.HasPrefix(r2Key, "service:"), "Key %s does not start with service:", r2Key)
 	require.NotEqual(t, r1Key, r2Key, "Cache keys for different services are not equal")
+	require.NotEqual(t, r3Key, r4Key, "Cache keys for different DNSSAN are not equal")
+	require.NotEqual(t, r5Key, r6Key, "Cache keys for different IPSAN are not equal")
 	r := ConnectCALeafRequest{Agent: "abc"}
 	require.Equal(t, "agent:abc", r.Key())
 }
