@@ -12,6 +12,7 @@ const IntentionMatchName = "intention-match"
 
 // IntentionMatch supports fetching the intentions via match queries.
 type IntentionMatch struct {
+	RegisterOptionsBlockingRefresh
 	RPC RPC
 }
 
@@ -43,7 +44,3 @@ func (c *IntentionMatch) Fetch(opts cache.FetchOptions, req cache.Request) (cach
 	result.Index = reply.Index
 	return result, nil
 }
-
-func (c *IntentionMatch) SupportsBlocking() bool {
-	return true
-}
