@@ -13,6 +13,7 @@ const NodeServicesName = "node-services"
 // NodeServices supports fetching discovering service instances via the
 // catalog.
 type NodeServices struct {
+	RegisterOptionsBlockingRefresh
 	RPC RPC
 }
 
@@ -50,7 +51,3 @@ func (c *NodeServices) Fetch(opts cache.FetchOptions, req cache.Request) (cache.
 	result.Index = reply.QueryMeta.Index
 	return result, nil
 }
-
-func (c *NodeServices) SupportsBlocking() bool {
-	return true
-}
