@@ -13,10 +13,11 @@ const PreparedQueryName = "prepared-query"
 // PreparedQuery supports fetching discovering service instances via prepared
 // queries.
 type PreparedQuery struct {
+	RegisterOptionsNoRefresh
 	RPC RPC
 }
 
-func (c *PreparedQuery) Fetch(opts cache.FetchOptions, req cache.Request) (cache.FetchResult, error) {
+func (c *PreparedQuery) Fetch(_ cache.FetchOptions, req cache.Request) (cache.FetchResult, error) {
 	var result cache.FetchResult
 
 	// The request should be a PreparedQueryExecuteRequest.
@@ -47,8 +48,3 @@ func (c *PreparedQuery) Fetch(opts cache.FetchOptions, req cache.Request) (cache
 
 	return result, nil
 }
-
-func (c *PreparedQuery) SupportsBlocking() bool {
-	// Prepared queries don't support blocking.
-	return false
-}
