@@ -30,7 +30,7 @@ func TestCheckAlias_remoteErrBackoff(t *testing.T) {
 		RPC:       rpc,
 	}
 
-	rpc.Reply.Store(fmt.Errorf("failure"))
+	rpc.AddReply("Health.NodeChecks", fmt.Errorf("failure"))
 
 	chk.Start()
 	defer chk.Stop()
@@ -62,7 +62,7 @@ func TestCheckAlias_remoteNoChecks(t *testing.T) {
 		RPC:       rpc,
 	}
 
-	rpc.Reply.Store(structs.IndexedHealthChecks{})
+	rpc.AddReply("Health.NodeChecks", structs.IndexedHealthChecks{})
 
 	chk.Start()
 	defer chk.Stop()
@@ -88,7 +88,7 @@ func TestCheckAlias_remoteNodeFailure(t *testing.T) {
 		RPC:       rpc,
 	}
 
-	rpc.Reply.Store(structs.IndexedHealthChecks{
+	rpc.AddReply("Health.NodeChecks", structs.IndexedHealthChecks{
 		HealthChecks: []*structs.HealthCheck{
 			// Should ignore non-matching node
 			&structs.HealthCheck{
@@ -137,7 +137,7 @@ func TestCheckAlias_remotePassing(t *testing.T) {
 		RPC:       rpc,
 	}
 
-	rpc.Reply.Store(structs.IndexedHealthChecks{
+	rpc.AddReply("Health.NodeChecks", structs.IndexedHealthChecks{
 		HealthChecks: []*structs.HealthCheck{
 			// Should ignore non-matching node
 			&structs.HealthCheck{
@@ -171,6 +171,116 @@ func TestCheckAlias_remotePassing(t *testing.T) {
 	})
 }
 
+// Remote service has no healtchecks, but service exists on remote host
+func TestCheckAlias_remotePassingWithoutChecksButWithService(t *testing.T) {
+	t.Parallel()
+
+	notify := newMockAliasNotify()
+	chkID := structs.NewCheckID("foo", nil)
+	rpc := &mockRPC{}
+	chk := &CheckAlias{
+		Node:      "remote",
+		ServiceID: structs.ServiceID{ID: "web"},
+		CheckID:   chkID,
+		Notify:    notify,
+		RPC:       rpc,
+	}
+
+	rpc.AddReply("Health.NodeChecks", structs.IndexedHealthChecks{
+		HealthChecks: []*structs.HealthCheck{
+			// Should ignore non-matching node
+			&structs.HealthCheck{
+				Node:      "A",
+				ServiceID: "web",
+				Status:    api.HealthCritical,
+			},
+
+			// Should ignore non-matching service
+			&structs.HealthCheck{
+				Node:      "remote",
+				ServiceID: "db",
+				Status:    api.HealthCritical,
+			},
+		},
+	})
+
+	injected := structs.IndexedNodeServices{
+		NodeServices: &structs.NodeServices{
+			Node: &structs.Node{
+				Node: "remote",
+			},
+			Services: make(map[string]*structs.NodeService),
+		},
+		QueryMeta: structs.QueryMeta{},
+	}
+	injected.NodeServices.Services["web"] = &structs.NodeService{
+		Service: "web",
+		ID:      "web",
+	}
+	rpc.AddReply("Catalog.NodeServices", injected)
+
+	chk.Start()
+	defer chk.Stop()
+	retry.Run(t, func(r *retry.R) {
+		if got, want := notify.State(chkID), api.HealthPassing; got != want {
+			r.Fatalf("got state %q want %q", got, want)
+		}
+	})
+}
+
+// Remote service has no healtchecks, service does not exists on remote host
+func TestCheckAlias_remotePassingWithoutChecksAndWithoutService(t *testing.T) {
+	t.Parallel()
+
+	notify := newMockAliasNotify()
+	chkID := structs.NewCheckID("foo", nil)
+	rpc := &mockRPC{}
+	chk := &CheckAlias{
+		Node:      "remote",
+		ServiceID: structs.ServiceID{ID: "web"},
+		CheckID:   chkID,
+		Notify:    notify,
+		RPC:       rpc,
+	}
+
+	rpc.AddReply("Health.NodeChecks", structs.IndexedHealthChecks{
+		HealthChecks: []*structs.HealthCheck{
+			// Should ignore non-matching node
+			&structs.HealthCheck{
+				Node:      "A",
+				ServiceID: "web",
+				Status:    api.HealthCritical,
+			},
+
+			// Should ignore non-matching service
+			&structs.HealthCheck{
+				Node:      "remote",
+				ServiceID: "db",
+				Status:    api.HealthCritical,
+			},
+		},
+	})
+
+	injected := structs.IndexedNodeServices{
+		NodeServices: &structs.NodeServices{
+			Node: &structs.Node{
+				Node: "remote",
+			},
+			Services: make(map[string]*structs.NodeService),
+		},
+		QueryMeta: structs.QueryMeta{},
+	}
+	rpc.AddReply("Catalog.NodeServices", injected)
+
+	chk.Start()
+	defer chk.Stop()
+	retry.Run(t, func(r *retry.R) {
+		if got, want := notify.State(chkID), api.HealthCritical; got != want {
+			r.Fatalf("got state %q want %q", got, want)
+		}
+	})
+}
+
 // If any checks are critical, it should be critical
 func TestCheckAlias_remoteCritical(t *testing.T) {
 	t.Parallel()
@@ -186,7 +296,7 @@ func TestCheckAlias_remoteCritical(t *testing.T) {
 		RPC:       rpc,
 	}
 
-	rpc.Reply.Store(structs.IndexedHealthChecks{
+	rpc.AddReply("Health.NodeChecks", structs.IndexedHealthChecks{
 		HealthChecks: []*structs.HealthCheck{
 			// Should ignore non-matching node
 			&structs.HealthCheck{
@@ -241,7 +351,7 @@ func TestCheckAlias_remoteWarning(t *testing.T) {
 		RPC:       rpc,
 	}
 
-	rpc.Reply.Store(structs.IndexedHealthChecks{
+	rpc.AddReply("Health.NodeChecks", structs.IndexedHealthChecks{
 		HealthChecks: []*structs.HealthCheck{
 			// Should ignore non-matching node
 			&structs.HealthCheck{
@@ -295,7 +405,7 @@ func TestCheckAlias_remoteNodeOnlyPassing(t *testing.T) {
 		RPC:     rpc,
 	}
 
-	rpc.Reply.Store(structs.IndexedHealthChecks{
+	rpc.AddReply("Health.NodeChecks", structs.IndexedHealthChecks{
 		HealthChecks: []*structs.HealthCheck{
 			// Should ignore non-matching node
 			&structs.HealthCheck{
@@ -342,7 +452,7 @@ func TestCheckAlias_remoteNodeOnlyCritical(t *testing.T) {
 		RPC:     rpc,
 	}
 
-	rpc.Reply.Store(structs.IndexedHealthChecks{
+	rpc.AddReply("Health.NodeChecks", structs.IndexedHealthChecks{
 		HealthChecks: []*structs.HealthCheck{
 			// Should ignore non-matching node
 			&structs.HealthCheck{
@@ -403,9 +513,19 @@ type mockRPC struct {
 	Calls uint32       // Read-only, number of RPC calls
 	Args  atomic.Value // Read-only, the last args sent
 
-	// Write-only, the reply to send. If of type "error" then an error will
+	// Write-only, the replies to send, indexed per method. If of type "error" then an error will
 	// be returned from the RPC call.
-	Reply atomic.Value
+	Replies map[string]*atomic.Value
+}
+
+func (m *mockRPC) AddReply(method string, reply interface{}) {
+	if m.Replies == nil {
+		m.Replies = make(map[string]*atomic.Value)
+	}
+	val := &atomic.Value{}
+	val.Store(reply)
+	m.Replies[method] = val
+
 }
 
 func (m *mockRPC) RPC(method string, args interface{}, reply interface{}) error {
@@ -424,12 +544,15 @@ func (m *mockRPC) RPC(method string, args interface{}, reply interface{}) error
 	}
 	replyv = replyv.Elem()                  // Get pointer value
 	replyv.Set(reflect.Zero(replyv.Type())) // Reset to zero value
-	if v := m.Reply.Load(); v != nil {
+	repl := m.Replies[method]
+	if repl == nil {
+		return fmt.Errorf("No Such Method: %s", method)
+	}
+	if v := m.Replies[method].Load(); v != nil {
 		// Return an error if the reply is an error type
 		if err, ok := v.(error); ok {
 			return err
 		}
-
 		replyv.Set(reflect.ValueOf(v)) // Set to reply value if non-nil
 	}
 
@@ -442,6 +565,8 @@ func TestCheckAlias_localInitialStatus(t *testing.T) {
 	t.Parallel()
 
 	notify := newMockAliasNotify()
+	// We fake a local service web to ensure check if passing works
+	notify.Notify.AddServiceID(structs.ServiceID{ID: "web"})
 	chkID := structs.NewCheckID(types.CheckID("foo"), nil)
 	rpc := &mockRPC{}
 	chk := &CheckAlias{
@@ -463,3 +588,27 @@ func TestCheckAlias_localInitialStatus(t *testing.T) {
 		}
 	})
 }
+
+// Local check on non-existing service
+func TestCheckAlias_localInitialStatusShouldFailBecauseNoService(t *testing.T) {
+	t.Parallel()
+
+	notify := newMockAliasNotify()
+	chkID := structs.NewCheckID(types.CheckID("foo"), nil)
+	rpc := &mockRPC{}
+	chk := &CheckAlias{
+		ServiceID: structs.ServiceID{ID: "web"},
+		CheckID:   chkID,
+		Notify:    notify,
+		RPC:       rpc,
+	}
+
+	chk.Start()
+	defer chk.Stop()
+
+	retry.Run(t, func(r *retry.R) {
+		if got, want := notify.State(chkID), api.HealthCritical; got != want {
+			r.Fatalf("got state %q want %q", got, want)
+		}
+	})
+}
