@@ -3,8 +3,6 @@ package checks
 import (
 	"bytes"
 	"fmt"
-	"io/ioutil"
-	"log"
 	"net"
 	"net/http"
 	"net/http/httptest"
@@ -16,10 +14,12 @@ import (
 	"time"
 
 	"github.com/hashicorp/consul/agent/mock"
+	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/api"
+	"github.com/hashicorp/consul/sdk/testutil"
 	"github.com/hashicorp/consul/sdk/testutil/retry"
-	"github.com/hashicorp/consul/types"
-	uuid "github.com/hashicorp/go-uuid"
+	"github.com/hashicorp/go-uuid"
+	"github.com/stretchr/testify/require"
 )
 
 func uniqueID() string {
@@ -43,21 +43,26 @@ func TestCheckMonitor_Script(t *testing.T) {
 	for _, tt := range tests {
 		t.Run(tt.status, func(t *testing.T) {
 			notif := mock.NewNotify()
+			logger := testutil.Logger(t)
+			statusHandler := NewStatusHandler(notif, logger, 0, 0)
+
+			cid := structs.NewCheckID("foo", nil)
 			check := &CheckMonitor{
 				Notify:        notif,
-				CheckID:       types.CheckID("foo"),
+				CheckID:       cid,
 				Script:        tt.script,
 				Interval:      25 * time.Millisecond,
-				Logger:        log.New(ioutil.Discard, uniqueID(), log.LstdFlags),
 				OutputMaxSize: DefaultBufSize,
+				Logger:        logger,
+				StatusHandler: statusHandler,
 			}
 			check.Start()
 			defer check.Stop()
 			retry.Run(t, func(r *retry.R) {
-				if got, want := notif.Updates("foo"), 2; got < want {
+				if got, want := notif.Updates(cid), 2; got < want {
 					r.Fatalf("got %d updates want at least %d", got, want)
 				}
-				if got, want := notif.State("foo"), tt.status; got != want {
+				if got, want := notif.State(cid), tt.status; got != want {
 					r.Fatalf("got state %q want %q", got, want)
 				}
 			})
@@ -79,21 +84,26 @@ func TestCheckMonitor_Args(t *testing.T) {
 	for _, tt := range tests {
 		t.Run(tt.status, func(t *testing.T) {
 			notif := mock.NewNotify()
+			logger := testutil.Logger(t)
+			statusHandler := NewStatusHandler(notif, logger, 0, 0)
+			cid := structs.NewCheckID("foo", nil)
+
 			check := &CheckMonitor{
 				Notify:        notif,
-				CheckID:       types.CheckID("foo"),
+				CheckID:       cid,
 				ScriptArgs:    tt.args,
 				Interval:      25 * time.Millisecond,
-				Logger:        log.New(ioutil.Discard, uniqueID(), log.LstdFlags),
 				OutputMaxSize: DefaultBufSize,
+				Logger:        logger,
+				StatusHandler: statusHandler,
 			}
 			check.Start()
 			defer check.Stop()
 			retry.Run(t, func(r *retry.R) {
-				if got, want := notif.Updates("foo"), 2; got < want {
+				if got, want := notif.Updates(cid), 2; got < want {
 					r.Fatalf("got %d updates want at least %d", got, want)
 				}
-				if got, want := notif.State("foo"), tt.status; got != want {
+				if got, want := notif.State(cid), tt.status; got != want {
 					r.Fatalf("got state %q want %q", got, want)
 				}
 			})
@@ -104,14 +114,19 @@ func TestCheckMonitor_Args(t *testing.T) {
 func TestCheckMonitor_Timeout(t *testing.T) {
 	// t.Parallel() // timing test. no parallel
 	notif := mock.NewNotify()
+	logger := testutil.Logger(t)
+	statusHandler := NewStatusHandler(notif, logger, 0, 0)
+
+	cid := structs.NewCheckID("foo", nil)
 	check := &CheckMonitor{
 		Notify:        notif,
-		CheckID:       types.CheckID("foo"),
+		CheckID:       cid,
 		ScriptArgs:    []string{"sh", "-c", "sleep 1 && exit 0"},
 		Interval:      50 * time.Millisecond,
 		Timeout:       25 * time.Millisecond,
-		Logger:        log.New(ioutil.Discard, uniqueID(), log.LstdFlags),
 		OutputMaxSize: DefaultBufSize,
+		Logger:        logger,
+		StatusHandler: statusHandler,
 	}
 	check.Start()
 	defer check.Stop()
@@ -119,10 +134,10 @@ func TestCheckMonitor_Timeout(t *testing.T) {
 	time.Sleep(250 * time.Millisecond)
 
 	// Should have at least 2 updates
-	if notif.Updates("foo") < 2 {
+	if notif.Updates(cid) < 2 {
 		t.Fatalf("should have at least 2 updates %v", notif.UpdatesMap())
 	}
-	if notif.State("foo") != "critical" {
+	if notif.State(cid) != "critical" {
 		t.Fatalf("should be critical %v", notif.StateMap())
 	}
 }
@@ -130,13 +145,19 @@ func TestCheckMonitor_Timeout(t *testing.T) {
 func TestCheckMonitor_RandomStagger(t *testing.T) {
 	// t.Parallel() // timing test. no parallel
 	notif := mock.NewNotify()
+	logger := testutil.Logger(t)
+	statusHandler := NewStatusHandler(notif, logger, 0, 0)
+
+	cid := structs.NewCheckID("foo", nil)
+
 	check := &CheckMonitor{
 		Notify:        notif,
-		CheckID:       types.CheckID("foo"),
+		CheckID:       cid,
 		ScriptArgs:    []string{"sh", "-c", "exit 0"},
 		Interval:      25 * time.Millisecond,
-		Logger:        log.New(ioutil.Discard, uniqueID(), log.LstdFlags),
 		OutputMaxSize: DefaultBufSize,
+		Logger:        logger,
+		StatusHandler: statusHandler,
 	}
 	check.Start()
 	defer check.Stop()
@@ -144,11 +165,11 @@ func TestCheckMonitor_RandomStagger(t *testing.T) {
 	time.Sleep(500 * time.Millisecond)
 
 	// Should have at least 1 update
-	if notif.Updates("foo") < 1 {
+	if notif.Updates(cid) < 1 {
 		t.Fatalf("should have 1 or more updates %v", notif.UpdatesMap())
 	}
 
-	if notif.State("foo") != api.HealthPassing {
+	if notif.State(cid) != api.HealthPassing {
 		t.Fatalf("should be %v %v", api.HealthPassing, notif.StateMap())
 	}
 }
@@ -156,13 +177,18 @@ func TestCheckMonitor_RandomStagger(t *testing.T) {
 func TestCheckMonitor_LimitOutput(t *testing.T) {
 	t.Parallel()
 	notif := mock.NewNotify()
+	logger := testutil.Logger(t)
+	statusHandler := NewStatusHandler(notif, logger, 0, 0)
+	cid := structs.NewCheckID("foo", nil)
+
 	check := &CheckMonitor{
 		Notify:        notif,
-		CheckID:       types.CheckID("foo"),
+		CheckID:       cid,
 		ScriptArgs:    []string{"od", "-N", "81920", "/dev/urandom"},
 		Interval:      25 * time.Millisecond,
-		Logger:        log.New(ioutil.Discard, uniqueID(), log.LstdFlags),
 		OutputMaxSize: DefaultBufSize,
+		Logger:        logger,
+		StatusHandler: statusHandler,
 	}
 	check.Start()
 	defer check.Stop()
@@ -170,7 +196,7 @@ func TestCheckMonitor_LimitOutput(t *testing.T) {
 	time.Sleep(50 * time.Millisecond)
 
 	// Allow for extra bytes for the truncation message
-	if len(notif.Output("foo")) > DefaultBufSize+100 {
+	if len(notif.Output(cid)) > DefaultBufSize+100 {
 		t.Fatalf("output size is too long")
 	}
 }
@@ -178,11 +204,14 @@ func TestCheckMonitor_LimitOutput(t *testing.T) {
 func TestCheckTTL(t *testing.T) {
 	// t.Parallel() // timing test. no parallel
 	notif := mock.NewNotify()
+	logger := testutil.Logger(t)
+	cid := structs.NewCheckID("foo", nil)
+
 	check := &CheckTTL{
 		Notify:  notif,
-		CheckID: types.CheckID("foo"),
+		CheckID: cid,
 		TTL:     200 * time.Millisecond,
-		Logger:  log.New(ioutil.Discard, uniqueID(), log.LstdFlags),
+		Logger:  logger,
 	}
 	check.Start()
 	defer check.Stop()
@@ -190,32 +219,32 @@ func TestCheckTTL(t *testing.T) {
 	time.Sleep(100 * time.Millisecond)
 	check.SetStatus(api.HealthPassing, "test-output")
 
-	if notif.Updates("foo") != 1 {
+	if notif.Updates(cid) != 1 {
 		t.Fatalf("should have 1 updates %v", notif.UpdatesMap())
 	}
 
-	if notif.State("foo") != api.HealthPassing {
+	if notif.State(cid) != api.HealthPassing {
 		t.Fatalf("should be passing %v", notif.StateMap())
 	}
 
 	// Ensure we don't fail early
 	time.Sleep(150 * time.Millisecond)
-	if notif.Updates("foo") != 1 {
+	if notif.Updates(cid) != 1 {
 		t.Fatalf("should have 1 updates %v", notif.UpdatesMap())
 	}
 
 	// Wait for the TTL to expire
 	time.Sleep(150 * time.Millisecond)
 
-	if notif.Updates("foo") != 2 {
+	if notif.Updates(cid) != 2 {
 		t.Fatalf("should have 2 updates %v", notif.UpdatesMap())
 	}
 
-	if notif.State("foo") != api.HealthCritical {
+	if notif.State(cid) != api.HealthCritical {
 		t.Fatalf("should be critical %v", notif.StateMap())
 	}
 
-	if !strings.Contains(notif.Output("foo"), "test-output") {
+	if !strings.Contains(notif.Output(cid), "test-output") {
 		t.Fatalf("should have retained output %v", notif.OutputMap())
 	}
 }
@@ -299,28 +328,32 @@ func TestCheckHTTP(t *testing.T) {
 			defer server.Close()
 
 			notif := mock.NewNotify()
+			logger := testutil.Logger(t)
+			statusHandler := NewStatusHandler(notif, logger, 0, 0)
+
+			cid := structs.NewCheckID("foo", nil)
+
 			check := &CheckHTTP{
-				Notify:        notif,
-				CheckID:       types.CheckID("foo"),
+				CheckID:       cid,
 				HTTP:          server.URL,
 				Method:        tt.method,
-				OutputMaxSize: DefaultBufSize,
 				Header:        tt.header,
 				Interval:      10 * time.Millisecond,
-				Logger:        log.New(ioutil.Discard, uniqueID(), log.LstdFlags),
+				Logger:        logger,
+				StatusHandler: statusHandler,
 			}
 			check.Start()
 			defer check.Stop()
 
 			retry.Run(t, func(r *retry.R) {
-				if got, want := notif.Updates("foo"), 2; got < want {
+				if got, want := notif.Updates(cid), 2; got < want {
 					r.Fatalf("got %d updates want at least %d", got, want)
 				}
-				if got, want := notif.State("foo"), tt.status; got != want {
+				if got, want := notif.State(cid), tt.status; got != want {
 					r.Fatalf("got state %q want %q", got, want)
 				}
 				// Allow slightly more data than DefaultBufSize, for the header
-				if n := len(notif.Output("foo")); n > (DefaultBufSize + 256) {
+				if n := len(notif.Output(cid)); n > (DefaultBufSize + 256) {
 					r.Fatalf("output too long: %d (%d-byte limit)", n, DefaultBufSize)
 				}
 			})
@@ -328,6 +361,156 @@ func TestCheckHTTP(t *testing.T) {
 	}
 }
 
+func TestCheckHTTP_Proxied(t *testing.T) {
+	t.Parallel()
+
+	proxy := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		fmt.Fprintln(w, "Proxy Server")
+	}))
+	defer proxy.Close()
+
+	notif := mock.NewNotify()
+
+	logger := testutil.Logger(t)
+	statusHandler := NewStatusHandler(notif, logger, 0, 0)
+	cid := structs.NewCheckID("foo", nil)
+
+	check := &CheckHTTP{
+		CheckID:       cid,
+		HTTP:          "",
+		Method:        "GET",
+		OutputMaxSize: DefaultBufSize,
+		Interval:      10 * time.Millisecond,
+		Logger:        logger,
+		ProxyHTTP:     proxy.URL,
+		StatusHandler: statusHandler,
+	}
+
+	check.Start()
+	defer check.Stop()
+
+	// If ProxyHTTP is set, check() reqs should go to that address
+	retry.Run(t, func(r *retry.R) {
+		output := notif.Output(cid)
+		if !strings.Contains(output, "Proxy Server") {
+			r.Fatalf("c.ProxyHTTP server did not receive request, but should")
+		}
+	})
+}
+
+func TestCheckHTTP_NotProxied(t *testing.T) {
+	t.Parallel()
+
+	server := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		fmt.Fprintln(w, "Original Server")
+	}))
+	defer server.Close()
+
+	notif := mock.NewNotify()
+	logger := testutil.Logger(t)
+	statusHandler := NewStatusHandler(notif, logger, 0, 0)
+	cid := structs.NewCheckID("foo", nil)
+
+	check := &CheckHTTP{
+		CheckID:       cid,
+		HTTP:          server.URL,
+		Method:        "GET",
+		OutputMaxSize: DefaultBufSize,
+		Interval:      10 * time.Millisecond,
+		Logger:        logger,
+		ProxyHTTP:     "",
+		StatusHandler: statusHandler,
+	}
+	check.Start()
+	defer check.Stop()
+
+	// If ProxyHTTP is not set, check() reqs should go to the address in CheckHTTP.HTTP
+	retry.Run(t, func(r *retry.R) {
+		output := notif.Output(cid)
+		if !strings.Contains(output, "Original Server") {
+			r.Fatalf("server did not receive request")
+		}
+	})
+}
+
+func TestCheckHTTPTCP_BigTimeout(t *testing.T) {
+	testCases := []struct {
+		timeoutIn, intervalIn, timeoutWant time.Duration
+	}{
+		{
+			timeoutIn:   31 * time.Second,
+			intervalIn:  30 * time.Second,
+			timeoutWant: 31 * time.Second,
+		},
+		{
+			timeoutIn:   30 * time.Second,
+			intervalIn:  30 * time.Second,
+			timeoutWant: 30 * time.Second,
+		},
+		{
+			timeoutIn:   29 * time.Second,
+			intervalIn:  30 * time.Second,
+			timeoutWant: 29 * time.Second,
+		},
+		{
+			timeoutIn:   0 * time.Second,
+			intervalIn:  10 * time.Second,
+			timeoutWant: 10 * time.Second,
+		},
+		{
+			timeoutIn:   0 * time.Second,
+			intervalIn:  30 * time.Second,
+			timeoutWant: 10 * time.Second,
+		},
+		{
+			timeoutIn:   10 * time.Second,
+			intervalIn:  30 * time.Second,
+			timeoutWant: 10 * time.Second,
+		},
+		{
+			timeoutIn:   9 * time.Second,
+			intervalIn:  30 * time.Second,
+			timeoutWant: 9 * time.Second,
+		},
+		{
+			timeoutIn:   -1 * time.Second,
+			intervalIn:  10 * time.Second,
+			timeoutWant: 10 * time.Second,
+		},
+		{
+			timeoutIn:   0 * time.Second,
+			intervalIn:  5 * time.Second,
+			timeoutWant: 10 * time.Second,
+		},
+	}
+
+	for _, tc := range testCases {
+		desc := fmt.Sprintf("timeoutIn: %v, intervalIn: %v", tc.timeoutIn, tc.intervalIn)
+		t.Run(desc, func(t *testing.T) {
+			checkHTTP := &CheckHTTP{
+				Timeout:  tc.timeoutIn,
+				Interval: tc.intervalIn,
+			}
+			checkHTTP.Start()
+			defer checkHTTP.Stop()
+			if checkHTTP.httpClient.Timeout != tc.timeoutWant {
+				t.Fatalf("expected HTTP timeout to be %v, got %v", tc.timeoutWant, checkHTTP.httpClient.Timeout)
+			}
+
+			checkTCP := &CheckTCP{
+				Timeout:  tc.timeoutIn,
+				Interval: tc.intervalIn,
+			}
+			checkTCP.Start()
+			defer checkTCP.Stop()
+			if checkTCP.dialer.Timeout != tc.timeoutWant {
+				t.Fatalf("expected TCP timeout to be %v, got %v", tc.timeoutWant, checkTCP.dialer.Timeout)
+			}
+		})
+
+	}
+}
+
 func TestCheckMaxOutputSize(t *testing.T) {
 	t.Parallel()
 	timeout := 5 * time.Millisecond
@@ -339,27 +522,30 @@ func TestCheckMaxOutputSize(t *testing.T) {
 	defer server.Close()
 
 	notif := mock.NewNotify()
+	logger := testutil.Logger(t)
 	maxOutputSize := 32
+	cid := structs.NewCheckID("bar", nil)
+
 	check := &CheckHTTP{
-		Notify:        notif,
-		CheckID:       types.CheckID("bar"),
+		CheckID:       cid,
 		HTTP:          server.URL + "/v1/agent/self",
 		Timeout:       timeout,
 		Interval:      2 * time.Millisecond,
-		Logger:        log.New(ioutil.Discard, uniqueID(), log.LstdFlags),
+		Logger:        logger,
 		OutputMaxSize: maxOutputSize,
+		StatusHandler: NewStatusHandler(notif, logger, 0, 0),
 	}
 
 	check.Start()
 	defer check.Stop()
 	retry.Run(t, func(r *retry.R) {
-		if got, want := notif.Updates("bar"), 2; got < want {
+		if got, want := notif.Updates(cid), 2; got < want {
 			r.Fatalf("got %d updates want at least %d", got, want)
 		}
-		if got, want := notif.State("bar"), api.HealthPassing; got != want {
+		if got, want := notif.State(cid), api.HealthPassing; got != want {
 			r.Fatalf("got state %q want %q", got, want)
 		}
-		if got, want := notif.Output("bar"), "HTTP GET "+server.URL+"/v1/agent/self: 200 OK Output: "+strings.Repeat("x", maxOutputSize); got != want {
+		if got, want := notif.Output(cid), "HTTP GET "+server.URL+"/v1/agent/self: 200 OK Output: "+strings.Repeat("x", maxOutputSize); got != want {
 			r.Fatalf("got state %q want %q", got, want)
 		}
 	})
@@ -374,34 +560,112 @@ func TestCheckHTTPTimeout(t *testing.T) {
 	defer server.Close()
 
 	notif := mock.NewNotify()
+	logger := testutil.Logger(t)
+	statusHandler := NewStatusHandler(notif, logger, 0, 0)
+
+	cid := structs.NewCheckID("bar", nil)
+
 	check := &CheckHTTP{
-		Notify:   notif,
-		CheckID:  types.CheckID("bar"),
-		HTTP:     server.URL,
-		Timeout:  timeout,
-		Interval: 10 * time.Millisecond,
-		Logger:   log.New(ioutil.Discard, uniqueID(), log.LstdFlags),
+		CheckID:       cid,
+		HTTP:          server.URL,
+		Timeout:       timeout,
+		Interval:      10 * time.Millisecond,
+		Logger:        logger,
+		StatusHandler: statusHandler,
 	}
 
 	check.Start()
 	defer check.Stop()
 	retry.Run(t, func(r *retry.R) {
-		if got, want := notif.Updates("bar"), 2; got < want {
+		if got, want := notif.Updates(cid), 2; got < want {
 			r.Fatalf("got %d updates want at least %d", got, want)
 		}
-		if got, want := notif.State("bar"), api.HealthCritical; got != want {
+		if got, want := notif.State(cid), api.HealthCritical; got != want {
 			r.Fatalf("got state %q want %q", got, want)
 		}
 	})
 }
 
+func TestCheckHTTPBody(t *testing.T) {
+	t.Parallel()
+	timeout := 5 * time.Millisecond
+
+	server := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		var (
+			buf  bytes.Buffer
+			body []byte
+		)
+		code := 200
+		if _, err := buf.ReadFrom(r.Body); err != nil {
+			code = 999
+			body = []byte(err.Error())
+		} else {
+			body = buf.Bytes()
+		}
+
+		w.WriteHeader(code)
+		w.Write(body)
+	}))
+	defer server.Close()
+
+	tests := []struct {
+		desc   string
+		method string
+		header http.Header
+		body   string
+	}{
+		{desc: "get body", method: "GET", body: "hello world"},
+		{desc: "post body", method: "POST", body: "hello world"},
+		{desc: "post json body", header: http.Header{"Content-Type": []string{"application/json"}}, method: "POST", body: "{\"foo\":\"bar\"}"},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.desc, func(t *testing.T) {
+			notif := mock.NewNotify()
+
+			cid := structs.NewCheckID("checkbody", nil)
+			logger := testutil.Logger(t)
+			check := &CheckHTTP{
+				CheckID:       cid,
+				HTTP:          server.URL,
+				Header:        tt.header,
+				Method:        tt.method,
+				Body:          tt.body,
+				Timeout:       timeout,
+				Interval:      2 * time.Millisecond,
+				Logger:        logger,
+				StatusHandler: NewStatusHandler(notif, logger, 0, 0),
+			}
+			check.Start()
+			defer check.Stop()
+
+			retry.Run(t, func(r *retry.R) {
+				if got, want := notif.Updates(cid), 2; got < want {
+					r.Fatalf("got %d updates want at least %d", got, want)
+				}
+				if got, want := notif.State(cid), api.HealthPassing; got != want {
+					r.Fatalf("got status %q want %q", got, want)
+				}
+				if got, want := notif.Output(cid), tt.body; !strings.HasSuffix(got, want) {
+					r.Fatalf("got output %q want suffix %q", got, want)
+				}
+			})
+		})
+	}
+}
+
 func TestCheckHTTP_disablesKeepAlives(t *testing.T) {
 	t.Parallel()
+	notif := mock.NewNotify()
+	logger := testutil.Logger(t)
+	cid := structs.NewCheckID("foo", nil)
+
 	check := &CheckHTTP{
-		CheckID:  types.CheckID("foo"),
-		HTTP:     "http://foo.bar/baz",
-		Interval: 10 * time.Second,
-		Logger:   log.New(ioutil.Discard, uniqueID(), log.LstdFlags),
+		CheckID:       cid,
+		HTTP:          "http://foo.bar/baz",
+		Interval:      10 * time.Second,
+		Logger:        logger,
+		StatusHandler: NewStatusHandler(notif, logger, 0, 0),
 	}
 
 	check.Start()
@@ -435,13 +699,17 @@ func TestCheckHTTP_TLS_SkipVerify(t *testing.T) {
 	}
 
 	notif := mock.NewNotify()
+	logger := testutil.Logger(t)
+	statusHandler := NewStatusHandler(notif, logger, 0, 0)
+
+	cid := structs.NewCheckID("skipverify_true", nil)
 	check := &CheckHTTP{
-		Notify:          notif,
-		CheckID:         types.CheckID("skipverify_true"),
+		CheckID:         cid,
 		HTTP:            server.URL,
 		Interval:        25 * time.Millisecond,
-		Logger:          log.New(ioutil.Discard, uniqueID(), log.LstdFlags),
+		Logger:          logger,
 		TLSClientConfig: tlsClientConfig,
+		StatusHandler:   statusHandler,
 	}
 
 	check.Start()
@@ -452,7 +720,7 @@ func TestCheckHTTP_TLS_SkipVerify(t *testing.T) {
 	}
 
 	retry.Run(t, func(r *retry.R) {
-		if got, want := notif.State("skipverify_true"), api.HealthPassing; got != want {
+		if got, want := notif.State(cid), api.HealthPassing; got != want {
 			r.Fatalf("got state %q want %q", got, want)
 		}
 	})
@@ -469,13 +737,17 @@ func TestCheckHTTP_TLS_BadVerify(t *testing.T) {
 	}
 
 	notif := mock.NewNotify()
+	logger := testutil.Logger(t)
+	statusHandler := NewStatusHandler(notif, logger, 0, 0)
+	cid := structs.NewCheckID("skipverify_false", nil)
+
 	check := &CheckHTTP{
-		Notify:          notif,
-		CheckID:         types.CheckID("skipverify_false"),
+		CheckID:         cid,
 		HTTP:            server.URL,
 		Interval:        100 * time.Millisecond,
-		Logger:          log.New(ioutil.Discard, uniqueID(), log.LstdFlags),
+		Logger:          logger,
 		TLSClientConfig: tlsClientConfig,
+		StatusHandler:   statusHandler,
 	}
 
 	check.Start()
@@ -487,10 +759,10 @@ func TestCheckHTTP_TLS_BadVerify(t *testing.T) {
 
 	retry.Run(t, func(r *retry.R) {
 		// This should fail due to an invalid SSL cert
-		if got, want := notif.State("skipverify_false"), api.HealthCritical; got != want {
+		if got, want := notif.State(cid), api.HealthCritical; got != want {
 			r.Fatalf("got state %q want %q", got, want)
 		}
-		if !strings.Contains(notif.Output("skipverify_false"), "certificate signed by unknown authority") {
+		if !strings.Contains(notif.Output(cid), "certificate signed by unknown authority") {
 			r.Fatalf("should fail with certificate error %v", notif.OutputMap())
 		}
 	})
@@ -517,25 +789,121 @@ func mockTCPServer(network string) net.Listener {
 
 func expectTCPStatus(t *testing.T, tcp string, status string) {
 	notif := mock.NewNotify()
+	logger := testutil.Logger(t)
+	statusHandler := NewStatusHandler(notif, logger, 0, 0)
+	cid := structs.NewCheckID("foo", nil)
+
 	check := &CheckTCP{
-		Notify:   notif,
-		CheckID:  types.CheckID("foo"),
-		TCP:      tcp,
-		Interval: 10 * time.Millisecond,
-		Logger:   log.New(ioutil.Discard, uniqueID(), log.LstdFlags),
+		CheckID:       cid,
+		TCP:           tcp,
+		Interval:      10 * time.Millisecond,
+		Logger:        logger,
+		StatusHandler: statusHandler,
 	}
 	check.Start()
 	defer check.Stop()
 	retry.Run(t, func(r *retry.R) {
-		if got, want := notif.Updates("foo"), 2; got < want {
+		if got, want := notif.Updates(cid), 2; got < want {
 			r.Fatalf("got %d updates want at least %d", got, want)
 		}
-		if got, want := notif.State("foo"), status; got != want {
+		if got, want := notif.State(cid), status; got != want {
 			r.Fatalf("got state %q want %q", got, want)
 		}
 	})
 }
 
+func TestStatusHandlerUpdateStatusAfterConsecutiveChecksThresholdIsReached(t *testing.T) {
+	t.Parallel()
+	cid := structs.NewCheckID("foo", nil)
+	notif := mock.NewNotify()
+	logger := testutil.Logger(t)
+	statusHandler := NewStatusHandler(notif, logger, 2, 3)
+
+	// Set the initial status to passing after a single success
+	statusHandler.updateCheck(cid, api.HealthPassing, "bar")
+
+	// Status should become critical after 3 failed checks only
+	statusHandler.updateCheck(cid, api.HealthCritical, "bar")
+	statusHandler.updateCheck(cid, api.HealthCritical, "bar")
+
+	retry.Run(t, func(r *retry.R) {
+		require.Equal(r, 1, notif.Updates(cid))
+		require.Equal(r, api.HealthPassing, notif.State(cid))
+	})
+
+	statusHandler.updateCheck(cid, api.HealthCritical, "bar")
+
+	retry.Run(t, func(r *retry.R) {
+		require.Equal(r, 2, notif.Updates(cid))
+		require.Equal(r, api.HealthCritical, notif.State(cid))
+	})
+
+	// Status should be passing after 2 passing check
+	statusHandler.updateCheck(cid, api.HealthPassing, "bar")
+
+	retry.Run(t, func(r *retry.R) {
+		require.Equal(r, 2, notif.Updates(cid))
+		require.Equal(r, api.HealthCritical, notif.State(cid))
+	})
+
+	statusHandler.updateCheck(cid, api.HealthPassing, "bar")
+
+	retry.Run(t, func(r *retry.R) {
+		require.Equal(r, 3, notif.Updates(cid))
+		require.Equal(r, api.HealthPassing, notif.State(cid))
+	})
+}
+
+func TestStatusHandlerResetCountersOnNonIdenticalsConsecutiveChecks(t *testing.T) {
+	t.Parallel()
+	cid := structs.NewCheckID("foo", nil)
+	notif := mock.NewNotify()
+	logger := testutil.Logger(t)
+	statusHandler := NewStatusHandler(notif, logger, 2, 3)
+
+	// Set the initial status to passing after a single success
+	statusHandler.updateCheck(cid, api.HealthPassing, "bar")
+
+	// Status should remain passing after FAIL PASS FAIL FAIL sequence
+	// Although we have 3 FAILS, they are not consecutive
+
+	statusHandler.updateCheck(cid, api.HealthCritical, "bar")
+	statusHandler.updateCheck(cid, api.HealthPassing, "bar")
+	statusHandler.updateCheck(cid, api.HealthCritical, "bar")
+	statusHandler.updateCheck(cid, api.HealthCritical, "bar")
+
+	retry.Run(t, func(r *retry.R) {
+		require.Equal(r, 1, notif.Updates(cid))
+		require.Equal(r, api.HealthPassing, notif.State(cid))
+	})
+
+	// Critical after a 3rd consecutive FAIL
+	statusHandler.updateCheck(cid, api.HealthCritical, "bar")
+
+	retry.Run(t, func(r *retry.R) {
+		require.Equal(r, 2, notif.Updates(cid))
+		require.Equal(r, api.HealthCritical, notif.State(cid))
+	})
+
+	// Status should remain critical after PASS FAIL PASS sequence
+	statusHandler.updateCheck(cid, api.HealthPassing, "bar")
+	statusHandler.updateCheck(cid, api.HealthCritical, "bar")
+	statusHandler.updateCheck(cid, api.HealthPassing, "bar")
+
+	retry.Run(t, func(r *retry.R) {
+		require.Equal(r, 2, notif.Updates(cid))
+		require.Equal(r, api.HealthCritical, notif.State(cid))
+	})
+
+	// Passing after a 2nd consecutive PASS
+	statusHandler.updateCheck(cid, api.HealthPassing, "bar")
+
+	retry.Run(t, func(r *retry.R) {
+		require.Equal(r, 3, notif.Updates(cid))
+		require.Equal(r, api.HealthPassing, notif.State(cid))
+	})
+}
+
 func TestCheckTCPCritical(t *testing.T) {
 	t.Parallel()
 	var (
@@ -830,14 +1198,17 @@ func TestCheck_Docker(t *testing.T) {
 			}
 
 			notif, upd := mock.NewNotifyChan()
-			id := types.CheckID("chk")
+			logger := testutil.Logger(t)
+			statusHandler := NewStatusHandler(notif, logger, 0, 0)
+			id := structs.NewCheckID("chk", nil)
+
 			check := &CheckDocker{
-				Notify:            notif,
 				CheckID:           id,
 				ScriptArgs:        []string{"/health.sh"},
 				DockerContainerID: "123",
 				Interval:          25 * time.Millisecond,
 				Client:            c,
+				StatusHandler:     statusHandler,
 			}
 			check.Start()
 			defer check.Stop()
