@@ -22,6 +22,9 @@ func TestSegments(t *testing.T) {
 			json: []string{`{ "server": true, "segment": "a" }`},
 			hcl:  []string{` server = true segment = "a" `},
 			err:  `Network segments are not supported in this version of Consul`,
+			warns: []string{
+				enterpriseConfigKeyError{key: "segment"}.Error(),
+			},
 		},
 		{
 			desc: "segment port must be set",
@@ -31,6 +34,9 @@ func TestSegments(t *testing.T) {
 			json: []string{`{ "segments":[{ "name":"x" }] }`},
 			hcl:  []string{`segments = [{ name = "x" }]`},
 			err:  `Port for segment "x" cannot be <= 0`,
+			warns: []string{
+				enterpriseConfigKeyError{key: "segments"}.Error(),
+			},
 		},
 		{
 			desc: "segments not in OSS",
@@ -40,6 +46,9 @@ func TestSegments(t *testing.T) {
 			json: []string{`{ "segments":[{ "name":"x", "port": 123 }] }`},
 			hcl:  []string{`segments = [{ name = "x" port = 123 }]`},
 			err:  `Network segments are not supported in this version of Consul`,
+			warns: []string{
+				enterpriseConfigKeyError{key: "segments"}.Error(),
+			},
 		},
 	}
 
