@@ -16,7 +16,7 @@ import (
 func TestConfig_Get(t *testing.T) {
 	t.Parallel()
 
-	a := NewTestAgent(t, t.Name(), "")
+	a := NewTestAgent(t, "")
 	defer a.Shutdown()
 	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
 
@@ -97,7 +97,7 @@ func TestConfig_Delete(t *testing.T) {
 	t.Parallel()
 
 	require := require.New(t)
-	a := NewTestAgent(t, t.Name(), "")
+	a := NewTestAgent(t, "")
 	defer a.Shutdown()
 	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
 
@@ -147,7 +147,7 @@ func TestConfig_Apply(t *testing.T) {
 	t.Parallel()
 
 	require := require.New(t)
-	a := NewTestAgent(t, t.Name(), "")
+	a := NewTestAgent(t, "")
 	defer a.Shutdown()
 	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
 
@@ -182,10 +182,135 @@ func TestConfig_Apply(t *testing.T) {
 	}
 }
 
+func TestConfig_Apply_TerminatingGateway(t *testing.T) {
+	t.Parallel()
+
+	a := NewTestAgent(t, "")
+	defer a.Shutdown()
+	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
+
+	// Create some config entries.
+	body := bytes.NewBuffer([]byte(`
+	{
+		"Kind": "terminating-gateway",
+		"Name": "west-gw-01",
+		"Services": [
+		  {
+			"Name": "web",
+			"CAFile": "/etc/web/ca.crt",
+			"CertFile": "/etc/web/client.crt",
+			"KeyFile": "/etc/web/tls.key"
+		  },
+		  {
+			"Name": "api"
+		  }
+		]
+	}`))
+
+	req, _ := http.NewRequest("PUT", "/v1/config", body)
+	resp := httptest.NewRecorder()
+	_, err := a.srv.ConfigApply(resp, req)
+	require.NoError(t, err)
+	require.Equal(t, 200, resp.Code, "!200 Response Code: %s", resp.Body.String())
+
+	// List all entries, there should only be one
+	{
+		args := structs.ConfigEntryQuery{
+			Kind:       structs.TerminatingGateway,
+			Datacenter: "dc1",
+		}
+		var out structs.IndexedConfigEntries
+		require.NoError(t, a.RPC("ConfigEntry.List", &args, &out))
+		require.NotNil(t, out)
+		require.Len(t, out.Entries, 1)
+
+		got := out.Entries[0].(*structs.TerminatingGatewayConfigEntry)
+		expect := []structs.LinkedService{
+			{
+				Name:           "web",
+				CAFile:         "/etc/web/ca.crt",
+				CertFile:       "/etc/web/client.crt",
+				KeyFile:        "/etc/web/tls.key",
+				EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
+			},
+			{
+				Name:           "api",
+				EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
+			},
+		}
+		require.Equal(t, expect, got.Services)
+	}
+}
+
+func TestConfig_Apply_IngressGateway(t *testing.T) {
+	t.Parallel()
+
+	a := NewTestAgent(t, "")
+	defer a.Shutdown()
+	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
+
+	// Create some config entries.
+	body := bytes.NewBuffer([]byte(`
+	{
+		"Kind": "ingress-gateway",
+		"Name": "ingress",
+		"Listeners": [
+		  {
+				"Port": 8080,
+				"Services": [
+					{ "Name": "web" }
+				]
+		  }
+		]
+	}`))
+
+	req, _ := http.NewRequest("PUT", "/v1/config", body)
+	resp := httptest.NewRecorder()
+	_, err := a.srv.ConfigApply(resp, req)
+	require.NoError(t, err)
+	require.Equal(t, 200, resp.Code, "!200 Response Code: %s", resp.Body.String())
+
+	// List all entries, there should only be one
+	{
+		args := structs.ConfigEntryQuery{
+			Kind:       structs.IngressGateway,
+			Datacenter: "dc1",
+		}
+		var out structs.IndexedConfigEntries
+		require.NoError(t, a.RPC("ConfigEntry.List", &args, &out))
+		require.NotNil(t, out)
+		require.Len(t, out.Entries, 1)
+
+		got := out.Entries[0].(*structs.IngressGatewayConfigEntry)
+		// Ignore create and modify indices
+		got.CreateIndex = 0
+		got.ModifyIndex = 0
+
+		expect := &structs.IngressGatewayConfigEntry{
+			Name: "ingress",
+			Kind: structs.IngressGateway,
+			Listeners: []structs.IngressListener{
+				{
+					Port:     8080,
+					Protocol: "tcp",
+					Services: []structs.IngressService{
+						{
+							Name:           "web",
+							EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
+						},
+					},
+				},
+			},
+			EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
+		}
+		require.Equal(t, expect, got)
+	}
+}
+
 func TestConfig_Apply_ProxyDefaultsMeshGateway(t *testing.T) {
 	t.Parallel()
 
-	a := NewTestAgent(t, t.Name(), "")
+	a := NewTestAgent(t, "")
 	defer a.Shutdown()
 	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
 
@@ -224,7 +349,7 @@ func TestConfig_Apply_CAS(t *testing.T) {
 	t.Parallel()
 
 	require := require.New(t)
-	a := NewTestAgent(t, t.Name(), "")
+	a := NewTestAgent(t, "")
 	defer a.Shutdown()
 	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
 
@@ -305,7 +430,7 @@ func TestConfig_Apply_CAS(t *testing.T) {
 func TestConfig_Apply_Decoding(t *testing.T) {
 	t.Parallel()
 
-	a := NewTestAgent(t, t.Name(), "")
+	a := NewTestAgent(t, "")
 	defer a.Shutdown()
 	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
 
@@ -377,7 +502,7 @@ func TestConfig_Apply_Decoding(t *testing.T) {
 func TestConfig_Apply_ProxyDefaultsExpose(t *testing.T) {
 	t.Parallel()
 
-	a := NewTestAgent(t, t.Name(), "")
+	a := NewTestAgent(t, "")
 	defer a.Shutdown()
 	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
 
