@@ -12,10 +12,16 @@ func TestSpiffeIDService(t testing.T, service string) *SpiffeIDService {
 // TestSpiffeIDServiceWithHost returns a SPIFFE ID representing a service with
 // the specified trust domain.
 func TestSpiffeIDServiceWithHost(t testing.T, service, host string) *SpiffeIDService {
+	return TestSpiffeIDServiceWithHostDC(t, service, host, "dc1")
+}
+
+// TestSpiffeIDServiceWithHostDC returns a SPIFFE ID representing a service with
+// the specified trust domain for the given datacenter.
+func TestSpiffeIDServiceWithHostDC(t testing.T, service, host, datacenter string) *SpiffeIDService {
 	return &SpiffeIDService{
 		Host:       host,
 		Namespace:  "default",
-		Datacenter: "dc1",
+		Datacenter: datacenter,
 		Service:    service,
 	}
 }
