@@ -99,10 +99,18 @@ func (id *missingIdentity) ServiceIdentityList() []*structs.ACLServiceIdentity {
 	return nil
 }
 
+func (id *missingIdentity) NodeIdentityList() []*structs.ACLNodeIdentity {
+	return nil
+}
+
 func (id *missingIdentity) IsExpired(asOf time.Time) bool {
 	return false
 }
 
+func (id *missingIdentity) IsLocal() bool {
+	return false
+}
+
 func (id *missingIdentity) EnterpriseMetadata() *structs.EnterpriseMeta {
 	return structs.DefaultEnterpriseMeta()
 }
@@ -644,8 +652,9 @@ func (r *ACLResolver) resolvePoliciesForIdentity(identity structs.ACLIdentity) (
 	policyIDs := identity.PolicyIDs()
 	roleIDs := identity.RoleIDs()
 	serviceIdentities := identity.ServiceIdentityList()
+	nodeIdentities := identity.NodeIdentityList()
 
-	if len(policyIDs) == 0 && len(serviceIdentities) == 0 && len(roleIDs) == 0 {
+	if len(policyIDs) == 0 && len(serviceIdentities) == 0 && len(roleIDs) == 0 && len(nodeIdentities) == 0 {
 		policy := identity.EmbeddedPolicy()
 		if policy != nil {
 			return []*structs.ACLPolicy{policy}, nil
@@ -667,14 +676,17 @@ func (r *ACLResolver) resolvePoliciesForIdentity(identity structs.ACLIdentity) (
 			policyIDs = append(policyIDs, link.ID)
 		}
 		serviceIdentities = append(serviceIdentities, role.ServiceIdentities...)
+		nodeIdentities = append(nodeIdentities, role.NodeIdentityList()...)
 	}
 
 	// Now deduplicate any policies or service identities that occur more than once.
 	policyIDs = dedupeStringSlice(policyIDs)
 	serviceIdentities = dedupeServiceIdentities(serviceIdentities)
+	nodeIdentities = dedupeNodeIdentities(nodeIdentities)
 
 	// Generate synthetic policies for all service identities in effect.
 	syntheticPolicies := r.synthesizePoliciesForServiceIdentities(serviceIdentities, identity.EnterpriseMetadata())
+	syntheticPolicies = append(syntheticPolicies, r.synthesizePoliciesForNodeIdentities(nodeIdentities)...)
 
 	// For the new ACLs policy replication is mandatory for correct operation on servers. Therefore
 	// we only attempt to resolve policies locally
@@ -701,6 +713,19 @@ func (r *ACLResolver) synthesizePoliciesForServiceIdentities(serviceIdentities [
 	return syntheticPolicies
 }
 
+func (r *ACLResolver) synthesizePoliciesForNodeIdentities(nodeIdentities []*structs.ACLNodeIdentity) []*structs.ACLPolicy {
+	if len(nodeIdentities) == 0 {
+		return nil
+	}
+
+	syntheticPolicies := make([]*structs.ACLPolicy, 0, len(nodeIdentities))
+	for _, n := range nodeIdentities {
+		syntheticPolicies = append(syntheticPolicies, n.SyntheticPolicy())
+	}
+
+	return syntheticPolicies
+}
+
 func dedupeServiceIdentities(in []*structs.ACLServiceIdentity) []*structs.ACLServiceIdentity {
 	// From: https://github.com/golang/go/wiki/SliceTricks#in-place-deduplicate-comparable
 
@@ -735,6 +760,38 @@ func dedupeServiceIdentities(in []*structs.ACLServiceIdentity) []*structs.ACLSer
 	return in[:j+1]
 }
 
+func dedupeNodeIdentities(in []*structs.ACLNodeIdentity) []*structs.ACLNodeIdentity {
+	// From: https://github.com/golang/go/wiki/SliceTricks#in-place-deduplicate-comparable
+
+	if len(in) <= 1 {
+		return in
+	}
+
+	sort.Slice(in, func(i, j int) bool {
+		if in[i].NodeName < in[j].NodeName {
+			return true
+		}
+
+		return in[i].Datacenter < in[j].Datacenter
+	})
+
+	j := 0
+	for i := 1; i < len(in); i++ {
+		if in[j].NodeName == in[i].NodeName && in[j].Datacenter == in[i].Datacenter {
+			continue
+		}
+		j++
+		in[j] = in[i]
+	}
+
+	// Discard the skipped items.
+	for i := j + 1; i < len(in); i++ {
+		in[i] = nil
+	}
+
+	return in[:j+1]
+}
+
 func mergeStringSlice(a, b []string) []string {
 	out := make([]string, 0, len(a)+len(b))
 	out = append(out, a...)
@@ -1113,6 +1170,30 @@ func (r *ACLResolver) ResolveToken(token string) (acl.Authorizer, error) {
 	return authz, err
 }
 
+func (r *ACLResolver) ResolveTokenToIdentity(token string) (structs.ACLIdentity, error) {
+	if !r.ACLsEnabled() {
+		return nil, nil
+	}
+
+	if acl.RootAuthorizer(token) != nil {
+		return nil, acl.ErrRootDenied
+	}
+
+	// handle the anonymous token
+	if token == "" {
+		token = anonymousToken
+	}
+
+	if r.delegate.UseLegacyACLs() {
+		identity, _, err := r.resolveTokenLegacy(token)
+		return identity, r.disableACLsWhenUpstreamDisabled(err)
+	}
+
+	defer metrics.MeasureSince([]string{"acl", "ResolveTokenToIdentity"}, time.Now())
+
+	return r.resolveIdentityFromToken(token)
+}
+
 func (r *ACLResolver) ACLsEnabled() bool {
 	// Whether we desire ACLs to be enabled according to configuration
 	if !r.delegate.ACLsEnabled() {
@@ -1145,30 +1226,41 @@ func (r *ACLResolver) GetMergedPolicyForToken(token string) (structs.ACLIdentity
 // aclFilter is used to filter results from our state store based on ACL rules
 // configured for the provided token.
 type aclFilter struct {
-	authorizer      acl.Authorizer
-	logger          hclog.Logger
-	enforceVersion8 bool
+	authorizer acl.Authorizer
+	logger     hclog.Logger
 }
 
 // newACLFilter constructs a new aclFilter.
-func newACLFilter(authorizer acl.Authorizer, logger hclog.Logger, enforceVersion8 bool) *aclFilter {
+func newACLFilter(authorizer acl.Authorizer, logger hclog.Logger) *aclFilter {
 	if logger == nil {
 		logger = hclog.New(&hclog.LoggerOptions{})
 	}
 	return &aclFilter{
-		authorizer:      authorizer,
-		logger:          logger,
-		enforceVersion8: enforceVersion8,
+		authorizer: authorizer,
+		logger:     logger,
 	}
 }
 
 // allowNode is used to determine if a node is accessible for an ACL.
 func (f *aclFilter) allowNode(node string, ent *acl.AuthorizerContext) bool {
-	if !f.enforceVersion8 {
-		return true
+	return f.authorizer.NodeRead(node, ent) == acl.Allow
+}
+
+// allowNode is used to determine if the gateway and service are accessible for an ACL
+func (f *aclFilter) allowGateway(gs *structs.GatewayService) bool {
+	var authzContext acl.AuthorizerContext
+
+	// Need read on service and gateway. Gateway may have different EnterpriseMeta so we fill authzContext twice
+	gs.Gateway.FillAuthzContext(&authzContext)
+	if !f.allowService(gs.Gateway.Name, &authzContext) {
+		return false
 	}
 
-	return f.authorizer.NodeRead(node, ent) == acl.Allow
+	gs.Service.FillAuthzContext(&authzContext)
+	if !f.allowService(gs.Service.Name, &authzContext) {
+		return false
+	}
+	return true
 }
 
 // allowService is used to determine if a service is accessible for an ACL.
@@ -1177,18 +1269,12 @@ func (f *aclFilter) allowService(service string, ent *acl.AuthorizerContext) boo
 		return true
 	}
 
-	if !f.enforceVersion8 && service == structs.ConsulServiceID {
-		return true
-	}
 	return f.authorizer.ServiceRead(service, ent) == acl.Allow
 }
 
 // allowSession is used to determine if a session for a node is accessible for
 // an ACL.
 func (f *aclFilter) allowSession(node string, ent *acl.AuthorizerContext) bool {
-	if !f.enforceVersion8 {
-		return true
-	}
 	return f.authorizer.SessionRead(node, ent) == acl.Allow
 }
 
@@ -1324,6 +1410,20 @@ func (f *aclFilter) filterCheckServiceNodes(nodes *structs.CheckServiceNodes) {
 	*nodes = csn
 }
 
+// filterDatacenterCheckServiceNodes is used to filter nodes based on ACL rules.
+func (f *aclFilter) filterDatacenterCheckServiceNodes(datacenterNodes *map[string]structs.CheckServiceNodes) {
+	dn := *datacenterNodes
+	out := make(map[string]structs.CheckServiceNodes)
+	for dc, _ := range dn {
+		nodes := dn[dc]
+		f.filterCheckServiceNodes(&nodes)
+		if len(nodes) > 0 {
+			out[dc] = nodes
+		}
+	}
+	*datacenterNodes = out
+}
+
 // filterSessions is used to filter a set of sessions based on ACLs.
 func (f *aclFilter) filterSessions(sessions *structs.Sessions) {
 	s := *sessions
@@ -1424,6 +1524,33 @@ func (f *aclFilter) filterNodeDump(dump *structs.NodeDump) {
 	*dump = nd
 }
 
+// filterServiceDump is used to filter nodes based on ACL rules.
+func (f *aclFilter) filterServiceDump(services *structs.ServiceDump) {
+	svcs := *services
+	var authzContext acl.AuthorizerContext
+
+	for i := 0; i < len(svcs); i++ {
+		service := svcs[i]
+
+		if f.allowGateway(service.GatewayService) {
+			// ServiceDump might only have gateway config and no node information
+			if service.Node == nil {
+				continue
+			}
+
+			service.Service.FillAuthzContext(&authzContext)
+			if f.allowNode(service.Node.Node, &authzContext) {
+				continue
+			}
+		}
+
+		f.logger.Debug("dropping service from result due to ACLs", "service", service.GatewayService.Service)
+		svcs = append(svcs[:i], svcs[i+1:]...)
+		i--
+	}
+	*services = svcs
+}
+
 // filterNodes is used to filter through all parts of a node list and remove
 // elements the provided ACL token cannot access.
 func (f *aclFilter) filterNodes(nodes *structs.Nodes) {
@@ -1688,12 +1815,30 @@ func (f *aclFilter) filterServiceList(services *structs.ServiceList) {
 	*services = ret
 }
 
+// filterGatewayServices is used to filter gateway to service mappings based on ACL rules.
+func (f *aclFilter) filterGatewayServices(mappings *structs.GatewayServices) {
+	ret := make(structs.GatewayServices, 0, len(*mappings))
+	for _, s := range *mappings {
+		// This filter only checks ServiceRead on the linked service.
+		// ServiceRead on the gateway is checked in the GatewayServices endpoint before filtering.
+		var authzContext acl.AuthorizerContext
+		s.Service.FillAuthzContext(&authzContext)
+
+		if f.authorizer.ServiceRead(s.Service.Name, &authzContext) != acl.Allow {
+			f.logger.Debug("dropping service from result due to ACLs", "service", s.Service.String())
+			continue
+		}
+		ret = append(ret, s)
+	}
+	*mappings = ret
+}
+
 func (r *ACLResolver) filterACLWithAuthorizer(authorizer acl.Authorizer, subj interface{}) error {
 	if authorizer == nil {
 		return nil
 	}
 	// Create the filter
-	filt := newACLFilter(authorizer, r.logger, r.config.ACLEnforceVersion8)
+	filt := newACLFilter(authorizer, r.logger)
 
 	switch v := subj.(type) {
 	case *structs.CheckServiceNodes:
@@ -1702,6 +1847,9 @@ func (r *ACLResolver) filterACLWithAuthorizer(authorizer acl.Authorizer, subj in
 	case *structs.IndexedCheckServiceNodes:
 		filt.filterCheckServiceNodes(&v.Nodes)
 
+	case *structs.DatacenterIndexedCheckServiceNodes:
+		filt.filterDatacenterCheckServiceNodes(&v.DatacenterNodes)
+
 	case *structs.IndexedCoordinates:
 		filt.filterCoordinates(&v.Coordinates)
 
@@ -1714,6 +1862,9 @@ func (r *ACLResolver) filterACLWithAuthorizer(authorizer acl.Authorizer, subj in
 	case *structs.IndexedNodeDump:
 		filt.filterNodeDump(&v.Dump)
 
+	case *structs.IndexedServiceDump:
+		filt.filterServiceDump(&v.Dump)
+
 	case *structs.IndexedNodes:
 		filt.filterNodes(&v.Nodes)
 
@@ -1769,6 +1920,10 @@ func (r *ACLResolver) filterACLWithAuthorizer(authorizer acl.Authorizer, subj in
 
 	case *structs.IndexedServiceList:
 		filt.filterServiceList(&v.Services)
+
+	case *structs.GatewayServices:
+		filt.filterGatewayServices(v)
+
 	default:
 		panic(fmt.Errorf("Unhandled type passed to ACL filter: %T %#v", subj, subj))
 	}
@@ -1989,7 +2144,7 @@ func vetNodeTxnOp(op *structs.TxnNodeOp, rule acl.Authorizer) error {
 	var authzContext acl.AuthorizerContext
 	op.FillAuthzContext(&authzContext)
 
-	if rule != nil && rule.NodeWrite(op.Node.Node, &authzContext) != acl.Allow {
+	if rule.NodeWrite(op.Node.Node, &authzContext) != acl.Allow {
 		return acl.ErrPermissionDenied
 	}
 
