@@ -7,8 +7,9 @@ import (
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/go-bexpr"
 
-	// register this as a builtin auth method
+	// register these as a builtin auth method
 	_ "github.com/hashicorp/consul/agent/consul/authmethod/kubeauth"
+	_ "github.com/hashicorp/consul/agent/consul/authmethod/ssoauth"
 )
 
 type authMethodValidatorEntry struct {
@@ -25,7 +26,7 @@ func (s *Server) loadAuthMethodValidator(idx uint64, method *structs.ACLAuthMeth
 		return v, nil
 	}
 
-	v, err := authmethod.NewValidator(method)
+	v, err := authmethod.NewValidator(s.logger, method)
 	if err != nil {
 		return nil, fmt.Errorf("auth method validator for %q could not be initialized: %v", method.Name, err)
 	}
@@ -35,6 +36,12 @@ func (s *Server) loadAuthMethodValidator(idx uint64, method *structs.ACLAuthMeth
 	return v, nil
 }
 
+type aclBindings struct {
+	roles             []structs.ACLTokenRoleLink
+	serviceIdentities []*structs.ACLServiceIdentity
+	nodeIdentities    []*structs.ACLNodeIdentity
+}
+
 // evaluateRoleBindings evaluates all current binding rules associated with the
 // given auth method against the verified data returned from the authentication
 // process.
@@ -42,59 +49,59 @@ func (s *Server) loadAuthMethodValidator(idx uint64, method *structs.ACLAuthMeth
 // A list of role links and service identities are returned.
 func (s *Server) evaluateRoleBindings(
 	validator authmethod.Validator,
-	verifiedFields map[string]string,
+	verifiedIdentity *authmethod.Identity,
 	methodMeta *structs.EnterpriseMeta,
 	targetMeta *structs.EnterpriseMeta,
-) ([]*structs.ACLServiceIdentity, []structs.ACLTokenRoleLink, error) {
+) (*aclBindings, error) {
 	// Only fetch rules that are relevant for this method.
 	_, rules, err := s.fsm.State().ACLBindingRuleList(nil, validator.Name(), methodMeta)
 	if err != nil {
-		return nil, nil, err
+		return nil, err
 	} else if len(rules) == 0 {
-		return nil, nil, nil
+		return nil, nil
 	}
 
-	// Convert the fields into something suitable for go-bexpr.
-	selectableVars := validator.MakeFieldMapSelectable(verifiedFields)
-
 	// Find all binding rules that match the provided fields.
 	var matchingRules []*structs.ACLBindingRule
 	for _, rule := range rules {
-		if doesBindingRuleMatch(rule, selectableVars) {
+		if doesSelectorMatch(rule.Selector, verifiedIdentity.SelectableFields) {
 			matchingRules = append(matchingRules, rule)
 		}
 	}
 	if len(matchingRules) == 0 {
-		return nil, nil, nil
+		return nil, nil
 	}
 
 	// For all matching rules compute the attributes of a token.
-	var (
-		roleLinks         []structs.ACLTokenRoleLink
-		serviceIdentities []*structs.ACLServiceIdentity
-	)
+	var bindings aclBindings
 	for _, rule := range matchingRules {
-		bindName, valid, err := computeBindingRuleBindName(rule.BindType, rule.BindName, verifiedFields)
+		bindName, valid, err := computeBindingRuleBindName(rule.BindType, rule.BindName, verifiedIdentity.ProjectedVars)
 		if err != nil {
-			return nil, nil, fmt.Errorf("cannot compute %q bind name for bind target: %v", rule.BindType, err)
+			return nil, fmt.Errorf("cannot compute %q bind name for bind target: %v", rule.BindType, err)
 		} else if !valid {
-			return nil, nil, fmt.Errorf("computed %q bind name for bind target is invalid: %q", rule.BindType, bindName)
+			return nil, fmt.Errorf("computed %q bind name for bind target is invalid: %q", rule.BindType, bindName)
 		}
 
 		switch rule.BindType {
 		case structs.BindingRuleBindTypeService:
-			serviceIdentities = append(serviceIdentities, &structs.ACLServiceIdentity{
+			bindings.serviceIdentities = append(bindings.serviceIdentities, &structs.ACLServiceIdentity{
 				ServiceName: bindName,
 			})
 
+		case structs.BindingRuleBindTypeNode:
+			bindings.nodeIdentities = append(bindings.nodeIdentities, &structs.ACLNodeIdentity{
+				NodeName:   bindName,
+				Datacenter: s.config.Datacenter,
+			})
+
 		case structs.BindingRuleBindTypeRole:
 			_, role, err := s.fsm.State().ACLRoleGetByName(nil, bindName, targetMeta)
 			if err != nil {
-				return nil, nil, err
+				return nil, err
 			}
 
 			if role != nil {
-				roleLinks = append(roleLinks, structs.ACLTokenRoleLink{
+				bindings.roles = append(bindings.roles, structs.ACLTokenRoleLink{
 					ID: role.ID,
 				})
 			}
@@ -104,17 +111,16 @@ func (s *Server) evaluateRoleBindings(
 		}
 	}
 
-	return serviceIdentities, roleLinks, nil
+	return &bindings, nil
 }
 
-// doesBindingRuleMatch checks that a single binding rule matches the provided
-// vars.
-func doesBindingRuleMatch(rule *structs.ACLBindingRule, selectableVars interface{}) bool {
-	if rule.Selector == "" {
+// doesSelectorMatch checks that a single selector matches the provided vars.
+func doesSelectorMatch(selector string, selectableVars interface{}) bool {
+	if selector == "" {
 		return true // catch-all
 	}
 
-	eval, err := bexpr.CreateEvaluatorForType(rule.Selector, nil, selectableVars)
+	eval, err := bexpr.CreateEvaluatorForType(selector, nil, selectableVars)
 	if err != nil {
 		return false // fails to match if selector is invalid
 	}
