@@ -3,15 +3,16 @@ package autopilot
 import (
 	"context"
 	"fmt"
+	"net"
+	"strconv"
+	"sync"
+	"time"
+
 	"github.com/hashicorp/consul/logging"
 	"github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/go-version"
 	"github.com/hashicorp/raft"
 	"github.com/hashicorp/serf/serf"
-	"net"
-	"strconv"
-	"sync"
-	"time"
 )
 
 // Delegate is the interface for the Autopilot mechanism
@@ -175,7 +176,7 @@ func (a *Autopilot) RemoveDeadServers() {
 }
 
 func canRemoveServers(peers, minQuorum, deadServers int) (bool, string) {
-	if peers-deadServers < int(minQuorum) {
+	if peers-deadServers < minQuorum {
 		return false, fmt.Sprintf("denied, because removing %d/%d servers would leave less then minimal allowed quorum of %d servers", deadServers, peers, minQuorum)
 	}
 
@@ -419,12 +420,6 @@ func (a *Autopilot) updateClusterHealth() error {
 	// consistent of a sample as possible. We capture the leader's index
 	// here as well so it roughly lines up with the same point in time.
 	targetLastIndex := raftNode.LastIndex()
-	var fetchList []*ServerInfo
-	for _, server := range servers {
-		if parts, ok := serverMap[string(server.ID)]; ok {
-			fetchList = append(fetchList, parts)
-		}
-	}
 	d := time.Now().Add(a.healthInterval / 2)
 	ctx, cancel := context.WithDeadline(context.Background(), d)
 	defer cancel()
