@@ -2,7 +2,6 @@ package consul
 
 import (
 	"fmt"
-	"io"
 	"net"
 	"os"
 	"time"
@@ -120,9 +119,6 @@ type Config struct {
 	// configured at this point.
 	NotifyListen func()
 
-	// NotifyShutdown is called after Server is completely Shutdown.
-	NotifyShutdown func()
-
 	// RPCAddr is the RPC address used by Consul. This should be reachable
 	// by the WAN and LAN
 	RPCAddr *net.TCPAddr
@@ -161,13 +157,6 @@ type Config struct {
 	// leader election.
 	ReconcileInterval time.Duration
 
-	// LogLevel is the level of the logs to write. Defaults to "INFO".
-	LogLevel string
-
-	// LogOutput is the location to write logs to. If this is not set,
-	// logs will go to stderr.
-	LogOutput io.Writer
-
 	// ProtocolVersion is the protocol version to speak. This must be between
 	// ProtocolVersionMin and ProtocolVersionMax.
 	ProtocolVersion uint8
@@ -237,10 +226,6 @@ type Config struct {
 	// ACLEnabled is used to enable ACLs
 	ACLsEnabled bool
 
-	// ACLEnforceVersion8 is used to gate a set of ACL policy features that
-	// are opt-in prior to Consul 0.8 and opt-out in Consul 0.8 and later.
-	ACLEnforceVersion8 bool
-
 	// ACLMasterToken is used to bootstrap the ACL system. It should be specified
 	// on the servers in the ACLDatacenter. When the leader comes online, it ensures
 	// that the Master token is available. This provides the initial token.
@@ -309,6 +294,17 @@ type Config struct {
 	// by default in Consul 1.0 and later.
 	ACLEnableKeyListPolicy bool
 
+	AutoConfigEnabled              bool
+	AutoConfigIntroToken           string
+	AutoConfigIntroTokenFile       string
+	AutoConfigServerAddresses      []string
+	AutoConfigDNSSANs              []string
+	AutoConfigIPSANs               []net.IP
+	AutoConfigAuthzEnabled         bool
+	AutoConfigAuthzAuthMethod      structs.ACLAuthMethod
+	AutoConfigAuthzClaimAssertions []string
+	AutoConfigAuthzAllowReuse      bool
+
 	// TombstoneTTL is used to control how long KV tombstones are retained.
 	// This provides a window of time where the X-Consul-Index is monotonic.
 	// Outside this window, the index may not be monotonic. This is a result
@@ -370,6 +366,20 @@ type Config struct {
 	// used to limit the amount of Raft bandwidth used for replication.
 	ConfigReplicationApplyLimit int
 
+	// FederationStateReplicationRate is the max number of replication rounds that can
+	// be run per second. Note that either 1 or 2 RPCs are used during each replication
+	// round
+	FederationStateReplicationRate int
+
+	// FederationStateReplicationBurst is how many replication rounds can be bursted after a
+	// period of idleness
+	FederationStateReplicationBurst int
+
+	// FederationStateReplicationApply limit is the max number of replication-related
+	// apply operations that we allow during a one second period. This is
+	// used to limit the amount of Raft bandwidth used for replication.
+	FederationStateReplicationApplyLimit int
+
 	// CoordinateUpdatePeriod controls how long a server batches coordinate
 	// updates before applying them in a Raft transaction. A larger period
 	// leads to fewer Raft transactions, but also the stored coordinates
@@ -436,6 +446,14 @@ type Config struct {
 	// ConnectEnabled is whether to enable Connect features such as the CA.
 	ConnectEnabled bool
 
+	// ConnectMeshGatewayWANFederationEnabled determines if wan federation of
+	// datacenters should exclusively traverse mesh gateways.
+	ConnectMeshGatewayWANFederationEnabled bool
+
+	// DisableFederationStateAntiEntropy solely exists for use in unit tests to
+	// disable a background routine.
+	DisableFederationStateAntiEntropy bool
+
 	// CAConfig is used to apply the initial Connect CA configuration when
 	// bootstrapping.
 	CAConfig *structs.CAConfiguration
@@ -501,7 +519,7 @@ func (c *Config) CheckACL() error {
 	return nil
 }
 
-// DefaultConfig returns a sane default configuration.
+// DefaultConfig returns a default configuration.
 func DefaultConfig() *Config {
 	hostname, err := os.Hostname()
 	if err != nil {
@@ -509,32 +527,35 @@ func DefaultConfig() *Config {
 	}
 
 	conf := &Config{
-		Build:                       version.Version,
-		Datacenter:                  DefaultDC,
-		NodeName:                    hostname,
-		RPCAddr:                     DefaultRPCAddr,
-		RaftConfig:                  raft.DefaultConfig(),
-		SerfLANConfig:               lib.SerfDefaultConfig(),
-		SerfWANConfig:               lib.SerfDefaultConfig(),
-		SerfFloodInterval:           60 * time.Second,
-		ReconcileInterval:           60 * time.Second,
-		ProtocolVersion:             ProtocolVersion2Compatible,
-		ACLRoleTTL:                  30 * time.Second,
-		ACLPolicyTTL:                30 * time.Second,
-		ACLTokenTTL:                 30 * time.Second,
-		ACLDefaultPolicy:            "allow",
-		ACLDownPolicy:               "extend-cache",
-		ACLReplicationRate:          1,
-		ACLReplicationBurst:         5,
-		ACLReplicationApplyLimit:    100, // ops / sec
-		ConfigReplicationRate:       1,
-		ConfigReplicationBurst:      5,
-		ConfigReplicationApplyLimit: 100, // ops / sec
-		TombstoneTTL:                15 * time.Minute,
-		TombstoneTTLGranularity:     30 * time.Second,
-		SessionTTLMin:               10 * time.Second,
-		ACLTokenMinExpirationTTL:    1 * time.Minute,
-		ACLTokenMaxExpirationTTL:    24 * time.Hour,
+		Build:                                version.Version,
+		Datacenter:                           DefaultDC,
+		NodeName:                             hostname,
+		RPCAddr:                              DefaultRPCAddr,
+		RaftConfig:                           raft.DefaultConfig(),
+		SerfLANConfig:                        lib.SerfDefaultConfig(),
+		SerfWANConfig:                        lib.SerfDefaultConfig(),
+		SerfFloodInterval:                    60 * time.Second,
+		ReconcileInterval:                    60 * time.Second,
+		ProtocolVersion:                      ProtocolVersion2Compatible,
+		ACLRoleTTL:                           30 * time.Second,
+		ACLPolicyTTL:                         30 * time.Second,
+		ACLTokenTTL:                          30 * time.Second,
+		ACLDefaultPolicy:                     "allow",
+		ACLDownPolicy:                        "extend-cache",
+		ACLReplicationRate:                   1,
+		ACLReplicationBurst:                  5,
+		ACLReplicationApplyLimit:             100, // ops / sec
+		ConfigReplicationRate:                1,
+		ConfigReplicationBurst:               5,
+		ConfigReplicationApplyLimit:          100, // ops / sec
+		FederationStateReplicationRate:       1,
+		FederationStateReplicationBurst:      5,
+		FederationStateReplicationApplyLimit: 100, // ops / sec
+		TombstoneTTL:                         15 * time.Minute,
+		TombstoneTTLGranularity:              30 * time.Second,
+		SessionTTLMin:                        10 * time.Second,
+		ACLTokenMinExpirationTTL:             1 * time.Minute,
+		ACLTokenMaxExpirationTTL:             24 * time.Hour,
 
 		// These are tuned to provide a total throughput of 128 updates
 		// per second. If you update these, you should update the client-
@@ -562,9 +583,9 @@ func DefaultConfig() *Config {
 		CAConfig: &structs.CAConfiguration{
 			Provider: "consul",
 			Config: map[string]interface{}{
-				"RotationPeriod":      "2160h",
-				"LeafCertTTL":         "72h",
-				"IntermediateCertTTL": "8760h", // 365 * 24h
+				"RotationPeriod":      structs.DefaultCARotationPeriod,
+				"LeafCertTTL":         structs.DefaultLeafCertTTL,
+				"IntermediateCertTTL": structs.DefaultIntermediateCertTTL,
 			},
 		},
 
