@@ -8,6 +8,7 @@ import (
 	"github.com/hashicorp/consul/agent/consul/state"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/go-msgpack/codec"
+	"github.com/hashicorp/go-raftchunking"
 	"github.com/hashicorp/raft"
 )
 
@@ -15,7 +16,8 @@ import (
 // state in a way that can be accessed concurrently with operations
 // that may modify the live state.
 type snapshot struct {
-	state *state.Snapshot
+	state      *state.Snapshot
+	chunkState *raftchunking.State
 }
 
 // snapshotHeader is the first entry in our snapshot
@@ -63,7 +65,7 @@ func (s *snapshot) Persist(sink raft.SnapshotSink) error {
 	header := snapshotHeader{
 		LastIndex: s.state.LastIndex(),
 	}
-	encoder := codec.NewEncoder(sink, msgpackHandle)
+	encoder := codec.NewEncoder(sink, structs.MsgpackHandle)
 	if err := encoder.Encode(&header); err != nil {
 		sink.Cancel()
 		return err
@@ -76,6 +78,18 @@ func (s *snapshot) Persist(sink raft.SnapshotSink) error {
 			return err
 		}
 	}
+
+	// Persist chunking state
+	if s.chunkState == nil {
+		return nil
+	}
+	if _, err := sink.Write([]byte{byte(structs.ChunkingStateType)}); err != nil {
+		return err
+	}
+	if err := encoder.Encode(s.chunkState); err != nil {
+		return err
+	}
+
 	return nil
 }
 
