@@ -31,6 +31,7 @@ func init() {
 	registerRestorer(structs.ACLRoleSetRequestType, restoreRole)
 	registerRestorer(structs.ACLBindingRuleSetRequestType, restoreBindingRule)
 	registerRestorer(structs.ACLAuthMethodSetRequestType, restoreAuthMethod)
+	registerRestorer(structs.FederationStateRequestType, restoreFederationState)
 }
 
 func persistOSS(s *snapshot, sink raft.SnapshotSink, encoder *codec.Encoder) error {
@@ -70,6 +71,9 @@ func persistOSS(s *snapshot, sink raft.SnapshotSink, encoder *codec.Encoder) err
 	if err := s.persistConfigEntries(sink, encoder); err != nil {
 		return err
 	}
+	if err := s.persistFederationStates(sink, encoder); err != nil {
+		return err
+	}
 	if err := s.persistIndex(sink, encoder); err != nil {
 		return err
 	}
@@ -95,6 +99,7 @@ func (s *snapshot) persistNodes(sink raft.SnapshotSink,
 			Address:         n.Address,
 			TaggedAddresses: n.TaggedAddresses,
 			NodeMeta:        n.Meta,
+			RaftIndex:       n.RaftIndex,
 		}
 
 		// Register the node itself
@@ -435,6 +440,30 @@ func (s *snapshot) persistConfigEntries(sink raft.SnapshotSink,
 	return nil
 }
 
+func (s *snapshot) persistFederationStates(sink raft.SnapshotSink, encoder *codec.Encoder) error {
+	fedStates, err := s.state.FederationStates()
+	if err != nil {
+		return err
+	}
+
+	for _, fedState := range fedStates {
+		if _, err := sink.Write([]byte{byte(structs.FederationStateRequestType)}); err != nil {
+			return err
+		}
+		// Encode the entry request without an operation since we don't need it for restoring.
+		// The request is used for its custom decoding/encoding logic around the ConfigEntry
+		// interface.
+		req := &structs.FederationStateRequest{
+			Op:    structs.FederationStateUpsert,
+			State: fedState,
+		}
+		if err := encoder.Encode(req); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
 func (s *snapshot) persistIndex(sink raft.SnapshotSink, encoder *codec.Encoder) error {
 	// Get all the indexes
 	iter, err := s.state.Indexes()
@@ -675,3 +704,11 @@ func restoreAuthMethod(header *snapshotHeader, restore *state.Restore, decoder *
 	}
 	return restore.ACLAuthMethod(&req)
 }
+
+func restoreFederationState(header *snapshotHeader, restore *state.Restore, decoder *codec.Decoder) error {
+	var req structs.FederationStateRequest
+	if err := decoder.Decode(&req); err != nil {
+		return err
+	}
+	return restore.FederationState(req.State)
+}
