@@ -1,7 +1,9 @@
 package consul
 
 import (
+	"context"
 	"crypto/tls"
+	"encoding/binary"
 	"errors"
 	"fmt"
 	"io"
@@ -11,7 +13,9 @@ import (
 	"time"
 
 	"github.com/armon/go-metrics"
+	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/consul/state"
+	"github.com/hashicorp/consul/agent/consul/wanfed"
 	"github.com/hashicorp/consul/agent/metadata"
 	"github.com/hashicorp/consul/agent/pool"
 	"github.com/hashicorp/consul/agent/structs"
@@ -89,10 +93,6 @@ func logConn(conn net.Conn) string {
 // handleConn is used to determine if this is a Raft or
 // Consul type RPC connection and invoke the correct handler
 func (s *Server) handleConn(conn net.Conn, isTLS bool) {
-
-	// Read a single byte
-	buf := make([]byte, 1)
-
 	// Limit how long the client can hold the connection open before they send the
 	// magic byte (and authenticate when mTLS is enabled). If `isTLS == true` then
 	// this also enforces a timeout on how long it takes for the handshake to
@@ -100,6 +100,34 @@ func (s *Server) handleConn(conn net.Conn, isTLS bool) {
 	if s.config.RPCHandshakeTimeout > 0 {
 		conn.SetReadDeadline(time.Now().Add(s.config.RPCHandshakeTimeout))
 	}
+
+	if !isTLS && s.tlsConfigurator.MutualTLSCapable() {
+		// See if actually this is native TLS multiplexed onto the old
+		// "type-byte" system.
+
+		peekedConn, nativeTLS, err := pool.PeekForTLS(conn)
+		if err != nil {
+			if err != io.EOF {
+				s.rpcLogger().Error(
+					"failed to read first byte",
+					"conn", logConn(conn),
+					"error", err,
+				)
+			}
+			conn.Close()
+			return
+		}
+
+		if nativeTLS {
+			s.handleNativeTLS(peekedConn)
+			return
+		}
+		conn = peekedConn
+	}
+
+	// Read a single byte
+	buf := make([]byte, 1)
+
 	if _, err := conn.Read(buf); err != nil {
 		if err != io.EOF {
 			s.rpcLogger().Error("failed to read byte",
@@ -171,12 +199,106 @@ func (s *Server) handleConn(conn net.Conn, isTLS bool) {
 	}
 }
 
+func (s *Server) handleNativeTLS(conn net.Conn) {
+	s.rpcLogger().Trace(
+		"detected actual TLS over RPC port",
+		"conn", logConn(conn),
+	)
+
+	tlscfg := s.tlsConfigurator.IncomingALPNRPCConfig(pool.RPCNextProtos)
+	tlsConn := tls.Server(conn, tlscfg)
+
+	// Force the handshake to conclude.
+	if err := tlsConn.Handshake(); err != nil {
+		s.rpcLogger().Error(
+			"TLS handshake failed",
+			"conn", logConn(conn),
+			"error", err,
+		)
+		conn.Close()
+		return
+	}
+
+	// Reset the deadline as we aren't sure what is expected next - it depends on
+	// the protocol.
+	if s.config.RPCHandshakeTimeout > 0 {
+		conn.SetReadDeadline(time.Time{})
+	}
+
+	var (
+		cs        = tlsConn.ConnectionState()
+		sni       = cs.ServerName
+		nextProto = cs.NegotiatedProtocol
+
+		transport = s.memberlistTransportWAN
+	)
+
+	s.rpcLogger().Trace(
+		"accepted nativeTLS RPC",
+		"sni", sni,
+		"protocol", nextProto,
+		"conn", logConn(conn),
+	)
+
+	switch nextProto {
+	case pool.ALPN_RPCConsul:
+		s.handleConsulConn(tlsConn)
+
+	case pool.ALPN_RPCRaft:
+		metrics.IncrCounter([]string{"rpc", "raft_handoff"}, 1)
+		s.raftLayer.Handoff(tlsConn)
+
+	case pool.ALPN_RPCMultiplexV2:
+		s.handleMultiplexV2(tlsConn)
+
+	case pool.ALPN_RPCSnapshot:
+		s.handleSnapshotConn(tlsConn)
+
+	case pool.ALPN_WANGossipPacket:
+		if err := s.handleALPN_WANGossipPacketStream(tlsConn); err != nil && err != io.EOF {
+			s.rpcLogger().Error(
+				"failed to ingest RPC",
+				"sni", sni,
+				"protocol", nextProto,
+				"conn", logConn(conn),
+				"error", err,
+			)
+		}
+
+	case pool.ALPN_WANGossipStream:
+		// No need to defer the conn.Close() here, the Ingest methods do that.
+		if err := transport.IngestStream(tlsConn); err != nil {
+			s.rpcLogger().Error(
+				"failed to ingest RPC",
+				"sni", sni,
+				"protocol", nextProto,
+				"conn", logConn(conn),
+				"error", err,
+			)
+		}
+
+	default:
+		if !s.handleEnterpriseNativeTLSConn(nextProto, conn) {
+			s.rpcLogger().Error(
+				"discarding RPC for unknown negotiated protocol",
+				"failed to ingest RPC",
+				"protocol", nextProto,
+				"conn", logConn(conn),
+			)
+			conn.Close()
+		}
+	}
+}
+
 // handleMultiplexV2 is used to multiplex a single incoming connection
 // using the Yamux multiplexer
 func (s *Server) handleMultiplexV2(conn net.Conn) {
 	defer conn.Close()
 	conf := yamux.DefaultConfig()
-	conf.LogOutput = s.config.LogOutput
+	// override the default because LogOutput conflicts with Logger
+	conf.LogOutput = nil
+	// TODO: should this be created once and cached?
+	conf.Logger = s.logger.StandardLogger(&hclog.StandardLoggerOptions{InferLevels: true})
 	server, _ := yamux.Server(conn, conf)
 	for {
 		sub, err := server.Accept()
@@ -292,6 +414,70 @@ func (s *Server) handleSnapshotConn(conn net.Conn) {
 	}()
 }
 
+func (s *Server) handleALPN_WANGossipPacketStream(conn net.Conn) error {
+	defer conn.Close()
+
+	transport := s.memberlistTransportWAN
+	for {
+		select {
+		case <-s.shutdownCh:
+			return nil
+		default:
+		}
+
+		// Note: if we need to change this format to have additional header
+		// information we can just negotiate a different ALPN protocol instead
+		// of needing any sort of version field here.
+		prefixLen, err := readUint32(conn, wanfed.GossipPacketMaxIdleTime)
+		if err != nil {
+			return err
+		}
+
+		// Avoid a memory exhaustion DOS vector here by capping how large this
+		// packet can be to something reasonable.
+		if prefixLen > wanfed.GossipPacketMaxByteSize {
+			return fmt.Errorf("gossip packet size %d exceeds threshold of %d", prefixLen, wanfed.GossipPacketMaxByteSize)
+		}
+
+		lc := &limitedConn{
+			Conn: conn,
+			lr:   io.LimitReader(conn, int64(prefixLen)),
+		}
+
+		if err := transport.IngestPacket(lc, conn.RemoteAddr(), time.Now(), false); err != nil {
+			return err
+		}
+	}
+}
+
+func readUint32(conn net.Conn, timeout time.Duration) (uint32, error) {
+	// Since requests are framed we can easily just set a deadline on
+	// reading that frame and then disable it for the rest of the body.
+	if err := conn.SetReadDeadline(time.Now().Add(timeout)); err != nil {
+		return 0, err
+	}
+
+	var v uint32
+	if err := binary.Read(conn, binary.BigEndian, &v); err != nil {
+		return 0, err
+	}
+
+	if err := conn.SetReadDeadline(time.Time{}); err != nil {
+		return 0, err
+	}
+
+	return v, nil
+}
+
+type limitedConn struct {
+	net.Conn
+	lr io.Reader
+}
+
+func (c *limitedConn) Read(b []byte) (n int, err error) {
+	return c.lr.Read(b)
+}
+
 // canRetry returns true if the given situation is safe for a retry.
 func canRetry(args interface{}, err error) bool {
 	// No leader errors are always safe to retry since no state could have
@@ -316,14 +502,31 @@ func canRetry(args interface{}, err error) bool {
 	return false
 }
 
-// forward is used to forward to a remote DC or to forward to the local leader
+// ForwardRPC is used to forward an RPC request to a remote DC or to the local leader
 // Returns a bool of if forwarding was performed, as well as any error
-func (s *Server) forward(method string, info structs.RPCInfo, args interface{}, reply interface{}) (bool, error) {
+func (s *Server) ForwardRPC(method string, info structs.RPCInfo, args interface{}, reply interface{}) (bool, error) {
 	var firstCheck time.Time
 
 	// Handle DC forwarding
 	dc := info.RequestDatacenter()
 	if dc != s.config.Datacenter {
+		// Local tokens only work within the current datacenter. Check to see
+		// if we are attempting to forward one to a remote datacenter and strip
+		// it, falling back on the anonymous token on the other end.
+		if token := info.TokenSecret(); token != "" {
+			done, ident, err := s.ResolveIdentityFromToken(token)
+			if done {
+				if err != nil && !acl.IsErrNotFound(err) {
+					return false, err
+				}
+				if ident != nil && ident.IsLocal() {
+					// Strip it from the request.
+					info.SetTokenSecret("")
+					defer info.SetTokenSecret(token)
+				}
+			}
+		}
+
 		err := s.forwardDC(method, dc, args, reply)
 		return true, err
 	}
@@ -352,8 +555,8 @@ CHECK_LEADER:
 	// Handle the case of a known leader
 	rpcErr := structs.ErrNoLeader
 	if leader != nil {
-		rpcErr = s.connPool.RPC(s.config.Datacenter, leader.Addr,
-			leader.Version, method, args, reply)
+		rpcErr = s.connPool.RPC(s.config.Datacenter, leader.ShortName, leader.Addr,
+			method, args, reply)
 		if rpcErr != nil && canRetry(info, rpcErr) {
 			goto RETRY
 		}
@@ -418,7 +621,7 @@ func (s *Server) forwardDC(method, dc string, args interface{}, reply interface{
 
 	metrics.IncrCounterWithLabels([]string{"rpc", "cross-dc"}, 1,
 		[]metrics.Label{{Name: "datacenter", Value: dc}})
-	if err := s.connPool.RPC(dc, server.Addr, server.Version, method, args, reply); err != nil {
+	if err := s.connPool.RPC(dc, server.ShortName, server.Addr, method, args, reply); err != nil {
 		manager.NotifyFailedServer(server)
 		s.rpcLogger().Error("RPC failed to server in DC",
 			"server", server.Addr,
@@ -432,22 +635,17 @@ func (s *Server) forwardDC(method, dc string, args interface{}, reply interface{
 	return nil
 }
 
-// globalRPC is used to forward an RPC request to one server in each datacenter.
-// This will only error for RPC-related errors. Otherwise, application-level
-// errors can be sent in the response objects.
-func (s *Server) globalRPC(method string, args interface{},
-	reply structs.CompoundResponse) error {
+// keyringRPCs is used to forward an RPC request to a server in each dc. This
+// will only error for RPC-related errors. Otherwise, application-level errors
+// can be sent in the response objects.
+func (s *Server) keyringRPCs(method string, args interface{}, dcs []string) (*structs.KeyringResponses, error) {
 
-	// Make a new request into each datacenter
-	dcs := s.router.GetDatacenters()
-
-	replies, total := 0, len(dcs)
-	errorCh := make(chan error, total)
-	respCh := make(chan interface{}, total)
+	errorCh := make(chan error, len(dcs))
+	respCh := make(chan *structs.KeyringResponses, len(dcs))
 
 	for _, dc := range dcs {
 		go func(dc string) {
-			rr := reply.New()
+			rr := &structs.KeyringResponses{}
 			if err := s.forwardDC(method, dc, args, &rr); err != nil {
 				errorCh <- err
 				return
@@ -456,16 +654,16 @@ func (s *Server) globalRPC(method string, args interface{},
 		}(dc)
 	}
 
-	for replies < total {
+	responses := &structs.KeyringResponses{}
+	for i := 0; i < len(dcs); i++ {
 		select {
 		case err := <-errorCh:
-			return err
+			return nil, err
 		case rr := <-respCh:
-			reply.Add(rr)
-			replies++
+			responses.Add(rr)
 		}
 	}
-	return nil
+	return responses, nil
 }
 
 type raftEncoder func(structs.MessageType, interface{}) ([]byte, error)
@@ -553,7 +751,9 @@ type queryFn func(memdb.WatchSet, *state.Store) error
 
 // blockingQuery is used to process a potentially blocking query operation.
 func (s *Server) blockingQuery(queryOpts structs.QueryOptionsCompat, queryMeta structs.QueryMetaCompat, fn queryFn) error {
-	var timeout *time.Timer
+	var cancel func()
+	var ctx context.Context = &lib.StopChannelContext{StopCh: s.shutdownCh}
+
 	var queriesBlocking uint64
 	var queryTimeout time.Duration
 
@@ -577,9 +777,9 @@ func (s *Server) blockingQuery(queryOpts structs.QueryOptionsCompat, queryMeta s
 	// Apply a small amount of jitter to the request.
 	queryTimeout += lib.RandomStagger(queryTimeout / jitterFraction)
 
-	// Setup a query timeout.
-	timeout = time.NewTimer(queryTimeout)
-	defer timeout.Stop()
+	// wrap the base context with a deadline
+	ctx, cancel = context.WithDeadline(ctx, time.Now().Add(queryTimeout))
+	defer cancel()
 
 	// instrument blockingQueries
 	// atomic inc our server's count of in-flight blockingQueries and store the new value
@@ -634,7 +834,9 @@ RUN_QUERY:
 	}
 	// block up to the timeout if we don't see anything fresh.
 	if err == nil && minQueryIndex > 0 && queryMeta.GetIndex() <= minQueryIndex {
-		if expired := ws.Watch(timeout.C); !expired {
+		if err := ws.WatchCtx(ctx); err == nil {
+			// a non-nil error only occurs when the context is cancelled
+
 			// If a restore may have woken us up then bail out from
 			// the query immediately. This is slightly race-ey since
 			// this might have been interrupted for other reasons,
