@@ -19,7 +19,6 @@ import (
 
 	metrics "github.com/armon/go-metrics"
 	"github.com/hashicorp/consul/acl"
-	ca "github.com/hashicorp/consul/agent/connect/ca"
 	"github.com/hashicorp/consul/agent/consul/authmethod"
 	"github.com/hashicorp/consul/agent/consul/authmethod/ssoauth"
 	"github.com/hashicorp/consul/agent/consul/autopilot"
@@ -139,20 +138,8 @@ type Server struct {
 	// autopilot is the Autopilot instance for this server.
 	autopilot *autopilot.Autopilot
 
-	// autopilotWaitGroup is used to block until Autopilot shuts down.
-	autopilotWaitGroup sync.WaitGroup
-
-	// caProviderReconfigurationLock guards the provider reconfiguration.
-	caProviderReconfigurationLock sync.Mutex
-	// caProvider is the current CA provider in use for Connect. This is
-	// only non-nil when we are the leader.
-	caProvider ca.Provider
-	// caProviderRoot is the CARoot that was stored along with the ca.Provider
-	// active. It's only updated in lock-step with the caProvider. This prevents
-	// races between state updates to active roots and the fetch of the provider
-	// instance.
-	caProviderRoot *structs.CARoot
-	caProviderLock sync.RWMutex
+	// caManager is used to synchronize CA operations across the leader and RPC functions.
+	caManager *CAManager
 
 	// rate limiter to use when signing leaf certificates
 	caLeafLimiter connectSignRateLimiter
@@ -302,10 +289,6 @@ type Server struct {
 	shutdownCh   chan struct{}
 	shutdownLock sync.Mutex
 
-	// State for whether this datacenter is acting as a secondary CA.
-	actingSecondaryCA   bool
-	actingSecondaryLock sync.RWMutex
-
 	// Manager to handle starting/stopping go routines when establishing/revoking raft leadership
 	leaderRoutineManager *LeaderRoutineManager
 
@@ -497,6 +480,7 @@ func NewServer(config *Config, options ...ConsulOption) (*Server, error) {
 		return nil, fmt.Errorf("Failed to start Raft: %v", err)
 	}
 
+	s.caManager = NewCAManager(&caDelegateWithState{s}, s.loggers.Named(logging.Connect), s.config)
 	if s.config.ConnectEnabled && (s.config.AutoEncryptAllowTLS || s.config.AutoConfigAuthzEnabled) {
 		go s.connectCARootsMonitor(&lib.StopChannelContext{StopCh: s.shutdownCh})
 	}
