@@ -157,7 +157,7 @@ func TestResetSessionTimerLocked(t *testing.T) {
 
 	testrpc.WaitForLeader(t, s1.RPC, "dc1")
 
-	s1.createSessionTimer("foo", 5*time.Millisecond)
+	s1.createSessionTimer("foo", 5*time.Millisecond, nil)
 	if s1.sessionTimers.Get("foo") == nil {
 		t.Fatalf("missing timer")
 	}
@@ -178,7 +178,7 @@ func TestResetSessionTimerLocked_Renew(t *testing.T) {
 
 	retry.Run(t, func(r *retry.R) {
 		// create the timer and make verify it was created
-		s1.createSessionTimer("foo", ttl)
+		s1.createSessionTimer("foo", ttl, nil)
 		if s1.sessionTimers.Get("foo") == nil {
 			r.Fatalf("missing timer")
 		}
@@ -194,7 +194,7 @@ func TestResetSessionTimerLocked_Renew(t *testing.T) {
 	retry.Run(t, func(r *retry.R) {
 		// renew the session which will reset the TTL to 2*ttl
 		// since that is the current SessionTTLMultiplier
-		s1.createSessionTimer("foo", ttl)
+		s1.createSessionTimer("foo", ttl, nil)
 		if s1.sessionTimers.Get("foo") == nil {
 			r.Fatal("missing timer")
 		}
@@ -231,6 +231,7 @@ func TestInvalidateSession(t *testing.T) {
 	if err := state.EnsureNode(1, &structs.Node{Node: "foo", Address: "127.0.0.1"}); err != nil {
 		t.Fatalf("err: %s", err)
 	}
+
 	session := &structs.Session{
 		ID:   generateUUID(),
 		Node: "foo",
@@ -241,10 +242,10 @@ func TestInvalidateSession(t *testing.T) {
 	}
 
 	// This should cause a destroy
-	s1.invalidateSession(session.ID)
+	s1.invalidateSession(session.ID, nil)
 
 	// Check it is gone
-	_, sess, err := state.SessionGet(nil, session.ID)
+	_, sess, err := state.SessionGet(nil, session.ID, nil)
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
@@ -259,7 +260,7 @@ func TestClearSessionTimer(t *testing.T) {
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
 
-	s1.createSessionTimer("foo", 5*time.Millisecond)
+	s1.createSessionTimer("foo", 5*time.Millisecond, nil)
 
 	err := s1.clearSessionTimer("foo")
 	if err != nil {
@@ -277,9 +278,9 @@ func TestClearAllSessionTimers(t *testing.T) {
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
 
-	s1.createSessionTimer("foo", 10*time.Millisecond)
-	s1.createSessionTimer("bar", 10*time.Millisecond)
-	s1.createSessionTimer("baz", 10*time.Millisecond)
+	s1.createSessionTimer("foo", 10*time.Millisecond, nil)
+	s1.createSessionTimer("bar", 10*time.Millisecond, nil)
+	s1.createSessionTimer("baz", 10*time.Millisecond, nil)
 
 	s1.clearAllSessionTimers()
 
