@@ -216,10 +216,7 @@ func (s *Snapshot) Checks(node string) (memdb.ResultIterator, error) {
 // performed within a single transaction to avoid race conditions on state
 // updates.
 func (s *Restore) Registration(idx uint64, req *structs.RegisterRequest) error {
-	if err := s.store.ensureRegistrationTxn(s.tx, idx, true, req); err != nil {
-		return err
-	}
-	return nil
+	return s.store.ensureRegistrationTxn(s.tx, idx, true, req, true)
 }
 
 // EnsureRegistration is used to make sure a node, service, and check
@@ -229,7 +226,7 @@ func (s *Store) EnsureRegistration(idx uint64, req *structs.RegisterRequest) err
 	tx := s.db.Txn(true)
 	defer tx.Abort()
 
-	if err := s.ensureRegistrationTxn(tx, idx, false, req); err != nil {
+	if err := s.ensureRegistrationTxn(tx, idx, false, req, false); err != nil {
 		return err
 	}
 
@@ -251,8 +248,8 @@ func (s *Store) ensureCheckIfNodeMatches(tx *memdb.Txn, idx uint64, preserveInde
 // ensureRegistrationTxn is used to make sure a node, service, and check
 // registration is performed within a single transaction to avoid race
 // conditions on state updates.
-func (s *Store) ensureRegistrationTxn(tx *memdb.Txn, idx uint64, preserveIndexes bool, req *structs.RegisterRequest) error {
-	if _, err := s.validateRegisterRequestTxn(tx, req); err != nil {
+func (s *Store) ensureRegistrationTxn(tx *memdb.Txn, idx uint64, preserveIndexes bool, req *structs.RegisterRequest, restore bool) error {
+	if _, err := s.validateRegisterRequestTxn(tx, req, restore); err != nil {
 		return err
 	}
 
