@@ -13,8 +13,7 @@ import (
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/testrpc"
 	"github.com/hashicorp/consul/types"
-	"github.com/hashicorp/net-rpc-msgpackrpc"
-	"github.com/pascaldekloe/goe/verify"
+	msgpackrpc "github.com/hashicorp/net-rpc-msgpackrpc"
 	"github.com/stretchr/testify/require"
 )
 
@@ -214,7 +213,7 @@ func TestTxn_Apply(t *testing.T) {
 
 	// Verify the state store directly.
 	state := s1.fsm.State()
-	_, d, err := state.KVSGet(nil, "test")
+	_, d, err := state.KVSGet(nil, "test", nil)
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
@@ -234,7 +233,7 @@ func TestTxn_Apply(t *testing.T) {
 		t.Fatalf("bad: %v", err)
 	}
 
-	_, s, err := state.NodeService("foo", "svc-foo")
+	_, s, err := state.NodeService("foo", "svc-foo", nil)
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
@@ -242,7 +241,7 @@ func TestTxn_Apply(t *testing.T) {
 		t.Fatalf("bad: %v", err)
 	}
 
-	_, c, err := state.NodeCheck("foo", types.CheckID("check-foo"))
+	_, c, err := state.NodeCheck("foo", types.CheckID("check-foo"), nil)
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
@@ -262,6 +261,7 @@ func TestTxn_Apply(t *testing.T) {
 						CreateIndex: d.CreateIndex,
 						ModifyIndex: d.ModifyIndex,
 					},
+					EnterpriseMeta: d.EnterpriseMeta,
 				},
 			},
 			&structs.TxnResult{
@@ -273,6 +273,7 @@ func TestTxn_Apply(t *testing.T) {
 						CreateIndex: d.CreateIndex,
 						ModifyIndex: d.ModifyIndex,
 					},
+					EnterpriseMeta: d.EnterpriseMeta,
 				},
 			},
 			&structs.TxnResult{
@@ -295,7 +296,7 @@ func TestTxn_Apply(t *testing.T) {
 			},
 		},
 	}
-	verify.Values(t, "", out, expected)
+	require.Equal(t, expected, out)
 }
 
 func TestTxn_Apply_ACLDeny(t *testing.T) {
@@ -609,7 +610,7 @@ func TestTxn_Apply_ACLDeny(t *testing.T) {
 		}
 	}
 
-	verify.Values(t, "", out, expected)
+	require.Equal(expected, out)
 }
 
 func TestTxn_Apply_LockDelay(t *testing.T) {
@@ -620,7 +621,7 @@ func TestTxn_Apply_LockDelay(t *testing.T) {
 	codec := rpcClient(t, s1)
 	defer codec.Close()
 
-	testrpc.WaitForLeader(t, s1.RPC, "dc1")
+	testrpc.WaitForTestAgent(t, s1.RPC, "dc1")
 
 	// Create and invalidate a session with a lock.
 	state := s1.fsm.State()
@@ -643,7 +644,8 @@ func TestTxn_Apply_LockDelay(t *testing.T) {
 	if ok, err := state.KVSLock(3, d); err != nil || !ok {
 		t.Fatalf("err: %v", err)
 	}
-	if err := state.SessionDestroy(4, id); err != nil {
+
+	if err := state.SessionDestroy(4, id, nil); err != nil {
 		t.Fatalf("err: %v", err)
 	}
 
@@ -728,10 +730,19 @@ func TestTxn_Read(t *testing.T) {
 	}
 	require.NoError(state.EnsureNode(2, node))
 
-	svc := structs.NodeService{ID: "svc-foo", Service: "svc-foo", Address: "127.0.0.1"}
+	svc := structs.NodeService{
+		ID:             "svc-foo",
+		Service:        "svc-foo",
+		Address:        "127.0.0.1",
+		EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
+	}
 	require.NoError(state.EnsureService(3, "foo", &svc))
 
-	check := structs.HealthCheck{Node: "foo", CheckID: types.CheckID("check-foo")}
+	check := structs.HealthCheck{
+		Node:           "foo",
+		CheckID:        types.CheckID("check-foo"),
+		EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
+	}
 	state.EnsureCheck(4, &check)
 
 	// Do a super basic request. The state store test covers the details so
@@ -777,6 +788,8 @@ func TestTxn_Read(t *testing.T) {
 	// Verify the transaction's return value.
 	svc.Weights = &structs.Weights{Passing: 1, Warning: 1}
 	svc.RaftIndex = structs.RaftIndex{CreateIndex: 3, ModifyIndex: 3}
+
+	entMeta := out.Results[0].KV.EnterpriseMeta
 	expected := structs.TxnReadResponse{
 		TxnResponse: structs.TxnResponse{
 			Results: structs.TxnResults{
@@ -788,6 +801,7 @@ func TestTxn_Read(t *testing.T) {
 							CreateIndex: 1,
 							ModifyIndex: 1,
 						},
+						EnterpriseMeta: entMeta,
 					},
 				},
 				&structs.TxnResult{
@@ -805,7 +819,7 @@ func TestTxn_Read(t *testing.T) {
 			KnownLeader: true,
 		},
 	}
-	verify.Values(t, "", out, expected)
+	require.Equal(expected, out)
 }
 
 func TestTxn_Read_ACLDeny(t *testing.T) {
