@@ -8,7 +8,7 @@ import (
 
 	cachetype "github.com/hashicorp/consul/agent/cache-types"
 	"github.com/hashicorp/consul/agent/structs"
-	"github.com/hashicorp/consul/lib"
+	"github.com/hashicorp/consul/lib/decode"
 	"github.com/mitchellh/mapstructure"
 )
 
@@ -60,7 +60,7 @@ func (s *HTTPServer) DiscoveryChainRead(resp http.ResponseWriter, req *http.Requ
 	defer setMeta(resp, &out.QueryMeta)
 
 	if s.agent.config.HTTPUseCache && args.QueryOptions.UseCache {
-		raw, m, err := s.agent.cache.Get(cachetype.CompiledDiscoveryChainName, &args)
+		raw, m, err := s.agent.cache.Get(req.Context(), cachetype.CompiledDiscoveryChainName, &args)
 		if err != nil {
 			return nil, err
 		}
@@ -90,66 +90,9 @@ func (s *HTTPServer) DiscoveryChainRead(resp http.ResponseWriter, req *http.Requ
 
 // discoveryChainReadRequest is the API variation of structs.DiscoveryChainRequest
 type discoveryChainReadRequest struct {
-	OverrideMeshGateway    structs.MeshGatewayConfig
-	OverrideProtocol       string
-	OverrideConnectTimeout time.Duration
-}
-
-func (t *discoveryChainReadRequest) UnmarshalJSON(data []byte) (err error) {
-	type Alias discoveryChainReadRequest
-	aux := &struct {
-		OverrideConnectTimeout interface{}
-		OverrideProtocol       interface{}
-		OverrideMeshGateway    *struct{ Mode interface{} }
-
-		OverrideConnectTimeoutSnake interface{}                 `json:"override_connect_timeout"`
-		OverrideProtocolSnake       interface{}                 `json:"override_protocol"`
-		OverrideMeshGatewaySnake    *struct{ Mode interface{} } `json:"override_mesh_gateway"`
-
-		*Alias
-	}{
-		Alias: (*Alias)(t),
-	}
-	if err = lib.UnmarshalJSON(data, &aux); err != nil {
-		return err
-	}
-
-	if aux.OverrideConnectTimeout == nil {
-		aux.OverrideConnectTimeout = aux.OverrideConnectTimeoutSnake
-	}
-	if aux.OverrideProtocol == nil {
-		aux.OverrideProtocol = aux.OverrideProtocolSnake
-	}
-	if aux.OverrideMeshGateway == nil {
-		aux.OverrideMeshGateway = aux.OverrideMeshGatewaySnake
-	}
-
-	// weakly typed input
-	if aux.OverrideProtocol != nil {
-		switch v := aux.OverrideProtocol.(type) {
-		case string, float64, bool:
-			t.OverrideProtocol = fmt.Sprintf("%v", v)
-		default:
-			return fmt.Errorf("OverrideProtocol: invalid type %T", v)
-		}
-	}
-	if aux.OverrideMeshGateway != nil {
-		t.OverrideMeshGateway.Mode = structs.MeshGatewayMode(fmt.Sprintf("%v", aux.OverrideMeshGateway.Mode))
-	}
-
-	// duration
-	if aux.OverrideConnectTimeout != nil {
-		switch v := aux.OverrideConnectTimeout.(type) {
-		case string:
-			if t.OverrideConnectTimeout, err = time.ParseDuration(v); err != nil {
-				return err
-			}
-		case float64:
-			t.OverrideConnectTimeout = time.Duration(v)
-		}
-	}
-
-	return nil
+	OverrideMeshGateway    structs.MeshGatewayConfig `alias:"override_mesh_gateway"`
+	OverrideProtocol       string                    `alias:"override_protocol"`
+	OverrideConnectTimeout time.Duration             `alias:"override_connect_timeout"`
 }
 
 // discoveryChainReadResponse is the API variation of structs.DiscoveryChainResponse
@@ -158,19 +101,17 @@ type discoveryChainReadResponse struct {
 }
 
 func decodeDiscoveryChainReadRequest(raw map[string]interface{}) (*discoveryChainReadRequest, error) {
-	// lib.TranslateKeys doesn't understand []map[string]interface{} so we have
-	// to do this part first.
-	raw = lib.PatchSliceOfMaps(raw, nil, nil)
-
-	lib.TranslateKeys(raw, map[string]string{
-		"override_mesh_gateway":    "overridemeshgateway",
-		"override_protocol":        "overrideprotocol",
-		"override_connect_timeout": "overrideconnecttimeout",
-	})
-
 	var apiReq discoveryChainReadRequest
+	// TODO(dnephin): at this time only JSON payloads are read, so it is unlikely
+	// that HookWeakDecodeFromSlice is necessary. It was added while porting
+	// from lib.PatchSliceOfMaps to decode.HookWeakDecodeFromSlice. It may be
+	// safe to remove in the future.
 	decodeConf := &mapstructure.DecoderConfig{
-		DecodeHook:       mapstructure.StringToTimeDurationHookFunc(),
+		DecodeHook: mapstructure.ComposeDecodeHookFunc(
+			decode.HookWeakDecodeFromSlice,
+			decode.HookTranslateKeys,
+			mapstructure.StringToTimeDurationHookFunc(),
+		),
 		Result:           &apiReq,
 		WeaklyTypedInput: true,
 	}
