@@ -526,7 +526,7 @@ START:
 	// Try to get a conn first
 	conn, err := p.acquire(dc, nodeName, addr)
 	if err != nil {
-		return nil, nil, fmt.Errorf("failed to get conn: %v", err)
+		return nil, nil, fmt.Errorf("failed to get conn: %w", err)
 	}
 
 	// Get a client
@@ -540,7 +540,7 @@ START:
 			retries++
 			goto START
 		}
-		return nil, nil, fmt.Errorf("failed to start stream: %v", err)
+		return nil, nil, fmt.Errorf("failed to start stream: %w", err)
 	}
 	return conn, client, nil
 }
@@ -582,14 +582,14 @@ func (p *ConnPool) rpcInsecure(dc string, nodeName string, addr net.Addr, method
 	var codec rpc.ClientCodec
 	conn, _, err := p.dial(dc, nodeName, addr, 0, RPCTLSInsecure)
 	if err != nil {
-		return fmt.Errorf("rpcinsecure error establishing connection: %v", err)
+		return fmt.Errorf("rpcinsecure error establishing connection: %w", err)
 	}
 	codec = msgpackrpc.NewCodecFromHandle(true, true, conn, structs.MsgpackHandle)
 
 	// Make the RPC call
 	err = msgpackrpc.CallWithCodec(codec, method, args, reply)
 	if err != nil {
-		return fmt.Errorf("rpcinsecure error making call: %v", err)
+		return fmt.Errorf("rpcinsecure error making call: %w", err)
 	}
 
 	return nil
@@ -601,7 +601,7 @@ func (p *ConnPool) rpc(dc string, nodeName string, addr net.Addr, method string,
 	// Get a usable client
 	conn, sc, err := p.getClient(dc, nodeName, addr)
 	if err != nil {
-		return fmt.Errorf("rpc error getting client: %v", err)
+		return fmt.Errorf("rpc error getting client: %w", err)
 	}
 
 	// Make the RPC call
@@ -618,7 +618,7 @@ func (p *ConnPool) rpc(dc string, nodeName string, addr net.Addr, method string,
 		}
 
 		p.releaseConn(conn)
-		return fmt.Errorf("rpc error making call: %v", err)
+		return fmt.Errorf("rpc error making call: %w", err)
 	}
 
 	// Done with the connection
