@@ -7,6 +7,7 @@ import (
 
 	"github.com/mitchellh/copystructure"
 	"github.com/stretchr/testify/require"
+	"golang.org/x/time/rate"
 
 	"github.com/hashicorp/consul/agent/cache"
 	cachetype "github.com/hashicorp/consul/agent/cache-types"
@@ -105,6 +106,7 @@ func TestManager_BasicLifecycle(t *testing.T) {
 		ID:      "web-sidecar-proxy",
 		Service: "web-sidecar-proxy",
 		Port:    9999,
+		Meta:    map[string]string{},
 		Proxy: structs.ConnectProxyConfig{
 			DestinationServiceID:   "web",
 			DestinationServiceName: "web",
@@ -197,22 +199,25 @@ func TestManager_BasicLifecycle(t *testing.T) {
 				Address:         webProxy.Address,
 				Port:            webProxy.Port,
 				Proxy:           mustCopyProxyConfig(t, webProxy),
+				ServiceMeta:     webProxy.Meta,
 				TaggedAddresses: make(map[string]structs.ServiceAddress),
 				Roots:           roots,
 				ConnectProxy: configSnapshotConnectProxy{
-					Leaf: leaf,
-					DiscoveryChain: map[string]*structs.CompiledDiscoveryChain{
-						"db": dbDefaultChain(),
-					},
-					WatchedUpstreams: nil, // Clone() clears this out
-					WatchedUpstreamEndpoints: map[string]map[string]structs.CheckServiceNodes{
-						"db": {
-							"db.default.dc1": TestUpstreamNodes(t),
+					ConfigSnapshotUpstreams: ConfigSnapshotUpstreams{
+						Leaf: leaf,
+						DiscoveryChain: map[string]*structs.CompiledDiscoveryChain{
+							"db": dbDefaultChain(),
+						},
+						WatchedUpstreams: nil, // Clone() clears this out
+						WatchedUpstreamEndpoints: map[string]map[string]structs.CheckServiceNodes{
+							"db": {
+								"db.default.dc1": TestUpstreamNodes(t),
+							},
+						},
+						WatchedGateways: nil, // Clone() clears this out
+						WatchedGatewayEndpoints: map[string]map[string]structs.CheckServiceNodes{
+							"db": {},
 						},
-					},
-					WatchedGateways: nil, // Clone() clears this out
-					WatchedGatewayEndpoints: map[string]map[string]structs.CheckServiceNodes{
-						"db": {},
 					},
 					PreparedQueryEndpoints: map[string]structs.CheckServiceNodes{},
 					WatchedServiceChecks:   map[structs.ServiceID][]structs.CheckType{},
@@ -241,23 +246,26 @@ func TestManager_BasicLifecycle(t *testing.T) {
 				Address:         webProxy.Address,
 				Port:            webProxy.Port,
 				Proxy:           mustCopyProxyConfig(t, webProxy),
+				ServiceMeta:     webProxy.Meta,
 				TaggedAddresses: make(map[string]structs.ServiceAddress),
 				Roots:           roots,
 				ConnectProxy: configSnapshotConnectProxy{
-					Leaf: leaf,
-					DiscoveryChain: map[string]*structs.CompiledDiscoveryChain{
-						"db": dbSplitChain(),
-					},
-					WatchedUpstreams: nil, // Clone() clears this out
-					WatchedUpstreamEndpoints: map[string]map[string]structs.CheckServiceNodes{
-						"db": {
-							"v1.db.default.dc1": TestUpstreamNodes(t),
-							"v2.db.default.dc1": TestUpstreamNodesAlternate(t),
+					ConfigSnapshotUpstreams: ConfigSnapshotUpstreams{
+						Leaf: leaf,
+						DiscoveryChain: map[string]*structs.CompiledDiscoveryChain{
+							"db": dbSplitChain(),
+						},
+						WatchedUpstreams: nil, // Clone() clears this out
+						WatchedUpstreamEndpoints: map[string]map[string]structs.CheckServiceNodes{
+							"db": {
+								"v1.db.default.dc1": TestUpstreamNodes(t),
+								"v2.db.default.dc1": TestUpstreamNodesAlternate(t),
+							},
+						},
+						WatchedGateways: nil, // Clone() clears this out
+						WatchedGatewayEndpoints: map[string]map[string]structs.CheckServiceNodes{
+							"db": {},
 						},
-					},
-					WatchedGateways: nil, // Clone() clears this out
-					WatchedGatewayEndpoints: map[string]map[string]structs.CheckServiceNodes{
-						"db": {},
 					},
 					PreparedQueryEndpoints: map[string]structs.CheckServiceNodes{},
 					WatchedServiceChecks:   map[structs.ServiceID][]structs.CheckType{},
@@ -328,7 +336,7 @@ func testManager_BasicLifecycle(
 	state.TriggerSyncChanges = func() {}
 
 	// Create manager
-	m, err := NewManager(ManagerConfig{c, state, source, logger})
+	m, err := NewManager(ManagerConfig{c, state, source, DNSConfig{}, logger, nil})
 	require.NoError(err)
 
 	// And run it
@@ -456,7 +464,7 @@ func TestManager_deliverLatest(t *testing.T) {
 	// None of these need to do anything to test this method just be valid
 	logger := testutil.Logger(t)
 	cfg := ManagerConfig{
-		Cache: cache.New(nil),
+		Cache: cache.New(cache.Options{EntryFetchRate: rate.Inf, EntryFetchMaxBurst: 2}),
 		State: local.NewState(local.Config{}, logger, &token.Store{}),
 		Source: &structs.QuerySource{
 			Node:       "node1",
@@ -478,12 +486,6 @@ func TestManager_deliverLatest(t *testing.T) {
 		Port:    2222,
 	}
 
-	// Put an overall time limit on this test case so we don't have to guard every
-	// call to ensure the whole test doesn't deadlock.
-	time.AfterFunc(100*time.Millisecond, func() {
-		t.Fatal("test timed out")
-	})
-
 	// test 1 buffered chan
 	ch1 := make(chan *ConfigSnapshot, 1)
 
