@@ -61,7 +61,7 @@ type ManagerSerfCluster interface {
 // Pinger is an interface wrapping client.ConnPool to prevent a cyclic import
 // dependency.
 type Pinger interface {
-	Ping(dc string, addr net.Addr, version int) (bool, error)
+	Ping(dc, nodeName string, addr net.Addr) (bool, error)
 }
 
 // serverList is a local copy of the struct used to maintain the list of
@@ -254,6 +254,9 @@ func (m *Manager) CheckServers(fn func(srv *metadata.Server) bool) {
 // getServerList is a convenience method which hides the locking semantics
 // of atomic.Value from the caller.
 func (m *Manager) getServerList() serverList {
+	if m == nil {
+		return serverList{}
+	}
 	return m.listValue.Load().(serverList)
 }
 
@@ -321,6 +324,25 @@ func (m *Manager) NumServers() int {
 	return len(l.servers)
 }
 
+func (m *Manager) healthyServer(server *metadata.Server) bool {
+	// Check to see if the manager is trying to ping itself. This
+	// is a small optimization to avoid performing an unnecessary
+	// RPC call.
+	// If this is true, we know there are healthy servers for this
+	// manager and we don't need to continue.
+	if m.serverName != "" && server.Name == m.serverName {
+		return true
+	}
+	if ok, err := m.connPoolPinger.Ping(server.Datacenter, server.ShortName, server.Addr); !ok {
+		m.logger.Debug("pinging server failed",
+			"server", server.String(),
+			"error", err,
+		)
+		return false
+	}
+	return true
+}
+
 // RebalanceServers shuffles the list of servers on this metadata.  The server
 // at the front of the list is selected for the next RPC.  RPC calls that
 // fail for a particular server are rotated to the end of the list.  This
@@ -345,19 +367,12 @@ func (m *Manager) RebalanceServers() {
 	// this loop mutates the server list in-place.
 	var foundHealthyServer bool
 	for i := 0; i < len(l.servers); i++ {
-		// Always test the first server.  Failed servers are cycled
+		// Always test the first server. Failed servers are cycled
 		// while Serf detects the node has failed.
-		srv := l.servers[0]
-
-		ok, err := m.connPoolPinger.Ping(srv.Datacenter, srv.Addr, srv.Version)
-		if ok {
+		if m.healthyServer(l.servers[0]) {
 			foundHealthyServer = true
 			break
 		}
-		m.logger.Debug("pinging server failed",
-			"server", srv.String(),
-			"error", err,
-		)
 		l.servers = l.cycleServer()
 	}
 
@@ -377,16 +392,17 @@ func (m *Manager) RebalanceServers() {
 			"number_of_servers", len(l.servers),
 			"active_server", l.servers[0].String(),
 		)
-	} else {
-		// reconcileServerList failed because Serf removed the server
-		// that was at the front of the list that had successfully
-		// been Ping'ed.  Between the Ping and reconcile, a Serf
-		// event had shown up removing the node.
-		//
-		// Instead of doing any heroics, "freeze in place" and
-		// continue to use the existing connection until the next
-		// rebalance occurs.
 	}
+	// else {
+	// reconcileServerList failed because Serf removed the server
+	// that was at the front of the list that had successfully
+	// been Ping'ed.  Between the Ping and reconcile, a Serf
+	// event had shown up removing the node.
+	//
+	// Instead of doing any heroics, "freeze in place" and
+	// continue to use the existing connection until the next
+	// rebalance occurs.
+	// }
 }
 
 // reconcileServerList returns true when the first server in serverList
