@@ -9,7 +9,6 @@ import (
 	"testing"
 
 	"github.com/hashicorp/consul/agent/structs"
-	"github.com/hashicorp/consul/sdk/testutil"
 	"github.com/hashicorp/consul/sdk/testutil/retry"
 	"github.com/hashicorp/consul/testrpc"
 	"github.com/stretchr/testify/require"
@@ -18,7 +17,7 @@ import (
 func TestServiceManager_RegisterService(t *testing.T) {
 	require := require.New(t)
 
-	a := NewTestAgent(t, t.Name(), "enable_central_service_config = true")
+	a := NewTestAgent(t, "enable_central_service_config = true")
 	defer a.Shutdown()
 
 	testrpc.WaitForLeader(t, a.RPC, "dc1")
@@ -65,7 +64,7 @@ func TestServiceManager_RegisterService(t *testing.T) {
 func TestServiceManager_RegisterSidecar(t *testing.T) {
 	require := require.New(t)
 
-	a := NewTestAgent(t, t.Name(), "enable_central_service_config = true")
+	a := NewTestAgent(t, "enable_central_service_config = true")
 	defer a.Shutdown()
 
 	testrpc.WaitForLeader(t, a.RPC, "dc1")
@@ -152,7 +151,7 @@ func TestServiceManager_RegisterSidecar(t *testing.T) {
 func TestServiceManager_RegisterMeshGateway(t *testing.T) {
 	require := require.New(t)
 
-	a := NewTestAgent(t, t.Name(), "enable_central_service_config = true")
+	a := NewTestAgent(t, "enable_central_service_config = true")
 	defer a.Shutdown()
 
 	testrpc.WaitForLeader(t, a.RPC, "dc1")
@@ -205,6 +204,62 @@ func TestServiceManager_RegisterMeshGateway(t *testing.T) {
 	}, gateway)
 }
 
+func TestServiceManager_RegisterTerminatingGateway(t *testing.T) {
+	require := require.New(t)
+
+	a := NewTestAgent(t, "enable_central_service_config = true")
+	defer a.Shutdown()
+
+	testrpc.WaitForLeader(t, a.RPC, "dc1")
+
+	// Register a global proxy and service config
+	testApplyConfigEntries(t, a,
+		&structs.ProxyConfigEntry{
+			Config: map[string]interface{}{
+				"foo": 1,
+			},
+		},
+		&structs.ServiceConfigEntry{
+			Kind:     structs.ServiceDefaults,
+			Name:     "terminating-gateway",
+			Protocol: "http",
+		},
+	)
+
+	// Now register a terminating-gateway.
+	svc := &structs.NodeService{
+		Kind:           structs.ServiceKindTerminatingGateway,
+		ID:             "terminating-gateway",
+		Service:        "terminating-gateway",
+		Port:           443,
+		EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
+	}
+
+	require.NoError(a.AddService(svc, nil, false, "", ConfigSourceLocal))
+
+	// Verify gateway got global config loaded
+	gateway := a.State.Service(structs.NewServiceID("terminating-gateway", nil))
+	require.NotNil(gateway)
+	require.Equal(&structs.NodeService{
+		Kind:            structs.ServiceKindTerminatingGateway,
+		ID:              "terminating-gateway",
+		Service:         "terminating-gateway",
+		Port:            443,
+		TaggedAddresses: map[string]structs.ServiceAddress{},
+		Proxy: structs.ConnectProxyConfig{
+			Config: map[string]interface{}{
+				"foo":      int64(1),
+				"protocol": "http",
+			},
+		},
+		Weights: &structs.Weights{
+			Passing: 1,
+			Warning: 1,
+		},
+		EnterpriseMeta: *structs.DefaultEnterpriseMeta(),
+	}, gateway)
+}
+
 func TestServiceManager_PersistService_API(t *testing.T) {
 	// This is the ServiceManager version of TestAgent_PersistService  and
 	// TestAgent_PurgeService.
@@ -213,7 +268,7 @@ func TestServiceManager_PersistService_API(t *testing.T) {
 	require := require.New(t)
 
 	// Launch a server to manage the config entries.
-	serverAgent := NewTestAgent(t, t.Name(), `enable_central_service_config = true`)
+	serverAgent := NewTestAgent(t, `enable_central_service_config = true`)
 	defer serverAgent.Shutdown()
 	testrpc.WaitForLeader(t, serverAgent.RPC, "dc1")
 
@@ -237,16 +292,12 @@ func TestServiceManager_PersistService_API(t *testing.T) {
 	)
 
 	// Now launch a single client agent
-	dataDir := testutil.TempDir(t, "agent") // we manage the data dir
-	defer os.RemoveAll(dataDir)
-
 	cfg := `
 	    enable_central_service_config = true
 		server = false
 		bootstrap = false
-		data_dir = "` + dataDir + `"
 	`
-	a := NewTestAgentWithFields(t, true, TestAgent{HCL: cfg, DataDir: dataDir})
+	a := StartTestAgent(t, TestAgent{HCL: cfg})
 	defer a.Shutdown()
 
 	// Join first
@@ -409,7 +460,7 @@ func TestServiceManager_PersistService_API(t *testing.T) {
 	serverAgent.Shutdown()
 
 	// Should load it back during later start.
-	a2 := NewTestAgentWithFields(t, true, TestAgent{HCL: cfg, DataDir: dataDir})
+	a2 := StartTestAgent(t, TestAgent{HCL: cfg, DataDir: a.DataDir})
 	defer a2.Shutdown()
 
 	{
@@ -430,7 +481,7 @@ func TestServiceManager_PersistService_ConfigFiles(t *testing.T) {
 	t.Parallel()
 
 	// Launch a server to manage the config entries.
-	serverAgent := NewTestAgent(t, t.Name(), `enable_central_service_config = true`)
+	serverAgent := NewTestAgent(t, `enable_central_service_config = true`)
 	defer serverAgent.Shutdown()
 	testrpc.WaitForLeader(t, serverAgent.RPC, "dc1")
 
@@ -454,9 +505,6 @@ func TestServiceManager_PersistService_ConfigFiles(t *testing.T) {
 	)
 
 	// Now launch a single client agent
-	dataDir := testutil.TempDir(t, "agent") // we manage the data dir
-	defer os.RemoveAll(dataDir)
-
 	serviceSnippet := `
 		service = {
 		  kind  = "connect-proxy"
@@ -479,12 +527,11 @@ func TestServiceManager_PersistService_ConfigFiles(t *testing.T) {
 
 	cfg := `
 	    enable_central_service_config = true
-		data_dir = "` + dataDir + `"
 		server = false
 		bootstrap = false
 	` + serviceSnippet
 
-	a := NewTestAgentWithFields(t, true, TestAgent{HCL: cfg, DataDir: dataDir})
+	a := StartTestAgent(t, TestAgent{HCL: cfg})
 	defer a.Shutdown()
 
 	// Join first
@@ -583,7 +630,7 @@ func TestServiceManager_PersistService_ConfigFiles(t *testing.T) {
 	serverAgent.Shutdown()
 
 	// Should load it back during later start.
-	a2 := NewTestAgentWithFields(t, true, TestAgent{HCL: cfg, DataDir: dataDir})
+	a2 := StartTestAgent(t, TestAgent{HCL: cfg, DataDir: a.DataDir})
 	defer a2.Shutdown()
 
 	{
@@ -601,7 +648,7 @@ func TestServiceManager_PersistService_ConfigFiles(t *testing.T) {
 func TestServiceManager_Disabled(t *testing.T) {
 	require := require.New(t)
 
-	a := NewTestAgent(t, t.Name(), "enable_central_service_config = false")
+	a := NewTestAgent(t, "enable_central_service_config = false")
 	defer a.Shutdown()
 
 	testrpc.WaitForLeader(t, a.RPC, "dc1")
