@@ -31,14 +31,14 @@ func (s *HTTPServer) Snapshot(resp http.ResponseWriter, req *http.Request) (inte
 		// Don't bother sending any request body through since it will
 		// be ignored.
 		var null bytes.Buffer
-		if err := s.agent.SnapshotRPC(&args, &null, resp, replyFn); err != nil {
+		if err := s.agent.delegate.SnapshotRPC(&args, &null, resp, replyFn); err != nil {
 			return nil, err
 		}
 		return nil, nil
 
 	case "PUT":
 		args.Op = structs.SnapshotRestore
-		if err := s.agent.SnapshotRPC(&args, req.Body, resp, nil); err != nil {
+		if err := s.agent.delegate.SnapshotRPC(&args, req.Body, resp, nil); err != nil {
 			return nil, err
 		}
 		return nil, nil
