@@ -38,6 +38,7 @@ type ServiceRouterConfigEntry struct {
 	// the default service.
 	Routes []ServiceRoute
 
+	Meta           map[string]string `json:",omitempty"`
 	EnterpriseMeta `hcl:",squash" mapstructure:",squash"`
 	RaftIndex
 }
@@ -54,6 +55,13 @@ func (e *ServiceRouterConfigEntry) GetName() string {
 	return e.Name
 }
 
+func (e *ServiceRouterConfigEntry) GetMeta() map[string]string {
+	if e == nil {
+		return nil
+	}
+	return e.Meta
+}
+
 func (e *ServiceRouterConfigEntry) Normalize() error {
 	if e == nil {
 		return fmt.Errorf("config entry is nil")
@@ -89,6 +97,10 @@ func (e *ServiceRouterConfigEntry) Validate() error {
 		return fmt.Errorf("Name is required")
 	}
 
+	if err := validateConfigEntryMeta(e.Meta); err != nil {
+		return err
+	}
+
 	// Technically you can have no explicit routes at all where just the
 	// catch-all is configured for you, but at that point maybe you should just
 	// delete it so it will default?
@@ -260,12 +272,12 @@ func (m *ServiceRouteMatch) IsEmpty() bool {
 
 // ServiceRouteHTTPMatch is a set of http-specific match criteria.
 type ServiceRouteHTTPMatch struct {
-	PathExact  string `json:",omitempty"`
-	PathPrefix string `json:",omitempty"`
-	PathRegex  string `json:",omitempty"`
+	PathExact  string `json:",omitempty" alias:"path_exact"`
+	PathPrefix string `json:",omitempty" alias:"path_prefix"`
+	PathRegex  string `json:",omitempty" alias:"path_regex"`
 
 	Header     []ServiceRouteHTTPMatchHeader     `json:",omitempty"`
-	QueryParam []ServiceRouteHTTPMatchQueryParam `json:",omitempty"`
+	QueryParam []ServiceRouteHTTPMatchQueryParam `json:",omitempty" alias:"query_param"`
 	Methods    []string                          `json:",omitempty"`
 }
 
@@ -308,7 +320,7 @@ type ServiceRouteDestination struct {
 	//
 	// If this field is specified then this route is ineligible for further
 	// splitting.
-	ServiceSubset string `json:",omitempty"`
+	ServiceSubset string `json:",omitempty" alias:"service_subset"`
 
 	// Namespace is the namespace to resolve the service from instead of the
 	// current namespace. If empty the current namespace is assumed.
@@ -320,24 +332,24 @@ type ServiceRouteDestination struct {
 	// PrefixRewrite allows for the proxied request to have its matching path
 	// prefix modified before being sent to the destination. Described more
 	// below in the envoy implementation section.
-	PrefixRewrite string `json:",omitempty"`
+	PrefixRewrite string `json:",omitempty" alias:"prefix_rewrite"`
 
 	// RequestTimeout is the total amount of time permitted for the entire
 	// downstream request (and retries) to be processed.
-	RequestTimeout time.Duration `json:",omitempty"`
+	RequestTimeout time.Duration `json:",omitempty" alias:"request_timeout"`
 
 	// NumRetries is the number of times to retry the request when a retryable
 	// result occurs. This seems fairly proxy agnostic.
-	NumRetries uint32 `json:",omitempty"`
+	NumRetries uint32 `json:",omitempty" alias:"num_retries"`
 
 	// RetryOnConnectFailure allows for connection failure errors to trigger a
 	// retry. This should be expressible in other proxies as it's just a layer
 	// 4 failure bubbling up to layer 7.
-	RetryOnConnectFailure bool `json:",omitempty"`
+	RetryOnConnectFailure bool `json:",omitempty" alias:"retry_on_connect_failure"`
 
 	// RetryOnStatusCodes is a flat list of http response status codes that are
 	// eligible for retry. This again should be feasible in any sane proxy.
-	RetryOnStatusCodes []uint32 `json:",omitempty"`
+	RetryOnStatusCodes []uint32 `json:",omitempty" alias:"retry_on_status_codes"`
 }
 
 func (e *ServiceRouteDestination) MarshalJSON() ([]byte, error) {
@@ -407,6 +419,7 @@ type ServiceSplitterConfigEntry struct {
 	// to the FIRST split.
 	Splits []ServiceSplit
 
+	Meta           map[string]string `json:",omitempty"`
 	EnterpriseMeta `hcl:",squash" mapstructure:",squash"`
 	RaftIndex
 }
@@ -423,6 +436,13 @@ func (e *ServiceSplitterConfigEntry) GetName() string {
 	return e.Name
 }
 
+func (e *ServiceSplitterConfigEntry) GetMeta() map[string]string {
+	if e == nil {
+		return nil
+	}
+	return e.Meta
+}
+
 func (e *ServiceSplitterConfigEntry) Normalize() error {
 	if e == nil {
 		return fmt.Errorf("config entry is nil")
@@ -449,7 +469,7 @@ func (e *ServiceSplitterConfigEntry) Normalize() error {
 
 func NormalizeServiceSplitWeight(weight float32) float32 {
 	weightScaled := scaleWeight(weight)
-	return float32(float32(weightScaled) / 100.0)
+	return float32(weightScaled) / 100.0
 }
 
 func (e *ServiceSplitterConfigEntry) Validate() error {
@@ -461,6 +481,10 @@ func (e *ServiceSplitterConfigEntry) Validate() error {
 		return fmt.Errorf("no splits configured")
 	}
 
+	if err := validateConfigEntryMeta(e.Meta); err != nil {
+		return err
+	}
+
 	const maxScaledWeight = 100 * 100
 
 	copyAsKey := func(s ServiceSplit) ServiceSplit {
@@ -576,7 +600,7 @@ type ServiceSplit struct {
 	//
 	// If this field is specified then this route is ineligible for further
 	// splitting.
-	ServiceSubset string `json:",omitempty"`
+	ServiceSubset string `json:",omitempty" alias:"service_subset"`
 
 	// Namespace is the namespace to resolve the service from instead of the
 	// current namespace. If empty the current namespace is assumed (optional).
@@ -604,7 +628,7 @@ type ServiceResolverConfigEntry struct {
 
 	// DefaultSubset is the subset to use when no explicit subset is
 	// requested. If empty the unnamed subset is used.
-	DefaultSubset string `json:",omitempty"`
+	DefaultSubset string `json:",omitempty" alias:"default_subset"`
 
 	// Subsets is a map of subset name to subset definition for all
 	// usable named subsets of this service. The map key is the name
@@ -637,8 +661,9 @@ type ServiceResolverConfigEntry struct {
 
 	// ConnectTimeout is the timeout for establishing new network connections
 	// to this service.
-	ConnectTimeout time.Duration `json:",omitempty"`
+	ConnectTimeout time.Duration `json:",omitempty" alias:"connect_timeout"`
 
+	Meta           map[string]string `json:",omitempty"`
 	EnterpriseMeta `hcl:",squash" mapstructure:",squash"`
 	RaftIndex
 }
@@ -710,6 +735,13 @@ func (e *ServiceResolverConfigEntry) GetName() string {
 	return e.Name
 }
 
+func (e *ServiceResolverConfigEntry) GetMeta() map[string]string {
+	if e == nil {
+		return nil
+	}
+	return e.Meta
+}
+
 func (e *ServiceResolverConfigEntry) Normalize() error {
 	if e == nil {
 		return fmt.Errorf("config entry is nil")
@@ -727,6 +759,10 @@ func (e *ServiceResolverConfigEntry) Validate() error {
 		return fmt.Errorf("Name is required")
 	}
 
+	if err := validateConfigEntryMeta(e.Meta); err != nil {
+		return err
+	}
+
 	if len(e.Subsets) > 0 {
 		for name, _ := range e.Subsets {
 			if name == "" {
@@ -884,7 +920,7 @@ type ServiceResolverSubset struct {
 	// to true, only instances with checks in the passing state will be
 	// returned. (behaves identically to the similarly named field on prepared
 	// queries).
-	OnlyPassing bool `json:",omitempty"`
+	OnlyPassing bool `json:",omitempty" alias:"only_passing"`
 }
 
 type ServiceResolverRedirect struct {
@@ -898,7 +934,7 @@ type ServiceResolverRedirect struct {
 	//
 	// If this is specified at least one of Service, Datacenter, or Namespace
 	// should be configured.
-	ServiceSubset string `json:",omitempty"`
+	ServiceSubset string `json:",omitempty" alias:"service_subset"`
 
 	// Namespace is the namespace to resolve the service from instead of the
 	// current one (optional).
@@ -926,7 +962,7 @@ type ServiceResolverFailover struct {
 	// requested service is used (optional).
 	//
 	// This is a DESTINATION during failover.
-	ServiceSubset string `json:",omitempty"`
+	ServiceSubset string `json:",omitempty" alias:"service_subset"`
 
 	// Namespace is the namespace to resolve the requested service from to form
 	// the failover group of instances. If empty the current namespace is used
