@@ -114,7 +114,7 @@ type ConnectProxyConfig struct {
 	Upstreams Upstreams `json:",omitempty"`
 
 	// MeshGateway defines the mesh gateway configuration for this upstream
-	MeshGateway MeshGatewayConfig `json:",omitempty"`
+	MeshGateway MeshGatewayConfig `json:",omitempty" alias:"mesh_gateway"`
 
 	// Expose defines whether checks or paths are exposed through the proxy
 	Expose ExposeConfig `json:",omitempty"`
@@ -247,6 +247,11 @@ type Upstream struct {
 
 	// MeshGateway is the configuration for mesh gateway usage of this upstream
 	MeshGateway MeshGatewayConfig `json:",omitempty"`
+
+	// IngressHosts are a list of hosts that should route to this upstream from
+	// an ingress gateway. This cannot and should not be set by a user, it is
+	// used internally to store the association of hosts to an upstream service.
+	IngressHosts []string `json:"-" bexpr:"-"`
 }
 
 func (t *Upstream) UnmarshalJSON(data []byte) (err error) {
@@ -358,7 +363,7 @@ func (u *Upstream) Identifier() string {
 	name := u.DestinationName
 	typ := u.DestinationType
 
-	if typ != UpstreamDestTypePreparedQuery && u.DestinationNamespace != "" && u.DestinationNamespace != "default" {
+	if typ != UpstreamDestTypePreparedQuery && u.DestinationNamespace != "" && u.DestinationNamespace != IntentionDefaultNamespace {
 		name = u.DestinationNamespace + "/" + u.DestinationName
 	}
 	if u.Datacenter != "" {
@@ -403,15 +408,26 @@ type ExposeConfig struct {
 	Paths []ExposePath `json:",omitempty"`
 }
 
+func (e ExposeConfig) Clone() ExposeConfig {
+	e2 := e
+	if len(e.Paths) > 0 {
+		e2.Paths = make([]ExposePath, 0, len(e.Paths))
+		for _, p := range e.Paths {
+			e2.Paths = append(e2.Paths, p)
+		}
+	}
+	return e2
+}
+
 type ExposePath struct {
 	// ListenerPort defines the port of the proxy's listener for exposed paths.
-	ListenerPort int `json:",omitempty"`
+	ListenerPort int `json:",omitempty" alias:"listener_port"`
 
 	// ExposePath is the path to expose through the proxy, ie. "/metrics."
 	Path string `json:",omitempty"`
 
 	// LocalPathPort is the port that the service is listening on for the given path.
-	LocalPathPort int `json:",omitempty"`
+	LocalPathPort int `json:",omitempty" alias:"local_path_port"`
 
 	// Protocol describes the upstream's service protocol.
 	// Valid values are "http" and "http2", defaults to "http"
