@@ -8,11 +8,13 @@ import (
 const (
 	errNoLeader                   = "No cluster leader"
 	errNoDCPath                   = "No path to datacenter"
+	errDCNotAvailable             = "Remote DC has no server currently reachable"
 	errNoServers                  = "No known Consul servers"
 	errNotReadyForConsistentReads = "Not ready to serve consistent reads"
 	errSegmentsNotSupported       = "Network segments are not supported in this version of Consul"
 	errRPCRateExceeded            = "RPC rate limit exceeded"
 	errServiceNotFound            = "Service not found: "
+	errQueryNotFound              = "Query not found"
 )
 
 var (
@@ -22,8 +24,18 @@ var (
 	ErrNotReadyForConsistentReads = errors.New(errNotReadyForConsistentReads)
 	ErrSegmentsNotSupported       = errors.New(errSegmentsNotSupported)
 	ErrRPCRateExceeded            = errors.New(errRPCRateExceeded)
+	ErrDCNotAvailable             = errors.New(errDCNotAvailable)
+	ErrQueryNotFound              = errors.New(errQueryNotFound)
 )
 
+func IsErrNoDCPath(err error) bool {
+	return err != nil && strings.Contains(err.Error(), errNoDCPath)
+}
+
+func IsErrQueryNotFound(err error) bool {
+	return err != nil && strings.Contains(err.Error(), errQueryNotFound)
+}
+
 func IsErrNoLeader(err error) bool {
 	return err != nil && strings.Contains(err.Error(), errNoLeader)
 }
