@@ -2,40 +2,42 @@ package agent
 
 import (
 	"bytes"
+	"context"
+	"crypto/ecdsa"
+	"crypto/elliptic"
+	"crypto/x509"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"math/rand"
 	"net/http/httptest"
-	"os"
 	"path/filepath"
 	"strconv"
-	"strings"
 	"testing"
 	"text/template"
 	"time"
 
 	metrics "github.com/armon/go-metrics"
+	"github.com/hashicorp/consul/sdk/testutil"
+	"github.com/hashicorp/errwrap"
 	"github.com/hashicorp/go-hclog"
 	uuid "github.com/hashicorp/go-uuid"
+	"github.com/stretchr/testify/require"
 
+	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/config"
 	"github.com/hashicorp/consul/agent/connect"
 	"github.com/hashicorp/consul/agent/consul"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/sdk/freeport"
-	"github.com/hashicorp/consul/sdk/testutil"
 	"github.com/hashicorp/consul/sdk/testutil/retry"
+	"github.com/hashicorp/consul/tlsutil"
 )
 
 func init() {
 	rand.Seed(time.Now().UnixNano()) // seed random number generator
 }
 
-// TempDir defines the base dir for temporary directories.
-var TempDir = os.TempDir()
-
 // TestAgent encapsulates an Agent with a default configuration and
 // startup procedure suitable for testing. It panics if there are errors
 // during creation or startup instead of returning errors. It manages a
@@ -53,22 +55,16 @@ type TestAgent struct {
 	// when Shutdown() is called.
 	Config *config.RuntimeConfig
 
-	// returnPortsFn will put the ports claimed for the test back into the
-	// general freeport pool
-	returnPortsFn func()
-
 	// LogOutput is the sink for the logs. If nil, logs are written
 	// to os.Stderr.
 	LogOutput io.Writer
 
-	// DataDir is the data directory which is used when Config.DataDir
-	// is not set. It is created automatically and removed when
-	// Shutdown() is called.
+	// DataDir may be set to a directory which exists. If is it not set,
+	// TestAgent.Start will create one and set DataDir to the directory path.
+	// In all cases the agent will be configured to use this path as the data directory,
+	// and the directory will be removed once the test ends.
 	DataDir string
 
-	// Key is the optional encryption key for the LAN and WAN keyring.
-	Key string
-
 	// UseTLS, if true, will disable the HTTP port and enable the HTTPS
 	// one.
 	UseTLS bool
@@ -81,38 +77,32 @@ type TestAgent struct {
 	// It is valid after Start().
 	srv *HTTPServer
 
+	// overrides is an hcl config source to use to override otherwise
+	// non-user settable configurations
+	Overrides string
+
 	// Agent is the embedded consul agent.
 	// It is valid after Start().
 	*Agent
 }
 
-// NewTestAgent returns a started agent with the given name and
-// configuration. It fails the test if the Agent could not be started. The
-// caller should call Shutdown() to stop the agent and remove temporary
-// directories.
-func NewTestAgent(t *testing.T, name string, hcl string) *TestAgent {
-	return NewTestAgentWithFields(t, true, TestAgent{Name: name, HCL: hcl})
+// NewTestAgent returns a started agent with the given configuration. It fails
+// the test if the Agent could not be started.
+// The caller is responsible for calling Shutdown() to stop the agent and remove
+// temporary directories.
+func NewTestAgent(t *testing.T, hcl string) *TestAgent {
+	return StartTestAgent(t, TestAgent{HCL: hcl})
 }
 
-// NewTestAgentWithFields takes a TestAgent struct with any number of fields set,
-// and a boolean 'start', which indicates whether or not the TestAgent should
-// be started. If no LogOutput is set, it will automatically be set to
-// testutil.TestWriter(t). Name will default to t.Name() if not specified.
-func NewTestAgentWithFields(t *testing.T, start bool, ta TestAgent) *TestAgent {
-	// copy values
-	a := ta
-	if a.Name == "" {
-		a.Name = t.Name()
-	}
-	if a.LogOutput == nil {
-		a.LogOutput = testutil.TestWriter(t)
-	}
-	if !start {
-		return nil
-	}
-
+// StartTestAgent and wait for it to become available. If the agent fails to
+// start the test will be marked failed and execution will stop.
+//
+// The caller is responsible for calling Shutdown() to stop the agent and remove
+// temporary directories.
+func StartTestAgent(t *testing.T, a TestAgent) *TestAgent {
+	t.Helper()
 	retry.RunWith(retry.ThreeTimes(), t, func(r *retry.R) {
-		if err := a.Start(); err != nil {
+		if err := a.Start(t); err != nil {
 			r.Fatal(err)
 		}
 	})
@@ -120,105 +110,102 @@ func NewTestAgentWithFields(t *testing.T, start bool, ta TestAgent) *TestAgent {
 	return &a
 }
 
+func TestConfigHCL(nodeID string) string {
+	return fmt.Sprintf(`
+		bind_addr = "127.0.0.1"
+		advertise_addr = "127.0.0.1"
+		datacenter = "dc1"
+		bootstrap = true
+		server = true
+		node_id = "%[1]s"
+		node_name = "Node-%[1]s"
+		connect {
+			enabled = true
+			ca_config {
+				cluster_id = "%[2]s"
+			}
+		}
+		performance {
+			raft_multiplier = 1
+		}`, nodeID, connect.TestClusterID,
+	)
+}
+
 // Start starts a test agent. It returns an error if the agent could not be started.
 // If no error is returned, the caller must call Shutdown() when finished.
-func (a *TestAgent) Start() (err error) {
+func (a *TestAgent) Start(t *testing.T) (err error) {
+	t.Helper()
 	if a.Agent != nil {
 		return fmt.Errorf("TestAgent already started")
 	}
 
-	var cleanupTmpDir = func() {
-		// Clean out the data dir if we are responsible for it before we
-		// try again, since the old ports may have gotten written to
-		// the data dir, such as in the Raft configuration.
-		if a.DataDir != "" {
-			if err := os.RemoveAll(a.DataDir); err != nil {
-				fmt.Printf("%s Error resetting data dir: %s", a.Name, err)
-			}
-		}
+	name := a.Name
+	if name == "" {
+		name = "TestAgent"
 	}
 
-	var hclDataDir string
 	if a.DataDir == "" {
-		name := "agent"
-		if a.Name != "" {
-			name = a.Name + "-agent"
-		}
-		name = strings.Replace(name, "/", "_", -1)
-		d, err := ioutil.TempDir(TempDir, name)
-		if err != nil {
-			return fmt.Errorf("Error creating data dir %s: %s", filepath.Join(TempDir, name), err)
-		}
-		// Convert windows style path to posix style path
-		// to avoid illegal char escape error when hcl
-		// parsing.
-		d = filepath.ToSlash(d)
-		hclDataDir = `data_dir = "` + d + `"`
+		dirname := name + "-agent"
+		a.DataDir = testutil.TempDir(t, dirname)
 	}
+	// Convert windows style path to posix style path to avoid illegal char escape
+	// error when hcl parsing.
+	d := filepath.ToSlash(a.DataDir)
+	hclDataDir := fmt.Sprintf(`data_dir = "%s"`, d)
 
 	logOutput := a.LogOutput
 	if logOutput == nil {
-		logOutput = os.Stderr
+		logOutput = testutil.NewLogBuffer(t)
 	}
 
 	logger := hclog.NewInterceptLogger(&hclog.LoggerOptions{
-		Name:   a.Name,
-		Level:  hclog.Debug,
-		Output: logOutput,
+		Level:      hclog.Debug,
+		Output:     logOutput,
+		TimeFormat: "04:05.000",
+		Name:       name,
 	})
 
 	portsConfig, returnPortsFn := randomPortsSource(a.UseTLS)
-	a.returnPortsFn = returnPortsFn
-	a.Config = TestConfig(logger,
-		portsConfig,
-		config.Source{Name: a.Name, Format: "hcl", Data: a.HCL},
-		config.Source{Name: a.Name + ".data_dir", Format: "hcl", Data: hclDataDir},
-	)
+	t.Cleanup(returnPortsFn)
 
-	defer func() {
-		if err != nil && a.returnPortsFn != nil {
-			a.returnPortsFn()
-			a.returnPortsFn = nil
+	// Create NodeID outside the closure, so that it does not change
+	testHCLConfig := TestConfigHCL(NodeID())
+	loader := func(source config.Source) (*config.RuntimeConfig, []string, error) {
+		opts := config.BuilderOpts{
+			HCL: []string{testHCLConfig, portsConfig, a.HCL, hclDataDir},
 		}
-	}()
-
-	// write the keyring
-	if a.Key != "" {
-		writeKey := func(key, filename string) error {
-			path := filepath.Join(a.Config.DataDir, filename)
-			if err := initKeyring(path, key); err != nil {
-				cleanupTmpDir()
-				return fmt.Errorf("Error creating keyring %s: %s", path, err)
-			}
-			return nil
+		overrides := []config.Source{
+			config.FileSource{
+				Name:   "test-overrides",
+				Format: "hcl",
+				Data:   a.Overrides},
+			config.DefaultConsulSource(),
+			config.DevConsulSource(),
 		}
-		if err = writeKey(a.Key, SerfLANKeyring); err != nil {
-			cleanupTmpDir()
-			return err
-		}
-		if err = writeKey(a.Key, SerfWANKeyring); err != nil {
-			cleanupTmpDir()
-			return err
+		cfg, warnings, err := config.Load(opts, source, overrides...)
+		if cfg != nil {
+			cfg.Telemetry.Disable = true
 		}
+		return cfg, warnings, err
 	}
+	bd, err := NewBaseDeps(loader, logOutput)
+	require.NoError(t, err)
 
-	agent, err := New(a.Config, logger)
+	bd.Logger = logger
+	bd.MetricsHandler = metrics.NewInmemSink(1*time.Second, time.Minute)
+	a.Config = bd.RuntimeConfig
+
+	agent, err := New(bd)
 	if err != nil {
-		cleanupTmpDir()
 		return fmt.Errorf("Error creating agent: %s", err)
 	}
 
-	agent.LogOutput = logOutput
-	agent.MemSink = metrics.NewInmemSink(1*time.Second, time.Minute)
-
 	id := string(a.Config.NodeID)
-
-	if err := agent.Start(); err != nil {
-		cleanupTmpDir()
+	if err := agent.Start(context.Background()); err != nil {
 		agent.ShutdownAgent()
 		agent.ShutdownEndpoints()
 
-		return fmt.Errorf("%s %s Error starting agent: %s", id, a.Name, err)
+		return fmt.Errorf("%s %s Error starting agent: %s", id, name, err)
 	}
 
 	a.Agent = agent
@@ -227,9 +214,9 @@ func (a *TestAgent) Start() (err error) {
 	a.Agent.StartSync()
 
 	if err := a.waitForUp(); err != nil {
-		cleanupTmpDir()
 		a.Shutdown()
-		return err
+		t.Logf("Error while waiting for test agent to start: %v", err)
+		return errwrap.Wrapf(name+": {{err}}", err)
 	}
 
 	a.dns = a.dnsServers[0]
@@ -247,7 +234,7 @@ func (a *TestAgent) waitForUp() error {
 	var out structs.IndexedNodes
 	for ; !time.Now().After(deadline); time.Sleep(timer.Wait) {
 		if len(a.httpServers) == 0 {
-			retErr = fmt.Errorf("%s: waiting for server", a.Name)
+			retErr = fmt.Errorf("waiting for server")
 			continue // fail, try again
 		}
 		if a.Config.Bootstrap && a.Config.ServerMode {
@@ -264,11 +251,11 @@ func (a *TestAgent) waitForUp() error {
 				continue // fail, try again
 			}
 			if !out.QueryMeta.KnownLeader {
-				retErr = fmt.Errorf("%s: No leader", a.Name)
+				retErr = fmt.Errorf("No leader")
 				continue // fail, try again
 			}
 			if out.Index == 0 {
-				retErr = fmt.Errorf("%s: Consul index is 0", a.Name)
+				retErr = fmt.Errorf("Consul index is 0")
 				continue // fail, try again
 			}
 			return nil // success
@@ -276,8 +263,12 @@ func (a *TestAgent) waitForUp() error {
 			req := httptest.NewRequest("GET", "/v1/agent/self", nil)
 			resp := httptest.NewRecorder()
 			_, err := a.httpServers[0].AgentSelf(resp, req)
-			if err != nil || resp.Code != 200 {
-				retErr = fmt.Errorf("%s: failed OK response: %v", a.Name, err)
+			if acl.IsErrPermissionDenied(err) || resp.Code == 403 {
+				// permission denied is enough to show that the client is
+				// connected to the servers as it would get a 503 if
+				// it couldn't connect to them.
+			} else if err != nil && resp.Code != 200 {
+				retErr = fmt.Errorf("failed OK response: %v", err)
 				continue
 			}
 			return nil // success
@@ -305,14 +296,6 @@ func (a *TestAgent) Shutdown() error {
 		return nil
 	}
 
-	// Return ports last of all
-	defer func() {
-		if a.returnPortsFn != nil {
-			a.returnPortsFn()
-			a.returnPortsFn = nil
-		}
-	}()
-
 	// shutdown agent before endpoints
 	defer a.Agent.ShutdownEndpoints()
 	if err := a.Agent.ShutdownAgent(); err != nil {
@@ -361,8 +344,11 @@ func (a *TestAgent) DNSDisableCompression(b bool) {
 	}
 }
 
+// FIXME: this should t.Fatal on error, not panic.
+// TODO: rename to newConsulConfig
+// TODO: remove TestAgent receiver, accept a.Agent.config as an arg
 func (a *TestAgent) consulConfig() *consul.Config {
-	c, err := a.Agent.consulConfig()
+	c, err := newConsulConfig(a.Agent.config, a.Agent.logger)
 	if err != nil {
 		panic(err)
 	}
@@ -377,8 +363,8 @@ func (a *TestAgent) consulConfig() *consul.Config {
 // chance of port conflicts for concurrently executed test binaries.
 // Instead of relying on one set of ports to be sufficient we retry
 // starting the agent with different ports on port conflict.
-func randomPortsSource(tls bool) (src config.Source, returnPortsFn func()) {
-	ports := freeport.MustTake(6)
+func randomPortsSource(tls bool) (data string, returnPortsFn func()) {
+	ports := freeport.MustTake(7)
 
 	var http, https int
 	if tls {
@@ -389,20 +375,17 @@ func randomPortsSource(tls bool) (src config.Source, returnPortsFn func()) {
 		https = -1
 	}
 
-	return config.Source{
-		Name:   "ports",
-		Format: "hcl",
-		Data: `
-			ports = {
-				dns = ` + strconv.Itoa(ports[0]) + `
-				http = ` + strconv.Itoa(http) + `
-				https = ` + strconv.Itoa(https) + `
-				serf_lan = ` + strconv.Itoa(ports[3]) + `
-				serf_wan = ` + strconv.Itoa(ports[4]) + `
-				server = ` + strconv.Itoa(ports[5]) + `
-			}
-		`,
-	}, func() { freeport.Return(ports) }
+	return `
+		ports = {
+			dns = ` + strconv.Itoa(ports[0]) + `
+			http = ` + strconv.Itoa(http) + `
+			https = ` + strconv.Itoa(https) + `
+			serf_lan = ` + strconv.Itoa(ports[3]) + `
+			serf_wan = ` + strconv.Itoa(ports[4]) + `
+			server = ` + strconv.Itoa(ports[5]) + `
+			grpc = ` + strconv.Itoa(ports[6]) + `
+		}
+	`, func() { freeport.Return(ports) }
 }
 
 func NodeID() string {
@@ -417,7 +400,7 @@ func NodeID() string {
 // agent.
 func TestConfig(logger hclog.Logger, sources ...config.Source) *config.RuntimeConfig {
 	nodeID := NodeID()
-	testsrc := config.Source{
+	testsrc := config.FileSource{
 		Name:   "test",
 		Format: "hcl",
 		Data: `
@@ -440,7 +423,7 @@ func TestConfig(logger hclog.Logger, sources ...config.Source) *config.RuntimeCo
 		`,
 	}
 
-	b, err := config.NewBuilder(config.Flags{})
+	b, err := config.NewBuilder(config.BuilderOpts{})
 	if err != nil {
 		panic("NewBuilder failed: " + err.Error())
 	}
@@ -474,7 +457,6 @@ func TestACLConfig() string {
 		acl_master_token = "root"
 		acl_agent_token = "root"
 		acl_agent_master_token = "towel"
-		acl_enforce_version_8 = true
 	`
 }
 
@@ -523,34 +505,34 @@ func TestACLConfigNew() string {
 }
 
 var aclConfigTpl = template.Must(template.New("ACL Config").Parse(`
-   {{if ne .PrimaryDatacenter ""}}
+   {{- if ne .PrimaryDatacenter "" -}}
 	primary_datacenter = "{{ .PrimaryDatacenter }}"
-	{{end}}
+	{{end -}}
 	acl {
 		enabled = true
-		{{if ne .DefaultPolicy ""}}
+		{{- if ne .DefaultPolicy ""}}
 		default_policy = "{{ .DefaultPolicy }}"
-		{{end}}
+		{{- end}}
 		enable_token_replication = {{printf "%t" .EnableTokenReplication }}
-		{{if .HasConfiguredTokens }}
+		{{- if .HasConfiguredTokens}}
 		tokens {
-			{{if ne .MasterToken ""}}
+			{{- if ne .MasterToken ""}}
 			master = "{{ .MasterToken }}"
-			{{end}}
-			{{if ne .AgentToken ""}}
+			{{- end}}
+			{{- if ne .AgentToken ""}}
 			agent = "{{ .AgentToken }}"
-			{{end}}
-			{{if ne .AgentMasterToken "" }}
+			{{- end}}
+			{{- if ne .AgentMasterToken "" }}
 			agent_master = "{{ .AgentMasterToken }}"
-			{{end}}
-			{{if ne .DefaultToken "" }}
+			{{- end}}
+			{{- if ne .DefaultToken "" }}
 			default = "{{ .DefaultToken }}"
-			{{end}}
-			{{if ne .ReplicationToken "" }}
+			{{- end}}
+			{{- if ne .ReplicationToken ""  }}
 			replication = "{{ .ReplicationToken }}"
-			{{end}}
+			{{- end}}
 		}
-		{{end}}
+		{{- end}}
 	}
 `))
 
@@ -569,3 +551,43 @@ func TestACLConfigWithParams(params *TestACLConfigParams) string {
 
 	return buf.String()
 }
+
+// testTLSCertificates Generates a TLS CA and server key/cert and returns them
+// in PEM encoded form.
+func testTLSCertificates(serverName string) (cert string, key string, cacert string, err error) {
+	// generate CA
+	serial, err := tlsutil.GenerateSerialNumber()
+	if err != nil {
+		return "", "", "", err
+	}
+	signer, err := ecdsa.GenerateKey(elliptic.P256(), rand.New(rand.NewSource(99)))
+	if err != nil {
+		return "", "", "", err
+	}
+	ca, err := tlsutil.GenerateCA(signer, serial, 365, nil)
+	if err != nil {
+		return "", "", "", err
+	}
+
+	// generate leaf
+	serial, err = tlsutil.GenerateSerialNumber()
+	if err != nil {
+		return "", "", "", err
+	}
+
+	cert, privateKey, err := tlsutil.GenerateCert(
+		signer,
+		ca,
+		serial,
+		"Test Cert Name",
+		365,
+		[]string{serverName},
+		nil,
+		[]x509.ExtKeyUsage{x509.ExtKeyUsageServerAuth, x509.ExtKeyUsageClientAuth},
+	)
+	if err != nil {
+		return "", "", "", err
+	}
+
+	return cert, privateKey, ca, nil
+}
