@@ -6,6 +6,7 @@ import (
 	"sort"
 	"strings"
 
+	"github.com/hashicorp/consul/agent/config"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/api"
 )
@@ -15,17 +16,27 @@ import (
 // to extract this.
 const metaExternalSource = "external-source"
 
+type GatewayConfig struct {
+	Addresses []string `json:",omitempty"`
+	// internal to track uniqueness
+	addressesSet map[string]struct{}
+}
+
 // ServiceSummary is used to summarize a service
 type ServiceSummary struct {
 	Kind              structs.ServiceKind `json:",omitempty"`
 	Name              string
 	Tags              []string
 	Nodes             []string
+	InstanceCount     int
+	ProxyFor          []string            `json:",omitempty"`
+	proxyForSet       map[string]struct{} // internal to track uniqueness
 	ChecksPassing     int
 	ChecksWarning     int
 	ChecksCritical    int
 	ExternalSources   []string
 	externalSourceSet map[string]struct{} // internal to track uniqueness
+	GatewayConfig     GatewayConfig       `json:",omitempty"`
 
 	structs.EnterpriseMeta
 }
@@ -152,10 +163,46 @@ RPC:
 	}
 
 	// Generate the summary
-	return summarizeServices(out.Nodes), nil
+	// TODO (gateways) (freddy) Have Internal.ServiceDump return ServiceDump instead. Need to add bexpr filtering for type.
+	return summarizeServices(out.Nodes.ToServiceDump(), s.agent.config, args.Datacenter), nil
 }
 
-func summarizeServices(dump structs.CheckServiceNodes) []*ServiceSummary {
+// UIGatewayServices is used to query all the nodes for services associated with a gateway along with their gateway config
+func (s *HTTPServer) UIGatewayServicesNodes(resp http.ResponseWriter, req *http.Request) (interface{}, error) {
+	// Parse arguments
+	args := structs.ServiceSpecificRequest{}
+	if err := s.parseEntMetaNoWildcard(req, &args.EnterpriseMeta); err != nil {
+		return nil, err
+	}
+	if done := s.parse(resp, req, &args.Datacenter, &args.QueryOptions); done {
+		return nil, nil
+	}
+
+	// Pull out the service name
+	args.ServiceName = strings.TrimPrefix(req.URL.Path, "/v1/internal/ui/gateway-services-nodes/")
+	if args.ServiceName == "" {
+		resp.WriteHeader(http.StatusBadRequest)
+		fmt.Fprint(resp, "Missing gateway name")
+		return nil, nil
+	}
+
+	// Make the RPC request
+	var out structs.IndexedServiceDump
+	defer setMeta(resp, &out.QueryMeta)
+RPC:
+	if err := s.agent.RPC("Internal.GatewayServiceDump", &args, &out); err != nil {
+		// Retry the request allowing stale data if no leader
+		if strings.Contains(err.Error(), structs.ErrNoLeader.Error()) && !args.AllowStale {
+			args.AllowStale = true
+			goto RPC
+		}
+		return nil, err
+	}
+
+	return summarizeServices(out.Dump, s.agent.config, args.Datacenter), nil
+}
+
+func summarizeServices(dump structs.ServiceDump, cfg *config.RuntimeConfig, datacenter string) []*ServiceSummary {
 	// Collect the summary information
 	var services []structs.ServiceID
 	summary := make(map[structs.ServiceID]*ServiceSummary)
@@ -165,6 +212,9 @@ func summarizeServices(dump structs.CheckServiceNodes) []*ServiceSummary {
 			serv = &ServiceSummary{
 				Name:           service.ID,
 				EnterpriseMeta: service.EnterpriseMeta,
+				// the other code will increment this unconditionally so we
+				// shouldn't initialize it to 1
+				InstanceCount: 0,
 			}
 			summary[service] = serv
 			services = append(services, service)
@@ -172,13 +222,33 @@ func summarizeServices(dump structs.CheckServiceNodes) []*ServiceSummary {
 		return serv
 	}
 
-	var sid structs.ServiceID
 	for _, csn := range dump {
-		svc := csn.Service
-		sid.Init(svc.Service, &svc.EnterpriseMeta)
+		if csn.GatewayService != nil {
+			gwsvc := csn.GatewayService
+			sum := getService(gwsvc.Service.ToServiceID())
+			modifySummaryForGatewayService(cfg, datacenter, sum, gwsvc)
+		}
+
+		// Will happen in cases where we only have the GatewayServices mapping
+		if csn.Service == nil {
+			continue
+		}
+		sid := structs.NewServiceID(csn.Service.Service, &csn.Service.EnterpriseMeta)
 		sum := getService(sid)
+
+		svc := csn.Service
 		sum.Nodes = append(sum.Nodes, csn.Node.Node)
 		sum.Kind = svc.Kind
+		sum.InstanceCount += 1
+		if svc.Kind == structs.ServiceKindConnectProxy {
+			if _, ok := sum.proxyForSet[svc.Proxy.DestinationServiceName]; !ok {
+				if sum.proxyForSet == nil {
+					sum.proxyForSet = make(map[string]struct{})
+				}
+				sum.proxyForSet[svc.Proxy.DestinationServiceName] = struct{}{}
+				sum.ProxyFor = append(sum.ProxyFor, svc.Proxy.DestinationServiceName)
+			}
+		}
 		for _, tag := range svc.Tags {
 			found := false
 			for _, existing := range sum.Tags {
@@ -226,10 +296,47 @@ func summarizeServices(dump structs.CheckServiceNodes) []*ServiceSummary {
 	})
 	output := make([]*ServiceSummary, len(summary))
 	for idx, service := range services {
-		// Sort the nodes
+		// Sort the nodes and tags
 		sum := summary[service]
 		sort.Strings(sum.Nodes)
+		sort.Strings(sum.Tags)
 		output[idx] = sum
 	}
 	return output
 }
+
+func modifySummaryForGatewayService(
+	cfg *config.RuntimeConfig,
+	datacenter string,
+	sum *ServiceSummary,
+	gwsvc *structs.GatewayService,
+) {
+	var dnsAddresses []string
+	for _, domain := range []string{cfg.DNSDomain, cfg.DNSAltDomain} {
+		// If the domain is empty, do not use it to construct a valid DNS
+		// address
+		if domain == "" {
+			continue
+		}
+		dnsAddresses = append(dnsAddresses, serviceIngressDNSName(
+			gwsvc.Service.Name,
+			datacenter,
+			domain,
+			&gwsvc.Service.EnterpriseMeta,
+		))
+	}
+
+	for _, addr := range gwsvc.Addresses(dnsAddresses) {
+		// check for duplicates, a service will have a ServiceInfo struct for
+		// every instance that is registered.
+		if _, ok := sum.GatewayConfig.addressesSet[addr]; !ok {
+			if sum.GatewayConfig.addressesSet == nil {
+				sum.GatewayConfig.addressesSet = make(map[string]struct{})
+			}
+			sum.GatewayConfig.addressesSet[addr] = struct{}{}
+			sum.GatewayConfig.Addresses = append(
+				sum.GatewayConfig.Addresses, addr,
+			)
+		}
+	}
+}
