@@ -48,13 +48,13 @@ func TestValidateUserEventParams(t *testing.T) {
 
 func TestShouldProcessUserEvent(t *testing.T) {
 	t.Parallel()
-	a := NewTestAgent(t, t.Name(), "")
+	a := NewTestAgent(t, "")
 	defer a.Shutdown()
 
 	srv1 := &structs.NodeService{
 		ID:      "mysql",
 		Service: "mysql",
-		Tags:    []string{"test", "foo", "bar", "master"},
+		Tags:    []string{"test", "foo", "bar", "primary"},
 		Port:    5000,
 	}
 	a.State.AddService(srv1, "")
@@ -99,7 +99,7 @@ func TestShouldProcessUserEvent(t *testing.T) {
 	// Bad tag name
 	p = &UserEvent{
 		ServiceFilter: ".*sql",
-		TagFilter:     "slave",
+		TagFilter:     "replica",
 	}
 	if a.shouldProcessUserEvent(p) {
 		t.Fatalf("bad")
@@ -108,7 +108,7 @@ func TestShouldProcessUserEvent(t *testing.T) {
 	// Good service name
 	p = &UserEvent{
 		ServiceFilter: ".*sql",
-		TagFilter:     "master",
+		TagFilter:     "primary",
 	}
 	if !a.shouldProcessUserEvent(p) {
 		t.Fatalf("bad")
@@ -117,7 +117,7 @@ func TestShouldProcessUserEvent(t *testing.T) {
 
 func TestIngestUserEvent(t *testing.T) {
 	t.Parallel()
-	a := NewTestAgent(t, t.Name(), "")
+	a := NewTestAgent(t, "")
 	defer a.Shutdown()
 
 	for i := 0; i < 512; i++ {
@@ -148,13 +148,13 @@ func TestIngestUserEvent(t *testing.T) {
 
 func TestFireReceiveEvent(t *testing.T) {
 	t.Parallel()
-	a := NewTestAgent(t, t.Name(), "")
+	a := NewTestAgent(t, "")
 	defer a.Shutdown()
 
 	srv1 := &structs.NodeService{
 		ID:      "mysql",
 		Service: "mysql",
-		Tags:    []string{"test", "foo", "bar", "master"},
+		Tags:    []string{"test", "foo", "bar", "primary"},
 		Port:    5000,
 	}
 	a.State.AddService(srv1, "")
@@ -184,7 +184,7 @@ func TestFireReceiveEvent(t *testing.T) {
 
 func TestUserEventToken(t *testing.T) {
 	t.Parallel()
-	a := NewTestAgent(t, t.Name(), TestACLConfig()+`
+	a := NewTestAgent(t, TestACLConfig()+`
 		acl_default_policy = "deny"
 	`)
 	defer a.Shutdown()
