@@ -12,26 +12,31 @@ import (
 	envoycore "github.com/envoyproxy/go-control-plane/envoy/api/v2/core"
 	envoyendpoint "github.com/envoyproxy/go-control-plane/envoy/api/v2/endpoint"
 	envoytype "github.com/envoyproxy/go-control-plane/envoy/type"
-	"github.com/gogo/protobuf/jsonpb"
-	"github.com/gogo/protobuf/proto"
-	"github.com/gogo/protobuf/types"
-
+	"github.com/golang/protobuf/jsonpb"
+	"github.com/golang/protobuf/proto"
+	"github.com/golang/protobuf/ptypes"
+	"github.com/golang/protobuf/ptypes/any"
 	"github.com/hashicorp/consul/agent/connect"
 	"github.com/hashicorp/consul/agent/proxycfg"
 	"github.com/hashicorp/consul/agent/structs"
+	"github.com/hashicorp/consul/logging"
 )
 
 // clustersFromSnapshot returns the xDS API representation of the "clusters" in the snapshot.
-func (s *Server) clustersFromSnapshot(cfgSnap *proxycfg.ConfigSnapshot, token string) ([]proto.Message, error) {
+func (s *Server) clustersFromSnapshot(_ connectionInfo, cfgSnap *proxycfg.ConfigSnapshot) ([]proto.Message, error) {
 	if cfgSnap == nil {
 		return nil, errors.New("nil config given")
 	}
 
 	switch cfgSnap.Kind {
 	case structs.ServiceKindConnectProxy:
-		return s.clustersFromSnapshotConnectProxy(cfgSnap, token)
+		return s.clustersFromSnapshotConnectProxy(cfgSnap)
+	case structs.ServiceKindTerminatingGateway:
+		return s.makeGatewayServiceClusters(cfgSnap)
 	case structs.ServiceKindMeshGateway:
-		return s.clustersFromSnapshotMeshGateway(cfgSnap, token)
+		return s.clustersFromSnapshotMeshGateway(cfgSnap)
+	case structs.ServiceKindIngressGateway:
+		return s.clustersFromSnapshotIngressGateway(cfgSnap)
 	default:
 		return nil, fmt.Errorf("Invalid service kind: %v", cfgSnap.Kind)
 	}
@@ -39,7 +44,7 @@ func (s *Server) clustersFromSnapshot(cfgSnap *proxycfg.ConfigSnapshot, token st
 
 // clustersFromSnapshot returns the xDS API representation of the "clusters"
 // (upstreams) in the snapshot.
-func (s *Server) clustersFromSnapshotConnectProxy(cfgSnap *proxycfg.ConfigSnapshot, token string) ([]proto.Message, error) {
+func (s *Server) clustersFromSnapshotConnectProxy(cfgSnap *proxycfg.ConfigSnapshot) ([]proto.Message, error) {
 	// TODO(rb): this sizing is a low bound.
 	clusters := make([]proto.Message, 0, len(cfgSnap.Proxy.Upstreams)+1)
 
@@ -63,7 +68,13 @@ func (s *Server) clustersFromSnapshotConnectProxy(cfgSnap *proxycfg.ConfigSnapsh
 
 		} else {
 			chain := cfgSnap.ConnectProxy.DiscoveryChain[id]
-			upstreamClusters, err := s.makeUpstreamClustersForDiscoveryChain(u, chain, cfgSnap)
+			chainEndpoints, ok := cfgSnap.ConnectProxy.WatchedUpstreamEndpoints[id]
+			if !ok {
+				// this should not happen
+				return nil, fmt.Errorf("no endpoint map for upstream %q", id)
+			}
+
+			upstreamClusters, err := s.makeUpstreamClustersForDiscoveryChain(u, chain, chainEndpoints, cfgSnap)
 			if err != nil {
 				return nil, err
 			}
@@ -112,54 +123,172 @@ func makeExposeClusterName(destinationPort int) string {
 // clustersFromSnapshotMeshGateway returns the xDS API representation of the "clusters"
 // for a mesh gateway. This will include 1 cluster per remote datacenter as well as
 // 1 cluster for each service subset.
-func (s *Server) clustersFromSnapshotMeshGateway(cfgSnap *proxycfg.ConfigSnapshot, token string) ([]proto.Message, error) {
+func (s *Server) clustersFromSnapshotMeshGateway(cfgSnap *proxycfg.ConfigSnapshot) ([]proto.Message, error) {
+	datacenters := cfgSnap.MeshGateway.Datacenters()
+
 	// 1 cluster per remote dc + 1 cluster per local service (this is a lower bound - all subset specific clusters will be appended)
-	clusters := make([]proto.Message, 0, len(cfgSnap.MeshGateway.GatewayGroups)+len(cfgSnap.MeshGateway.ServiceGroups))
+	clusters := make([]proto.Message, 0, len(datacenters)+len(cfgSnap.MeshGateway.ServiceGroups))
 
 	// generate the remote dc clusters
-	for dc, _ := range cfgSnap.MeshGateway.GatewayGroups {
-		clusterName := connect.DatacenterSNI(dc, cfgSnap.Roots.TrustDomain)
-
-		cluster, err := s.makeMeshGatewayCluster(clusterName, cfgSnap)
-		if err != nil {
-			return nil, err
+	for _, dc := range datacenters {
+		if dc == cfgSnap.Datacenter {
+			continue // skip local
 		}
+
+		opts := gatewayClusterOpts{
+			name:              connect.DatacenterSNI(dc, cfgSnap.Roots.TrustDomain),
+			hostnameEndpoints: cfgSnap.MeshGateway.HostnameDatacenters[dc],
+			isRemote:          dc != cfgSnap.Datacenter,
+		}
+		cluster := s.makeGatewayCluster(cfgSnap, opts)
 		clusters = append(clusters, cluster)
 	}
 
-	// generate the per-service clusters
-	for svc, _ := range cfgSnap.MeshGateway.ServiceGroups {
-		clusterName := connect.ServiceSNI(svc.ID, "", svc.NamespaceOrDefault(), cfgSnap.Datacenter, cfgSnap.Roots.TrustDomain)
-		resolver, hasResolver := cfgSnap.MeshGateway.ServiceResolvers[svc]
+	if cfgSnap.ServiceMeta[structs.MetaWANFederationKey] == "1" && cfgSnap.ServerSNIFn != nil {
+		// Add all of the remote wildcard datacenter mappings for servers.
+		for _, dc := range datacenters {
+			hostnameEndpoints := cfgSnap.MeshGateway.HostnameDatacenters[dc]
 
-		// Create the cluster for default/unnamed services
-		var cluster *envoy.Cluster
-		var err error
-		if hasResolver {
-			cluster, err = s.makeMeshGatewayClusterWithConnectTimeout(clusterName, cfgSnap, resolver.ConnectTimeout)
-		} else {
-			cluster, err = s.makeMeshGatewayCluster(clusterName, cfgSnap)
-		}
-		if err != nil {
-			return nil, err
-		}
-		clusters = append(clusters, cluster)
-
-		// if there is a service-resolver for this service then also setup subset clusters for it
-		if hasResolver {
-			// generate 1 cluster for each service subset
-			for subsetName := range resolver.Subsets {
-				clusterName := connect.ServiceSNI(svc.ID, subsetName, svc.NamespaceOrDefault(), cfgSnap.Datacenter, cfgSnap.Roots.TrustDomain)
-
-				cluster, err := s.makeMeshGatewayClusterWithConnectTimeout(clusterName, cfgSnap, resolver.ConnectTimeout)
-				if err != nil {
-					return nil, err
-				}
-				clusters = append(clusters, cluster)
+			// If the DC is our current DC then this cluster is for traffic from a remote DC to a local server.
+			// HostnameDatacenters is populated with gateway addresses, so it does not apply here.
+			if dc == cfgSnap.Datacenter {
+				hostnameEndpoints = nil
 			}
+			opts := gatewayClusterOpts{
+				name:              cfgSnap.ServerSNIFn(dc, ""),
+				hostnameEndpoints: hostnameEndpoints,
+				isRemote:          dc != cfgSnap.Datacenter,
+			}
+			cluster := s.makeGatewayCluster(cfgSnap, opts)
+			clusters = append(clusters, cluster)
+		}
+
+		// And for the current datacenter, send all flavors appropriately.
+		for _, srv := range cfgSnap.MeshGateway.ConsulServers {
+			opts := gatewayClusterOpts{
+				name: cfgSnap.ServerSNIFn(cfgSnap.Datacenter, srv.Node.Node),
+			}
+			cluster := s.makeGatewayCluster(cfgSnap, opts)
+			clusters = append(clusters, cluster)
 		}
 	}
 
+	// generate the per-service/subset clusters
+	c, err := s.makeGatewayServiceClusters(cfgSnap)
+	if err != nil {
+		return nil, err
+	}
+	clusters = append(clusters, c...)
+
+	return clusters, nil
+}
+
+func (s *Server) makeGatewayServiceClusters(cfgSnap *proxycfg.ConfigSnapshot) ([]proto.Message, error) {
+	var services map[structs.ServiceName]structs.CheckServiceNodes
+	var resolvers map[structs.ServiceName]*structs.ServiceResolverConfigEntry
+	var hostnameEndpoints structs.CheckServiceNodes
+
+	switch cfgSnap.Kind {
+	case structs.ServiceKindTerminatingGateway:
+		services = cfgSnap.TerminatingGateway.ServiceGroups
+		resolvers = cfgSnap.TerminatingGateway.ServiceResolvers
+	case structs.ServiceKindMeshGateway:
+		services = cfgSnap.MeshGateway.ServiceGroups
+		resolvers = cfgSnap.MeshGateway.ServiceResolvers
+	default:
+		return nil, fmt.Errorf("unsupported gateway kind %q", cfgSnap.Kind)
+	}
+
+	clusters := make([]proto.Message, 0, len(services))
+
+	for svc, _ := range services {
+		clusterName := connect.ServiceSNI(svc.Name, "", svc.NamespaceOrDefault(), cfgSnap.Datacenter, cfgSnap.Roots.TrustDomain)
+		resolver, hasResolver := resolvers[svc]
+
+		if !hasResolver {
+			// Use a zero value resolver with no timeout and no subsets
+			resolver = &structs.ServiceResolverConfigEntry{}
+		}
+
+		// When making service clusters we only pass endpoints with hostnames if the kind is a terminating gateway
+		// This is because the services a mesh gateway will route to are not external services and are not addressed by a hostname.
+		if cfgSnap.Kind == structs.ServiceKindTerminatingGateway {
+			hostnameEndpoints = cfgSnap.TerminatingGateway.HostnameServices[svc]
+		}
+
+		opts := gatewayClusterOpts{
+			name:              clusterName,
+			hostnameEndpoints: hostnameEndpoints,
+			connectTimeout:    resolver.ConnectTimeout,
+		}
+		cluster := s.makeGatewayCluster(cfgSnap, opts)
+
+		if cfgSnap.Kind == structs.ServiceKindTerminatingGateway {
+			injectTerminatingGatewayTLSContext(cfgSnap, cluster, svc)
+		}
+		clusters = append(clusters, cluster)
+
+		// If there is a service-resolver for this service then also setup a cluster for each subset
+		for name, subset := range resolver.Subsets {
+			subsetHostnameEndpoints, err := s.filterSubsetEndpoints(&subset, hostnameEndpoints)
+			if err != nil {
+				return nil, err
+			}
+
+			opts := gatewayClusterOpts{
+				name:              connect.ServiceSNI(svc.Name, name, svc.NamespaceOrDefault(), cfgSnap.Datacenter, cfgSnap.Roots.TrustDomain),
+				hostnameEndpoints: subsetHostnameEndpoints,
+				onlyPassing:       subset.OnlyPassing,
+				connectTimeout:    resolver.ConnectTimeout,
+			}
+			cluster := s.makeGatewayCluster(cfgSnap, opts)
+
+			if cfgSnap.Kind == structs.ServiceKindTerminatingGateway {
+				injectTerminatingGatewayTLSContext(cfgSnap, cluster, svc)
+			}
+			clusters = append(clusters, cluster)
+		}
+	}
+
+	return clusters, nil
+}
+
+func (s *Server) clustersFromSnapshotIngressGateway(cfgSnap *proxycfg.ConfigSnapshot) ([]proto.Message, error) {
+	var clusters []proto.Message
+	createdClusters := make(map[string]bool)
+	for _, upstreams := range cfgSnap.IngressGateway.Upstreams {
+		for _, u := range upstreams {
+			id := u.Identifier()
+
+			// If we've already created a cluster for this upstream, skip it. Multiple listeners may
+			// reference the same upstream, so we don't need to create duplicate clusters in that case.
+			if createdClusters[id] {
+				continue
+			}
+
+			chain, ok := cfgSnap.IngressGateway.DiscoveryChain[id]
+			if !ok {
+				// this should not happen
+				return nil, fmt.Errorf("no discovery chain for upstream %q", id)
+			}
+
+			chainEndpoints, ok := cfgSnap.IngressGateway.WatchedUpstreamEndpoints[id]
+			if !ok {
+				// this should not happen
+				return nil, fmt.Errorf("no endpoint map for upstream %q", id)
+			}
+
+			upstreamClusters, err := s.makeUpstreamClustersForDiscoveryChain(u, chain, chainEndpoints, cfgSnap)
+			if err != nil {
+				return nil, err
+			}
+
+			for _, c := range upstreamClusters {
+				clusters = append(clusters, c)
+			}
+			createdClusters[id] = true
+		}
+	}
 	return clusters, nil
 }
 
@@ -185,13 +314,13 @@ func (s *Server) makeAppCluster(cfgSnap *proxycfg.ConfigSnapshot, name, pathProt
 	}
 	c = &envoy.Cluster{
 		Name:                 name,
-		ConnectTimeout:       time.Duration(cfg.LocalConnectTimeoutMs) * time.Millisecond,
+		ConnectTimeout:       ptypes.DurationProto(time.Duration(cfg.LocalConnectTimeoutMs) * time.Millisecond),
 		ClusterDiscoveryType: &envoy.Cluster_Type{Type: envoy.Cluster_STATIC},
 		LoadAssignment: &envoy.ClusterLoadAssignment{
 			ClusterName: name,
-			Endpoints: []envoyendpoint.LocalityLbEndpoints{
+			Endpoints: []*envoyendpoint.LocalityLbEndpoints{
 				{
-					LbEndpoints: []envoyendpoint.LbEndpoint{
+					LbEndpoints: []*envoyendpoint.LbEndpoint{
 						makeEndpoint(name,
 							addr,
 							port),
@@ -200,7 +329,11 @@ func (s *Server) makeAppCluster(cfgSnap *proxycfg.ConfigSnapshot, name, pathProt
 			},
 		},
 	}
-	if cfg.Protocol == "http2" || cfg.Protocol == "grpc" || pathProtocol == "http2" {
+	protocol := pathProtocol
+	if protocol == "" {
+		protocol = cfg.Protocol
+	}
+	if protocol == "http2" || protocol == "grpc" {
 		c.Http2ProtocolOptions = &envoycore.Http2ProtocolOptions{}
 	}
 
@@ -234,7 +367,7 @@ func (s *Server) makeUpstreamClusterForPreparedQuery(upstream structs.Upstream,
 	if c == nil {
 		c = &envoy.Cluster{
 			Name:                 sni,
-			ConnectTimeout:       time.Duration(cfg.ConnectTimeoutMs) * time.Millisecond,
+			ConnectTimeout:       ptypes.DurationProto(time.Duration(cfg.ConnectTimeoutMs) * time.Millisecond),
 			ClusterDiscoveryType: &envoy.Cluster_Type{Type: envoy.Cluster_EDS},
 			EdsClusterConfig: &envoy.Cluster_EdsClusterConfig{
 				EdsConfig: &envoycore.ConfigSource{
@@ -246,8 +379,7 @@ func (s *Server) makeUpstreamClusterForPreparedQuery(upstream structs.Upstream,
 			CircuitBreakers: &envoycluster.CircuitBreakers{
 				Thresholds: makeThresholdsIfNeeded(cfg.Limits),
 			},
-			// Having an empty config enables outlier detection with default config.
-			OutlierDetection: &envoycluster.OutlierDetection{},
+			OutlierDetection: cfg.PassiveHealthCheck.AsOutlierDetection(),
 		}
 		if cfg.Protocol == "http2" || cfg.Protocol == "grpc" {
 			c.Http2ProtocolOptions = &envoycore.Http2ProtocolOptions{}
@@ -256,7 +388,7 @@ func (s *Server) makeUpstreamClusterForPreparedQuery(upstream structs.Upstream,
 
 	// Enable TLS upstream with the configured client certificate.
 	c.TlsContext = &envoyauth.UpstreamTlsContext{
-		CommonTlsContext: makeCommonTLSContext(cfgSnap),
+		CommonTlsContext: makeCommonTLSContextFromLeaf(cfgSnap, cfgSnap.Leaf()),
 		Sni:              sni,
 	}
 
@@ -266,6 +398,7 @@ func (s *Server) makeUpstreamClusterForPreparedQuery(upstream structs.Upstream,
 func (s *Server) makeUpstreamClustersForDiscoveryChain(
 	upstream structs.Upstream,
 	chain *structs.CompiledDiscoveryChain,
+	chainEndpoints map[string]structs.CheckServiceNodes,
 	cfgSnap *proxycfg.ConfigSnapshot,
 ) ([]*envoy.Cluster, error) {
 	if chain == nil {
@@ -296,15 +429,7 @@ func (s *Server) makeUpstreamClustersForDiscoveryChain(
 		}
 	}
 
-	id := upstream.Identifier()
-	chainEndpointMap, ok := cfgSnap.ConnectProxy.WatchedUpstreamEndpoints[id]
-	if !ok {
-		// this should not happen
-		return nil, fmt.Errorf("no endpoint map for upstream %q", id)
-	}
-
 	var out []*envoy.Cluster
-
 	for _, node := range chain.Nodes {
 		if node.Type != structs.DiscoveryGraphNodeTypeResolver {
 			continue
@@ -323,7 +448,7 @@ func (s *Server) makeUpstreamClustersForDiscoveryChain(
 		if failoverThroughMeshGateway {
 			actualTargetID := firstHealthyTarget(
 				chain.Targets,
-				chainEndpointMap,
+				chainEndpoints,
 				targetID,
 				failover.Targets,
 			)
@@ -338,7 +463,7 @@ func (s *Server) makeUpstreamClustersForDiscoveryChain(
 		c := &envoy.Cluster{
 			Name:                 clusterName,
 			AltStatName:          clusterName,
-			ConnectTimeout:       node.Resolver.ConnectTimeout,
+			ConnectTimeout:       ptypes.DurationProto(node.Resolver.ConnectTimeout),
 			ClusterDiscoveryType: &envoy.Cluster_Type{Type: envoy.Cluster_EDS},
 			CommonLbConfig: &envoy.Cluster_CommonLbConfig{
 				HealthyPanicThreshold: &envoytype.Percent{
@@ -355,8 +480,7 @@ func (s *Server) makeUpstreamClustersForDiscoveryChain(
 			CircuitBreakers: &envoycluster.CircuitBreakers{
 				Thresholds: makeThresholdsIfNeeded(cfg.Limits),
 			},
-			// Having an empty config enables outlier detection with default config.
-			OutlierDetection: &envoycluster.OutlierDetection{},
+			OutlierDetection: cfg.PassiveHealthCheck.AsOutlierDetection(),
 		}
 
 		proto := cfg.Protocol
@@ -374,7 +498,7 @@ func (s *Server) makeUpstreamClustersForDiscoveryChain(
 
 		// Enable TLS upstream with the configured client certificate.
 		c.TlsContext = &envoyauth.UpstreamTlsContext{
-			CommonTlsContext: makeCommonTLSContext(cfgSnap),
+			CommonTlsContext: makeCommonTLSContextFromLeaf(cfgSnap, cfgSnap.Leaf()),
 			Sni:              sni,
 		}
 
@@ -424,7 +548,7 @@ func makeClusterFromUserConfig(configJSON string) (*envoy.Cluster, error) {
 
 	if _, ok := jsonFields["@type"]; ok {
 		// Type field is present so decode it as a types.Any
-		var any types.Any
+		var any any.Any
 		err := jsonpb.UnmarshalString(configJSON, &any)
 		if err != nil {
 			return nil, err
@@ -442,40 +566,142 @@ func makeClusterFromUserConfig(configJSON string) (*envoy.Cluster, error) {
 	return &c, err
 }
 
-func (s *Server) makeMeshGatewayCluster(clusterName string, cfgSnap *proxycfg.ConfigSnapshot) (*envoy.Cluster, error) {
-	return s.makeMeshGatewayClusterWithConnectTimeout(clusterName, cfgSnap, 0)
+type gatewayClusterOpts struct {
+	// name for the cluster
+	name string
+
+	// isRemote determines whether the cluster is in a remote DC and we should prefer a WAN address
+	isRemote bool
+
+	// onlyPassing determines whether endpoints that do not have a passing status should be considered unhealthy
+	onlyPassing bool
+
+	// connectTimeout is the timeout for new network connections to hosts in the cluster
+	connectTimeout time.Duration
+
+	// hostnameEndpoints is a list of endpoints with a hostname as their address
+	hostnameEndpoints structs.CheckServiceNodes
 }
 
-// makeMeshGatewayClusterWithConnectTimeout initializes a mesh gateway cluster
-// with the specified connect timeout. If the timeout is 0, the connect timeout
-// defaults to use the mesh gateway timeout.
-func (s *Server) makeMeshGatewayClusterWithConnectTimeout(clusterName string, cfgSnap *proxycfg.ConfigSnapshot,
-	connectTimeout time.Duration) (*envoy.Cluster, error) {
-	cfg, err := ParseMeshGatewayConfig(cfgSnap.Proxy.Config)
+// makeGatewayCluster creates an Envoy cluster for a mesh or terminating gateway
+func (s *Server) makeGatewayCluster(snap *proxycfg.ConfigSnapshot, opts gatewayClusterOpts) *envoy.Cluster {
+	cfg, err := ParseGatewayConfig(snap.Proxy.Config)
 	if err != nil {
 		// Don't hard fail on a config typo, just warn. The parse func returns
 		// default config if there is an error so it's safe to continue.
-		s.Logger.Warn("failed to parse mesh gateway config", "error", err)
+		s.Logger.Warn("failed to parse gateway config", "error", err)
+	}
+	if opts.connectTimeout <= 0 {
+		opts.connectTimeout = time.Duration(cfg.ConnectTimeoutMs) * time.Millisecond
 	}
 
-	if connectTimeout <= 0 {
-		connectTimeout = time.Duration(cfg.ConnectTimeoutMs) * time.Millisecond
+	cluster := &envoy.Cluster{
+		Name:           opts.name,
+		ConnectTimeout: ptypes.DurationProto(opts.connectTimeout),
+
+		// Having an empty config enables outlier detection with default config.
+		OutlierDetection: &envoycluster.OutlierDetection{},
 	}
 
-	return &envoy.Cluster{
-		Name:                 clusterName,
-		ConnectTimeout:       connectTimeout,
-		ClusterDiscoveryType: &envoy.Cluster_Type{Type: envoy.Cluster_EDS},
-		EdsClusterConfig: &envoy.Cluster_EdsClusterConfig{
+	useEDS := true
+	if len(opts.hostnameEndpoints) > 0 {
+		useEDS = false
+	}
+
+	// If none of the service instances are addressed by a hostname we provide the endpoint IP addresses via EDS
+	if useEDS {
+		cluster.ClusterDiscoveryType = &envoy.Cluster_Type{Type: envoy.Cluster_EDS}
+		cluster.EdsClusterConfig = &envoy.Cluster_EdsClusterConfig{
 			EdsConfig: &envoycore.ConfigSource{
 				ConfigSourceSpecifier: &envoycore.ConfigSource_Ads{
 					Ads: &envoycore.AggregatedConfigSource{},
 				},
 			},
+		}
+		return cluster
+	}
+
+	// When a service instance is addressed by a hostname we have Envoy do the DNS resolution
+	// by setting a DNS cluster type and passing the hostname endpoints via CDS.
+	rate := 10 * time.Second
+	cluster.DnsRefreshRate = ptypes.DurationProto(rate)
+	cluster.DnsLookupFamily = envoy.Cluster_V4_ONLY
+
+	discoveryType := envoy.Cluster_Type{Type: envoy.Cluster_LOGICAL_DNS}
+	if cfg.DNSDiscoveryType == "strict_dns" {
+		discoveryType.Type = envoy.Cluster_STRICT_DNS
+	}
+	cluster.ClusterDiscoveryType = &discoveryType
+
+	endpoints := make([]*envoyendpoint.LbEndpoint, 0, 1)
+	uniqueHostnames := make(map[string]bool)
+
+	var (
+		hostname string
+		idx      int
+		fallback *envoyendpoint.LbEndpoint
+	)
+	for i, e := range opts.hostnameEndpoints {
+		addr, port := e.BestAddress(opts.isRemote)
+		uniqueHostnames[addr] = true
+
+		health, weight := calculateEndpointHealthAndWeight(e, opts.onlyPassing)
+		if health == envoycore.HealthStatus_UNHEALTHY {
+			fallback = makeLbEndpoint(addr, port, health, weight)
+			continue
+		}
+
+		if len(endpoints) == 0 {
+			endpoints = append(endpoints, makeLbEndpoint(addr, port, health, weight))
+
+			hostname = addr
+			idx = i
+			break
+		}
+	}
+
+	dc := opts.hostnameEndpoints[idx].Node.Datacenter
+	service := opts.hostnameEndpoints[idx].Service.CompoundServiceName()
+
+	loggerName := logging.TerminatingGateway
+	if snap.Kind == structs.ServiceKindMeshGateway {
+		loggerName = logging.MeshGateway
+	}
+
+	// Fall back to last unhealthy endpoint if none were healthy
+	if len(endpoints) == 0 {
+		s.Logger.Named(loggerName).Warn("upstream service does not contain any healthy instances",
+			"dc", dc, "service", service.String())
+
+		endpoints = append(endpoints, fallback)
+	}
+	if len(uniqueHostnames) > 1 {
+		s.Logger.Named(loggerName).
+			Warn(fmt.Sprintf("service contains instances with more than one unique hostname; only %q be resolved by Envoy", hostname),
+				"dc", dc, "service", service.String())
+	}
+
+	cluster.LoadAssignment = &envoy.ClusterLoadAssignment{
+		ClusterName: cluster.Name,
+		Endpoints: []*envoyendpoint.LocalityLbEndpoints{
+			{
+				LbEndpoints: endpoints,
+			},
 		},
-		// Having an empty config enables outlier detection with default config.
-		OutlierDetection: &envoycluster.OutlierDetection{},
-	}, nil
+	}
+	return cluster
+}
+
+// injectTerminatingGatewayTLSContext adds an UpstreamTlsContext to a cluster for TLS origination
+func injectTerminatingGatewayTLSContext(cfgSnap *proxycfg.ConfigSnapshot, cluster *envoy.Cluster, service structs.ServiceName) {
+	if mapping, ok := cfgSnap.TerminatingGateway.GatewayServices[service]; ok && mapping.CAFile != "" {
+		cluster.TlsContext = &envoyauth.UpstreamTlsContext{
+			CommonTlsContext: makeCommonTLSContextFromFiles(mapping.CAFile, mapping.CertFile, mapping.KeyFile),
+
+			// TODO (gateways) (freddy) If mapping.SNI is empty, does Envoy behave any differently if TlsContext.Sni is excluded?
+			Sni: mapping.SNI,
+		}
+	}
 }
 
 func makeThresholdsIfNeeded(limits UpstreamLimits) []*envoycluster.CircuitBreakers_Thresholds {
@@ -501,3 +727,24 @@ func makeThresholdsIfNeeded(limits UpstreamLimits) []*envoycluster.CircuitBreake
 
 	return []*envoycluster.CircuitBreakers_Thresholds{threshold}
 }
+
+func makeLbEndpoint(addr string, port int, health envoycore.HealthStatus, weight int) *envoyendpoint.LbEndpoint {
+	return &envoyendpoint.LbEndpoint{
+		HostIdentifier: &envoyendpoint.LbEndpoint_Endpoint{
+			Endpoint: &envoyendpoint.Endpoint{
+				Address: &envoycore.Address{
+					Address: &envoycore.Address_SocketAddress{
+						SocketAddress: &envoycore.SocketAddress{
+							Address: addr,
+							PortSpecifier: &envoycore.SocketAddress_PortValue{
+								PortValue: uint32(port),
+							},
+						},
+					},
+				},
+			},
+		},
+		HealthStatus:        health,
+		LoadBalancingWeight: makeUint32Value(weight),
+	}
+}
