@@ -19,40 +19,48 @@ import (
 	envoyhttp "github.com/envoyproxy/go-control-plane/envoy/config/filter/network/http_connection_manager/v2"
 	envoytcp "github.com/envoyproxy/go-control-plane/envoy/config/filter/network/tcp_proxy/v2"
 	envoytype "github.com/envoyproxy/go-control-plane/envoy/type"
-	"github.com/envoyproxy/go-control-plane/pkg/util"
-	"github.com/gogo/protobuf/jsonpb"
-	"github.com/gogo/protobuf/proto"
-	"github.com/gogo/protobuf/types"
-
+	"github.com/envoyproxy/go-control-plane/pkg/conversion"
+	"github.com/envoyproxy/go-control-plane/pkg/wellknown"
+	"github.com/golang/protobuf/jsonpb"
+	"github.com/golang/protobuf/proto"
+	"github.com/golang/protobuf/ptypes/any"
+	pbstruct "github.com/golang/protobuf/ptypes/struct"
+	"github.com/golang/protobuf/ptypes/wrappers"
 	"github.com/hashicorp/consul/agent/connect"
 	"github.com/hashicorp/consul/agent/proxycfg"
 	"github.com/hashicorp/consul/agent/structs"
+	"github.com/hashicorp/consul/logging"
+	"github.com/hashicorp/go-hclog"
 )
 
 // listenersFromSnapshot returns the xDS API representation of the "listeners" in the snapshot.
-func (s *Server) listenersFromSnapshot(cfgSnap *proxycfg.ConfigSnapshot, token string) ([]proto.Message, error) {
+func (s *Server) listenersFromSnapshot(cInfo connectionInfo, cfgSnap *proxycfg.ConfigSnapshot) ([]proto.Message, error) {
 	if cfgSnap == nil {
 		return nil, errors.New("nil config given")
 	}
 
 	switch cfgSnap.Kind {
 	case structs.ServiceKindConnectProxy:
-		return s.listenersFromSnapshotConnectProxy(cfgSnap, token)
+		return s.listenersFromSnapshotConnectProxy(cInfo, cfgSnap)
+	case structs.ServiceKindTerminatingGateway:
+		return s.listenersFromSnapshotGateway(cInfo, cfgSnap)
 	case structs.ServiceKindMeshGateway:
-		return s.listenersFromSnapshotMeshGateway(cfgSnap, token)
+		return s.listenersFromSnapshotGateway(cInfo, cfgSnap)
+	case structs.ServiceKindIngressGateway:
+		return s.listenersFromSnapshotGateway(cInfo, cfgSnap)
 	default:
 		return nil, fmt.Errorf("Invalid service kind: %v", cfgSnap.Kind)
 	}
 }
 
 // listenersFromSnapshotConnectProxy returns the "listeners" for a connect proxy service
-func (s *Server) listenersFromSnapshotConnectProxy(cfgSnap *proxycfg.ConfigSnapshot, token string) ([]proto.Message, error) {
+func (s *Server) listenersFromSnapshotConnectProxy(cInfo connectionInfo, cfgSnap *proxycfg.ConfigSnapshot) ([]proto.Message, error) {
 	// One listener for each upstream plus the public one
 	resources := make([]proto.Message, len(cfgSnap.Proxy.Upstreams)+1)
 
 	// Configure public listener
 	var err error
-	resources[0], err = s.makePublicListener(cfgSnap, token)
+	resources[0], err = s.makePublicListener(cInfo, cfgSnap)
 	if err != nil {
 		return nil, err
 	}
@@ -65,11 +73,13 @@ func (s *Server) listenersFromSnapshotConnectProxy(cfgSnap *proxycfg.ConfigSnaps
 		}
 
 		var upstreamListener proto.Message
-		if chain == nil || chain.IsDefault() {
-			upstreamListener, err = s.makeUpstreamListenerIgnoreDiscoveryChain(&u, chain, cfgSnap)
-		} else {
-			upstreamListener, err = s.makeUpstreamListenerForDiscoveryChain(&u, chain, cfgSnap)
-		}
+		upstreamListener, err = s.makeUpstreamListenerForDiscoveryChain(
+			&u,
+			u.LocalBindAddress,
+			chain,
+			cfgSnap,
+			nil,
+		)
 		if err != nil {
 			return nil, err
 		}
@@ -179,17 +189,23 @@ func parseCheckPath(check structs.CheckType) (structs.ExposePath, error) {
 	return path, nil
 }
 
-// listenersFromSnapshotMeshGateway returns the "listener" for a mesh-gateway service
-func (s *Server) listenersFromSnapshotMeshGateway(cfgSnap *proxycfg.ConfigSnapshot, token string) ([]proto.Message, error) {
-	cfg, err := ParseMeshGatewayConfig(cfgSnap.Proxy.Config)
+// listenersFromSnapshotGateway returns the "listener" for a terminating-gateway or mesh-gateway service
+func (s *Server) listenersFromSnapshotGateway(cInfo connectionInfo, cfgSnap *proxycfg.ConfigSnapshot) ([]proto.Message, error) {
+	cfg, err := ParseGatewayConfig(cfgSnap.Proxy.Config)
 	if err != nil {
 		// Don't hard fail on a config typo, just warn. The parse func returns
 		// default config if there is an error so it's safe to continue.
 		s.Logger.Warn("failed to parse Connect.Proxy.Config", "error", err)
 	}
 
-	// TODO - prevent invalid configurations of binding to the same port/addr
-	//        twice including with the any addresses
+	// Prevent invalid configurations of binding to the same port/addr twice
+	// including with the any addresses
+	type namedAddress struct {
+		name string
+		structs.ServiceAddress
+	}
+	seen := make(map[structs.ServiceAddress]bool)
+	addrs := make([]namedAddress, 0)
 
 	var resources []proto.Message
 	if !cfg.NoDefaultBind {
@@ -198,34 +214,126 @@ func (s *Server) listenersFromSnapshotMeshGateway(cfgSnap *proxycfg.ConfigSnapsh
 			addr = "0.0.0.0"
 		}
 
-		l, err := s.makeGatewayListener("default", addr, cfgSnap.Port, cfgSnap)
-		if err != nil {
-			return nil, err
+		a := structs.ServiceAddress{
+			Address: addr,
+			Port:    cfgSnap.Port,
+		}
+		if !seen[a] {
+			addrs = append(addrs, namedAddress{name: "default", ServiceAddress: a})
+			seen[a] = true
 		}
-		resources = append(resources, l)
 	}
 
 	if cfg.BindTaggedAddresses {
 		for name, addrCfg := range cfgSnap.TaggedAddresses {
-			l, err := s.makeGatewayListener(name, addrCfg.Address, addrCfg.Port, cfgSnap)
-			if err != nil {
-				return nil, err
+			a := structs.ServiceAddress{
+				Address: addrCfg.Address,
+				Port:    addrCfg.Port,
+			}
+			if !seen[a] {
+				addrs = append(addrs, namedAddress{name: name, ServiceAddress: a})
+				seen[a] = true
 			}
-			resources = append(resources, l)
 		}
 	}
 
 	for name, addrCfg := range cfg.BindAddresses {
-		l, err := s.makeGatewayListener(name, addrCfg.Address, addrCfg.Port, cfgSnap)
-		if err != nil {
-			return nil, err
+		a := structs.ServiceAddress{
+			Address: addrCfg.Address,
+			Port:    addrCfg.Port,
+		}
+		if !seen[a] {
+			addrs = append(addrs, namedAddress{name: name, ServiceAddress: a})
+			seen[a] = true
 		}
-		resources = append(resources, l)
 	}
 
+	// Make listeners once deduplicated
+	for _, a := range addrs {
+		var l *envoy.Listener
+
+		switch cfgSnap.Kind {
+		case structs.ServiceKindTerminatingGateway:
+			l, err = s.makeTerminatingGatewayListener(cInfo, cfgSnap, a.name, a.Address, a.Port)
+			if err != nil {
+				return nil, err
+			}
+		case structs.ServiceKindIngressGateway:
+			listeners, err := s.makeIngressGatewayListeners(a.Address, cfgSnap)
+			if err != nil {
+				return nil, err
+			}
+			resources = append(resources, listeners...)
+		case structs.ServiceKindMeshGateway:
+			l, err = s.makeMeshGatewayListener(a.name, a.Address, a.Port, cfgSnap)
+			if err != nil {
+				return nil, err
+			}
+		}
+		if l != nil {
+			resources = append(resources, l)
+		}
+	}
 	return resources, err
 }
 
+func (s *Server) makeIngressGatewayListeners(address string, cfgSnap *proxycfg.ConfigSnapshot) ([]proto.Message, error) {
+	var resources []proto.Message
+
+	for listenerKey, upstreams := range cfgSnap.IngressGateway.Upstreams {
+		var tlsContext *envoyauth.DownstreamTlsContext
+		if cfgSnap.IngressGateway.TLSEnabled {
+			tlsContext = &envoyauth.DownstreamTlsContext{
+				CommonTlsContext:         makeCommonTLSContextFromLeaf(cfgSnap, cfgSnap.Leaf()),
+				RequireClientCertificate: &wrappers.BoolValue{Value: false},
+			}
+		}
+
+		if listenerKey.Protocol == "tcp" {
+			// We rely on the invariant of upstreams slice always having at least 1
+			// member, because this key/value pair is created only when a
+			// GatewayService is returned in the RPC
+			u := upstreams[0]
+			id := u.Identifier()
+
+			chain := cfgSnap.IngressGateway.DiscoveryChain[id]
+
+			var upstreamListener proto.Message
+			upstreamListener, err := s.makeUpstreamListenerForDiscoveryChain(
+				&u,
+				address,
+				chain,
+				cfgSnap,
+				tlsContext,
+			)
+			if err != nil {
+				return nil, err
+			}
+			resources = append(resources, upstreamListener)
+		} else {
+			// If multiple upstreams share this port, make a special listener for the protocol.
+			listener := makeListener(listenerKey.Protocol, address, listenerKey.Port)
+			filter, err := makeListenerFilter(
+				true, listenerKey.Protocol, listenerKey.RouteName(), "", "ingress_upstream_", "", false)
+			if err != nil {
+				return nil, err
+			}
+
+			listener.FilterChains = []*envoylistener.FilterChain{
+				{
+					Filters: []*envoylistener.Filter{
+						filter,
+					},
+					TlsContext: tlsContext,
+				},
+			}
+			resources = append(resources, listener)
+		}
+	}
+
+	return resources, nil
+}
+
 // makeListener returns a listener with name and bind details set. Filters must
 // be added before it's useful.
 //
@@ -263,7 +371,7 @@ func makeListener(name, addr string, port int) *envoy.Listener {
 // from rather than our slight variant in JSON/hcl.
 func makeListenerFromUserConfig(configJSON string) (*envoy.Listener, error) {
 	// Figure out if there is an @type field. We don't require is since we know
-	// this will be a listener but unmarshalling into types.Any fails if it's not
+	// this will be a listener but unmarshalling into any.Any fails if it's not
 	// there and unmarshalling into listener directly fails if it is...
 	var jsonFields map[string]*json.RawMessage
 	if err := json.Unmarshal([]byte(configJSON), &jsonFields); err != nil {
@@ -273,8 +381,8 @@ func makeListenerFromUserConfig(configJSON string) (*envoy.Listener, error) {
 	var l envoy.Listener
 
 	if _, ok := jsonFields["@type"]; ok {
-		// Type field is present so decode it as a types.Any
-		var any types.Any
+		// Type field is present so decode it as a any.Any
+		var any any.Any
 		err := jsonpb.UnmarshalString(configJSON, &any)
 		if err != nil {
 			return nil, err
@@ -299,26 +407,25 @@ func makeListenerFromUserConfig(configJSON string) (*envoy.Listener, error) {
 // specify custom listener params in config but still get our certs delivered
 // dynamically and intentions enforced without coming up with some complicated
 // templating/merging solution.
-func injectConnectFilters(cfgSnap *proxycfg.ConfigSnapshot, token string, listener *envoy.Listener) error {
-	authFilter, err := makeExtAuthFilter(token)
+func injectConnectFilters(cInfo connectionInfo, cfgSnap *proxycfg.ConfigSnapshot, listener *envoy.Listener) error {
+	authFilter, err := makeExtAuthFilter(cInfo.Token)
 	if err != nil {
 		return err
 	}
 	for idx := range listener.FilterChains {
 		// Insert our authz filter before any others
 		listener.FilterChains[idx].Filters =
-			append([]envoylistener.Filter{authFilter}, listener.FilterChains[idx].Filters...)
+			append([]*envoylistener.Filter{authFilter}, listener.FilterChains[idx].Filters...)
 
-		// Force our TLS for all filter chains on a public listener
 		listener.FilterChains[idx].TlsContext = &envoyauth.DownstreamTlsContext{
-			CommonTlsContext:         makeCommonTLSContext(cfgSnap),
-			RequireClientCertificate: &types.BoolValue{Value: true},
+			CommonTlsContext:         makeCommonTLSContextFromLeaf(cfgSnap, cfgSnap.Leaf()),
+			RequireClientCertificate: &wrappers.BoolValue{Value: true},
 		}
 	}
 	return nil
 }
 
-func (s *Server) makePublicListener(cfgSnap *proxycfg.ConfigSnapshot, token string) (proto.Message, error) {
+func (s *Server) makePublicListener(cInfo connectionInfo, cfgSnap *proxycfg.ConfigSnapshot) (proto.Message, error) {
 	var l *envoy.Listener
 	var err error
 
@@ -363,16 +470,16 @@ func (s *Server) makePublicListener(cfgSnap *proxycfg.ConfigSnapshot, token stri
 		if err != nil {
 			return nil, err
 		}
-		l.FilterChains = []envoylistener.FilterChain{
+		l.FilterChains = []*envoylistener.FilterChain{
 			{
-				Filters: []envoylistener.Filter{
+				Filters: []*envoylistener.Filter{
 					filter,
 				},
 			},
 		}
 	}
 
-	err = injectConnectFilters(cfgSnap, token, l)
+	err = injectConnectFilters(cInfo, cfgSnap, l)
 	return l, err
 }
 
@@ -408,8 +515,8 @@ func (s *Server) makeExposedCheckListener(cfgSnap *proxycfg.ConfigSnapshot, clus
 		return nil, err
 	}
 
-	chain := envoylistener.FilterChain{
-		Filters: []envoylistener.Filter{f},
+	chain := &envoylistener.FilterChain{
+		Filters: []*envoylistener.Filter{f},
 	}
 
 	// For registered checks restrict traffic sources to localhost and Consul's advertise addr
@@ -427,67 +534,123 @@ func (s *Server) makeExposedCheckListener(cfgSnap *proxycfg.ConfigSnapshot, clus
 
 		chain.FilterChainMatch = &envoylistener.FilterChainMatch{
 			SourcePrefixRanges: []*envoycore.CidrRange{
-				{AddressPrefix: "127.0.0.1", PrefixLen: &types.UInt32Value{Value: 8}},
-				{AddressPrefix: "::1", PrefixLen: &types.UInt32Value{Value: 128}},
-				{AddressPrefix: advertise, PrefixLen: &types.UInt32Value{Value: uint32(advertiseLen)}},
+				{AddressPrefix: "127.0.0.1", PrefixLen: &wrappers.UInt32Value{Value: 8}},
+				{AddressPrefix: "::1", PrefixLen: &wrappers.UInt32Value{Value: 128}},
+				{AddressPrefix: advertise, PrefixLen: &wrappers.UInt32Value{Value: uint32(advertiseLen)}},
 			},
 		}
 	}
 
-	l.FilterChains = []envoylistener.FilterChain{chain}
+	l.FilterChains = []*envoylistener.FilterChain{chain}
 
 	return l, err
 }
 
-// makeUpstreamListenerIgnoreDiscoveryChain counterintuitively takes an (optional) chain
-func (s *Server) makeUpstreamListenerIgnoreDiscoveryChain(
-	u *structs.Upstream,
-	chain *structs.CompiledDiscoveryChain,
+func (s *Server) makeTerminatingGatewayListener(
+	cInfo connectionInfo,
 	cfgSnap *proxycfg.ConfigSnapshot,
-) (proto.Message, error) {
-	cfg, err := ParseUpstreamConfig(u.Config)
+	name, addr string,
+	port int,
+) (*envoy.Listener, error) {
+	l := makeListener(name, addr, port)
+
+	tlsInspector, err := makeTLSInspectorListenerFilter()
 	if err != nil {
-		// Don't hard fail on a config typo, just warn. The parse func returns
-		// default config if there is an error so it's safe to continue.
-		s.Logger.Warn("failed to parse", "upstream", u.Identifier(), "error", err)
+		return nil, err
 	}
-	if cfg.ListenerJSON != "" {
-		return makeListenerFromUserConfig(cfg.ListenerJSON)
+	l.ListenerFilters = []*envoylistener.ListenerFilter{tlsInspector}
+
+	// Make a FilterChain for each linked service
+	// Match on the cluster name,
+	for svc, _ := range cfgSnap.TerminatingGateway.ServiceGroups {
+		clusterName := connect.ServiceSNI(svc.Name, "", svc.NamespaceOrDefault(), cfgSnap.Datacenter, cfgSnap.Roots.TrustDomain)
+		resolver, hasResolver := cfgSnap.TerminatingGateway.ServiceResolvers[svc]
+
+		// Skip the service if we don't have a cert to present for mTLS
+		if cert, ok := cfgSnap.TerminatingGateway.ServiceLeaves[svc]; !ok || cert == nil {
+			// TODO (gateways) (freddy) Should the error suggest that the issue may be ACLs? (need service:write on service)
+			s.Logger.Named(logging.TerminatingGateway).
+				Error("no client certificate available for linked service, skipping filter chain creation",
+					"service", svc.String(), "error", err)
+			continue
+		}
+
+		clusterChain, err := s.sniFilterChainTerminatingGateway(cInfo, cfgSnap, name, clusterName, svc)
+		if err != nil {
+			return nil, fmt.Errorf("failed to make filter chain for cluster %q: %v", clusterName, err)
+		}
+		l.FilterChains = append(l.FilterChains, clusterChain)
+
+		// if there is a service-resolver for this service then also setup subset filter chains for it
+		if hasResolver {
+			// generate 1 filter chain for each service subset
+			for subsetName := range resolver.Subsets {
+				clusterName := connect.ServiceSNI(svc.Name, subsetName, svc.NamespaceOrDefault(), cfgSnap.Datacenter, cfgSnap.Roots.TrustDomain)
+
+				clusterChain, err := s.sniFilterChainTerminatingGateway(cInfo, cfgSnap, name, clusterName, svc)
+				if err != nil {
+					return nil, fmt.Errorf("failed to make filter chain for cluster %q: %v", clusterName, err)
+				}
+				l.FilterChains = append(l.FilterChains, clusterChain)
+			}
+		}
 	}
 
-	addr := u.LocalBindAddress
-	if addr == "" {
-		addr = "127.0.0.1"
+	// This fallback catch-all filter ensures a listener will be present for health checks to pass
+	// Envoy will reset these connections since known endpoints are caught by filter chain matches above
+	tcpProxy, err := makeTCPProxyFilter(name, "", "terminating_gateway_")
+	if err != nil {
+		return nil, err
 	}
-
-	upstreamID := u.Identifier()
-
-	dc := u.Datacenter
-	if dc == "" {
-		dc = cfgSnap.Datacenter
+	fallback := &envoylistener.FilterChain{
+		Filters: []*envoylistener.Filter{
+			{Name: "envoy.filters.network.sni_cluster"},
+			tcpProxy,
+		},
 	}
-	sni := connect.UpstreamSNI(u, "", dc, cfgSnap.Roots.TrustDomain)
+	l.FilterChains = append(l.FilterChains, fallback)
 
-	clusterName := CustomizeClusterName(sni, chain)
+	return l, nil
+}
 
-	l := makeListener(upstreamID, addr, u.LocalBindPort)
-	filter, err := makeListenerFilter(
-		false, cfg.Protocol, upstreamID, clusterName, "upstream_", "", false)
+func (s *Server) sniFilterChainTerminatingGateway(
+	cInfo connectionInfo,
+	cfgSnap *proxycfg.ConfigSnapshot,
+	listener, cluster string,
+	service structs.ServiceName,
+) (*envoylistener.FilterChain, error) {
+
+	authFilter, err := makeExtAuthFilter(cInfo.Token)
+	if err != nil {
+		return nil, err
+	}
+	sniCluster, err := makeSNIClusterFilter()
 	if err != nil {
 		return nil, err
 	}
 
-	l.FilterChains = []envoylistener.FilterChain{
-		{
-			Filters: []envoylistener.Filter{
-				filter,
-			},
-		},
+	// The cluster name here doesn't matter as the sni_cluster filter will fill it in for us.
+	statPrefix := fmt.Sprintf("terminating_gateway_%s_%s_", service.NamespaceOrDefault(), service.Name)
+	tcpProxy, err := makeTCPProxyFilter(listener, "", statPrefix)
+	if err != nil {
+		return nil, err
 	}
-	return l, nil
+
+	return &envoylistener.FilterChain{
+		FilterChainMatch: makeSNIFilterChainMatch(cluster),
+		Filters: []*envoylistener.Filter{
+			authFilter,
+			sniCluster,
+			tcpProxy,
+		},
+		TlsContext: &envoyauth.DownstreamTlsContext{
+			CommonTlsContext:         makeCommonTLSContextFromLeaf(cfgSnap, cfgSnap.TerminatingGateway.ServiceLeaves[service]),
+			RequireClientCertificate: &wrappers.BoolValue{Value: true},
+		},
+	}, err
 }
 
-func (s *Server) makeGatewayListener(name, addr string, port int, cfgSnap *proxycfg.ConfigSnapshot) (*envoy.Listener, error) {
+func (s *Server) makeMeshGatewayListener(name, addr string, port int, cfgSnap *proxycfg.ConfigSnapshot) (*envoy.Listener, error) {
 	tlsInspector, err := makeTLSInspectorListenerFilter()
 	if err != nil {
 		return nil, err
@@ -505,19 +668,23 @@ func (s *Server) makeGatewayListener(name, addr string, port int, cfgSnap *proxy
 		return nil, err
 	}
 
-	sniClusterChain := envoylistener.FilterChain{
-		Filters: []envoylistener.Filter{
+	sniClusterChain := &envoylistener.FilterChain{
+		Filters: []*envoylistener.Filter{
 			sniCluster,
 			tcpProxy,
 		},
 	}
 
 	l := makeListener(name, addr, port)
-	l.ListenerFilters = []envoylistener.ListenerFilter{tlsInspector}
+	l.ListenerFilters = []*envoylistener.ListenerFilter{tlsInspector}
 
 	// TODO (mesh-gateway) - Do we need to create clusters for all the old trust domains as well?
 	// We need 1 Filter Chain per datacenter
-	for dc := range cfgSnap.MeshGateway.GatewayGroups {
+	datacenters := cfgSnap.MeshGateway.Datacenters()
+	for _, dc := range datacenters {
+		if dc == cfgSnap.Datacenter {
+			continue // skip local
+		}
 		clusterName := connect.DatacenterSNI(dc, cfgSnap.Roots.TrustDomain)
 		filterName := fmt.Sprintf("%s_%s", name, dc)
 		dcTCPProxy, err := makeTCPProxyFilter(filterName, clusterName, "mesh_gateway_remote_")
@@ -525,16 +692,59 @@ func (s *Server) makeGatewayListener(name, addr string, port int, cfgSnap *proxy
 			return nil, err
 		}
 
-		l.FilterChains = append(l.FilterChains, envoylistener.FilterChain{
+		l.FilterChains = append(l.FilterChains, &envoylistener.FilterChain{
 			FilterChainMatch: &envoylistener.FilterChainMatch{
 				ServerNames: []string{fmt.Sprintf("*.%s", clusterName)},
 			},
-			Filters: []envoylistener.Filter{
+			Filters: []*envoylistener.Filter{
 				dcTCPProxy,
 			},
 		})
 	}
 
+	if cfgSnap.ServiceMeta[structs.MetaWANFederationKey] == "1" && cfgSnap.ServerSNIFn != nil {
+		for _, dc := range datacenters {
+			if dc == cfgSnap.Datacenter {
+				continue // skip local
+			}
+			clusterName := cfgSnap.ServerSNIFn(dc, "")
+			filterName := fmt.Sprintf("%s_%s", name, dc)
+			dcTCPProxy, err := makeTCPProxyFilter(filterName, clusterName, "mesh_gateway_remote_")
+			if err != nil {
+				return nil, err
+			}
+
+			l.FilterChains = append(l.FilterChains, &envoylistener.FilterChain{
+				FilterChainMatch: &envoylistener.FilterChainMatch{
+					ServerNames: []string{fmt.Sprintf("*.%s", clusterName)},
+				},
+				Filters: []*envoylistener.Filter{
+					dcTCPProxy,
+				},
+			})
+		}
+
+		// Wildcard all flavors to each server.
+		for _, srv := range cfgSnap.MeshGateway.ConsulServers {
+			clusterName := cfgSnap.ServerSNIFn(cfgSnap.Datacenter, srv.Node.Node)
+
+			filterName := fmt.Sprintf("%s_%s", name, cfgSnap.Datacenter)
+			dcTCPProxy, err := makeTCPProxyFilter(filterName, clusterName, "mesh_gateway_local_server_")
+			if err != nil {
+				return nil, err
+			}
+
+			l.FilterChains = append(l.FilterChains, &envoylistener.FilterChain{
+				FilterChainMatch: &envoylistener.FilterChainMatch{
+					ServerNames: []string{fmt.Sprintf("%s", clusterName)},
+				},
+				Filters: []*envoylistener.Filter{
+					dcTCPProxy,
+				},
+			})
+		}
+	}
+
 	// This needs to get tacked on at the end as it has no
 	// matching and will act as a catch all
 	l.FilterChains = append(l.FilterChains, sniClusterChain)
@@ -544,72 +754,115 @@ func (s *Server) makeGatewayListener(name, addr string, port int, cfgSnap *proxy
 
 func (s *Server) makeUpstreamListenerForDiscoveryChain(
 	u *structs.Upstream,
+	address string,
 	chain *structs.CompiledDiscoveryChain,
 	cfgSnap *proxycfg.ConfigSnapshot,
+	tlsContext *envoyauth.DownstreamTlsContext,
 ) (proto.Message, error) {
-	cfg, err := ParseUpstreamConfigNoDefaults(u.Config)
-	if err != nil {
-		// Don't hard fail on a config typo, just warn. The parse func returns
-		// default config if there is an error so it's safe to continue.
-		s.Logger.Warn("failed to parse", "upstream", u.Identifier(), "error", err)
+	if address == "" {
+		address = "127.0.0.1"
 	}
-	if cfg.ListenerJSON != "" {
-		s.Logger.Warn("ignoring escape hatch setting because already configured for",
-			"discovery chain", chain.ServiceName, "upstream", u.Identifier(), "config", "envoy_listener_json")
-	}
-
-	addr := u.LocalBindAddress
-	if addr == "" {
-		addr = "127.0.0.1"
-	}
-
 	upstreamID := u.Identifier()
+	l := makeListener(upstreamID, address, u.LocalBindPort)
 
-	l := makeListener(upstreamID, addr, u.LocalBindPort)
-
-	proto := cfg.Protocol
-	if proto == "" {
-		proto = chain.Protocol
-	}
-
-	if proto == "" {
-		proto = "tcp"
+	cfg := getAndModifyUpstreamConfigForListener(s.Logger, u, chain)
+	if cfg.ListenerJSON != "" {
+		return makeListenerFromUserConfig(cfg.ListenerJSON)
 	}
 
 	useRDS := true
 	clusterName := ""
-	if proto == "tcp" {
+	if chain == nil || chain.IsDefault() {
+		dc := u.Datacenter
+		if dc == "" {
+			dc = cfgSnap.Datacenter
+		}
+		sni := connect.UpstreamSNI(u, "", dc, cfgSnap.Roots.TrustDomain)
+
+		useRDS = false
+		clusterName = CustomizeClusterName(sni, chain)
+
+	} else if cfg.Protocol == "tcp" {
 		startNode := chain.Nodes[chain.StartNode]
 		if startNode == nil {
-			panic("missing first node in compiled discovery chain for: " + chain.ServiceName)
+			return nil, fmt.Errorf("missing first node in compiled discovery chain for: %s", chain.ServiceName)
 		} else if startNode.Type != structs.DiscoveryGraphNodeTypeResolver {
-			panic(fmt.Sprintf("unexpected first node in discovery chain using protocol=%q: %s", proto, startNode.Type))
+			return nil, fmt.Errorf("unexpected first node in discovery chain using protocol=%q: %s", cfg.Protocol, startNode.Type)
 		}
 		targetID := startNode.Resolver.Target
 		target := chain.Targets[targetID]
-		clusterName = CustomizeClusterName(target.Name, chain)
+
 		useRDS = false
+		clusterName = CustomizeClusterName(target.Name, chain)
 	}
 
 	filter, err := makeListenerFilter(
-		useRDS, proto, upstreamID, clusterName, "upstream_", "", false)
+		useRDS, cfg.Protocol, upstreamID, clusterName, "upstream_", "", false)
 	if err != nil {
 		return nil, err
 	}
 
-	l.FilterChains = []envoylistener.FilterChain{
+	l.FilterChains = []*envoylistener.FilterChain{
 		{
-			Filters: []envoylistener.Filter{
+			Filters: []*envoylistener.Filter{
 				filter,
 			},
+			TlsContext: tlsContext,
 		},
 	}
 	return l, nil
 }
 
+func getAndModifyUpstreamConfigForListener(logger hclog.Logger, u *structs.Upstream, chain *structs.CompiledDiscoveryChain) UpstreamConfig {
+	var (
+		cfg UpstreamConfig
+		err error
+	)
+
+	if chain == nil || chain.IsDefault() {
+		cfg, err = ParseUpstreamConfig(u.Config)
+		if err != nil {
+			// Don't hard fail on a config typo, just warn. The parse func returns
+			// default config if there is an error so it's safe to continue.
+			logger.Warn("failed to parse", "upstream", u.Identifier(), "error", err)
+		}
+	} else {
+		// Use NoDefaults here so that we can set the protocol to the chain
+		// protocol if necessary
+		cfg, err = ParseUpstreamConfigNoDefaults(u.Config)
+		if err != nil {
+			// Don't hard fail on a config typo, just warn. The parse func returns
+			// default config if there is an error so it's safe to continue.
+			logger.Warn("failed to parse", "upstream", u.Identifier(), "error", err)
+		}
+
+		if cfg.ListenerJSON != "" {
+			logger.Warn("ignoring escape hatch setting because already configured for",
+				"discovery chain", chain.ServiceName, "upstream", u.Identifier(), "config", "envoy_listener_json")
+
+			// Remove from config struct so we don't use it later on
+			cfg.ListenerJSON = ""
+		}
+
+		proto := cfg.Protocol
+		if proto == "" {
+			proto = chain.Protocol
+		}
+
+		if proto == "" {
+			proto = "tcp"
+		}
+
+		// set back on the config so that we can use it from return value
+		cfg.Protocol = proto
+	}
+
+	return cfg
+}
+
 func makeListenerFilter(
 	useRDS bool,
-	protocol, filterName, cluster, statPrefix, routePath string, ingress bool) (envoylistener.Filter, error) {
+	protocol, filterName, cluster, statPrefix, routePath string, ingress bool) (*envoylistener.Filter, error) {
 
 	switch protocol {
 	case "grpc":
@@ -622,31 +875,30 @@ func makeListenerFilter(
 		fallthrough
 	default:
 		if useRDS {
-			return envoylistener.Filter{}, fmt.Errorf("RDS is not compatible with the tcp proxy filter")
+			return nil, fmt.Errorf("RDS is not compatible with the tcp proxy filter")
 		} else if cluster == "" {
-			return envoylistener.Filter{}, fmt.Errorf("cluster name is required for a tcp proxy filter")
+			return nil, fmt.Errorf("cluster name is required for a tcp proxy filter")
 		}
 		return makeTCPProxyFilter(filterName, cluster, statPrefix)
 	}
 }
 
-func makeTLSInspectorListenerFilter() (envoylistener.ListenerFilter, error) {
-	return envoylistener.ListenerFilter{Name: util.TlsInspector}, nil
+func makeTLSInspectorListenerFilter() (*envoylistener.ListenerFilter, error) {
+	return &envoylistener.ListenerFilter{Name: wellknown.TlsInspector}, nil
 }
 
-// TODO(rb): should this be dead code?
-func makeSNIFilterChainMatch(sniMatch string) (*envoylistener.FilterChainMatch, error) {
+func makeSNIFilterChainMatch(sniMatch string) *envoylistener.FilterChainMatch {
 	return &envoylistener.FilterChainMatch{
 		ServerNames: []string{sniMatch},
-	}, nil
+	}
 }
 
-func makeSNIClusterFilter() (envoylistener.Filter, error) {
+func makeSNIClusterFilter() (*envoylistener.Filter, error) {
 	// This filter has no config which is why we are not calling make
-	return envoylistener.Filter{Name: "envoy.filters.network.sni_cluster"}, nil
+	return &envoylistener.Filter{Name: "envoy.filters.network.sni_cluster"}, nil
 }
 
-func makeTCPProxyFilter(filterName, cluster, statPrefix string) (envoylistener.Filter, error) {
+func makeTCPProxyFilter(filterName, cluster, statPrefix string) (*envoylistener.Filter, error) {
 	cfg := &envoytcp.TcpProxy{
 		StatPrefix:       makeStatPrefix("tcp", statPrefix, filterName),
 		ClusterSpecifier: &envoytcp.TcpProxy_Cluster{Cluster: cluster},
@@ -665,10 +917,10 @@ func makeHTTPFilter(
 	useRDS bool,
 	filterName, cluster, statPrefix, routePath string,
 	ingress, grpc, http2 bool,
-) (envoylistener.Filter, error) {
-	op := envoyhttp.INGRESS
+) (*envoylistener.Filter, error) {
+	op := envoyhttp.HttpConnectionManager_Tracing_INGRESS
 	if !ingress {
-		op = envoyhttp.EGRESS
+		op = envoyhttp.HttpConnectionManager_Tracing_EGRESS
 	}
 	proto := "http"
 	if grpc {
@@ -677,7 +929,7 @@ func makeHTTPFilter(
 
 	cfg := &envoyhttp.HttpConnectionManager{
 		StatPrefix: makeStatPrefix(proto, statPrefix, filterName),
-		CodecType:  envoyhttp.AUTO,
+		CodecType:  envoyhttp.HttpConnectionManager_AUTO,
 		HttpFilters: []*envoyhttp.HttpFilter{
 			&envoyhttp.HttpFilter{
 				Name: "envoy.router",
@@ -694,12 +946,12 @@ func makeHTTPFilter(
 
 	if useRDS {
 		if cluster != "" {
-			return envoylistener.Filter{}, fmt.Errorf("cannot specify cluster name when using RDS")
+			return nil, fmt.Errorf("cannot specify cluster name when using RDS")
 		}
 		cfg.RouteSpecifier = &envoyhttp.HttpConnectionManager_Rds{
 			Rds: &envoyhttp.Rds{
 				RouteConfigName: filterName,
-				ConfigSource: envoycore.ConfigSource{
+				ConfigSource: &envoycore.ConfigSource{
 					ConfigSourceSpecifier: &envoycore.ConfigSource_Ads{
 						Ads: &envoycore.AggregatedConfigSource{},
 					},
@@ -708,10 +960,10 @@ func makeHTTPFilter(
 		}
 	} else {
 		if cluster == "" {
-			return envoylistener.Filter{}, fmt.Errorf("must specify cluster name when not using RDS")
+			return nil, fmt.Errorf("must specify cluster name when not using RDS")
 		}
-		route := envoyroute.Route{
-			Match: envoyroute.RouteMatch{
+		route := &envoyroute.Route{
+			Match: &envoyroute.RouteMatch{
 				PathSpecifier: &envoyroute.RouteMatch_Prefix{
 					Prefix: "/",
 				},
@@ -737,11 +989,11 @@ func makeHTTPFilter(
 		cfg.RouteSpecifier = &envoyhttp.HttpConnectionManager_RouteConfig{
 			RouteConfig: &envoy.RouteConfiguration{
 				Name: filterName,
-				VirtualHosts: []envoyroute.VirtualHost{
+				VirtualHosts: []*envoyroute.VirtualHost{
 					{
 						Name:    filterName,
 						Domains: []string{"*"},
-						Routes: []envoyroute.Route{
+						Routes: []*envoyroute.Route{
 							route,
 						},
 					},
@@ -758,14 +1010,14 @@ func makeHTTPFilter(
 		// Add grpc bridge before router
 		cfg.HttpFilters = append([]*envoyhttp.HttpFilter{{
 			Name:       "envoy.grpc_http1_bridge",
-			ConfigType: &envoyhttp.HttpFilter_Config{Config: &types.Struct{}},
+			ConfigType: &envoyhttp.HttpFilter_Config{Config: &pbstruct.Struct{}},
 		}}, cfg.HttpFilters...)
 	}
 
 	return makeFilter("envoy.http_connection_manager", cfg)
 }
 
-func makeExtAuthFilter(token string) (envoylistener.Filter, error) {
+func makeExtAuthFilter(token string) (*envoylistener.Filter, error) {
 	cfg := &extauthz.ExtAuthz{
 		StatPrefix: "connect_authz",
 		GrpcService: &envoycore.GrpcService{
@@ -790,21 +1042,21 @@ func makeExtAuthFilter(token string) (envoylistener.Filter, error) {
 	return makeFilter("envoy.ext_authz", cfg)
 }
 
-func makeFilter(name string, cfg proto.Message) (envoylistener.Filter, error) {
-	// Ridiculous dance to make that pbstruct into types.Struct by... encoding it
+func makeFilter(name string, cfg proto.Message) (*envoylistener.Filter, error) {
+	// Ridiculous dance to make that struct into pbstruct.Struct by... encoding it
 	// as JSON and decoding again!!
-	cfgStruct, err := util.MessageToStruct(cfg)
+	cfgStruct, err := conversion.MessageToStruct(cfg)
 	if err != nil {
-		return envoylistener.Filter{}, err
+		return nil, err
 	}
 
-	return envoylistener.Filter{
+	return &envoylistener.Filter{
 		Name:       name,
 		ConfigType: &envoylistener.Filter_Config{Config: cfgStruct},
 	}, nil
 }
 
-func makeCommonTLSContext(cfgSnap *proxycfg.ConfigSnapshot) *envoyauth.CommonTlsContext {
+func makeCommonTLSContextFromLeaf(cfgSnap *proxycfg.ConfigSnapshot, leaf *structs.IssuedCert) *envoyauth.CommonTlsContext {
 	// Concatenate all the root PEMs into one.
 	// TODO(banks): verify this actually works with Envoy (docs are not clear).
 	rootPEMS := ""
@@ -821,12 +1073,12 @@ func makeCommonTLSContext(cfgSnap *proxycfg.ConfigSnapshot) *envoyauth.CommonTls
 			&envoyauth.TlsCertificate{
 				CertificateChain: &envoycore.DataSource{
 					Specifier: &envoycore.DataSource_InlineString{
-						InlineString: cfgSnap.ConnectProxy.Leaf.CertPEM,
+						InlineString: leaf.CertPEM,
 					},
 				},
 				PrivateKey: &envoycore.DataSource{
 					Specifier: &envoycore.DataSource_InlineString{
-						InlineString: cfgSnap.ConnectProxy.Leaf.PrivateKeyPEM,
+						InlineString: leaf.PrivateKeyPEM,
 					},
 				},
 			},
@@ -843,3 +1095,42 @@ func makeCommonTLSContext(cfgSnap *proxycfg.ConfigSnapshot) *envoyauth.CommonTls
 		},
 	}
 }
+
+func makeCommonTLSContextFromFiles(caFile, certFile, keyFile string) *envoyauth.CommonTlsContext {
+	ctx := envoyauth.CommonTlsContext{
+		TlsParams: &envoyauth.TlsParameters{},
+	}
+
+	// Verify certificate of peer if caFile is specified
+	if caFile != "" {
+		ctx.ValidationContextType = &envoyauth.CommonTlsContext_ValidationContext{
+			ValidationContext: &envoyauth.CertificateValidationContext{
+				TrustedCa: &envoycore.DataSource{
+					Specifier: &envoycore.DataSource_Filename{
+						Filename: caFile,
+					},
+				},
+			},
+		}
+	}
+
+	// Present certificate for mTLS if cert and key files are specified
+	if certFile != "" && keyFile != "" {
+		ctx.TlsCertificates = []*envoyauth.TlsCertificate{
+			{
+				CertificateChain: &envoycore.DataSource{
+					Specifier: &envoycore.DataSource_Filename{
+						Filename: certFile,
+					},
+				},
+				PrivateKey: &envoycore.DataSource{
+					Specifier: &envoycore.DataSource_Filename{
+						Filename: keyFile,
+					},
+				},
+			},
+		}
+	}
+
+	return &ctx
+}
