@@ -7,18 +7,12 @@ import (
 	"sync/atomic"
 	"time"
 
-	"google.golang.org/grpc"
-	"google.golang.org/grpc/codes"
-	"google.golang.org/grpc/credentials"
-	"google.golang.org/grpc/metadata"
-	"google.golang.org/grpc/status"
-
 	envoy "github.com/envoyproxy/go-control-plane/envoy/api/v2"
+	envoycore "github.com/envoyproxy/go-control-plane/envoy/api/v2/core"
 	envoyauthz "github.com/envoyproxy/go-control-plane/envoy/service/auth/v2"
 	envoyauthzalpha "github.com/envoyproxy/go-control-plane/envoy/service/auth/v2alpha"
 	envoydisco "github.com/envoyproxy/go-control-plane/envoy/service/discovery/v2"
-	"github.com/gogo/googleapis/google/rpc"
-	"github.com/gogo/protobuf/proto"
+	"github.com/golang/protobuf/proto"
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/cache"
 	"github.com/hashicorp/consul/agent/connect"
@@ -27,6 +21,12 @@ import (
 	"github.com/hashicorp/consul/logging"
 	"github.com/hashicorp/consul/tlsutil"
 	"github.com/hashicorp/go-hclog"
+	rpcstatus "google.golang.org/genproto/googleapis/rpc/status"
+	"google.golang.org/grpc"
+	"google.golang.org/grpc/codes"
+	"google.golang.org/grpc/credentials"
+	"google.golang.org/grpc/metadata"
+	"google.golang.org/grpc/status"
 )
 
 // ADSStream is a shorter way of referring to this thing...
@@ -190,48 +190,61 @@ func (s *Server) process(stream ADSStream, reqCh <-chan *envoy.DiscoveryRequest)
 	var nonce uint64
 
 	// xDS works with versions of configs. Internally we don't have a consistent
-	// version. We could just hash the config since versions don't have to be
-	// ordered as far as I can tell, but it's cheaper just to increment a counter
+	// version. We could hash the config since versions don't have to be
+	// ordered as far as I can tell, but it is cheaper to increment a counter
 	// every time we observe a new config since the upstream proxycfg package only
 	// delivers updates when there are actual changes.
 	var configVersion uint64
 
 	// Loop state
-	var cfgSnap *proxycfg.ConfigSnapshot
-	var req *envoy.DiscoveryRequest
-	var ok bool
-	var stateCh <-chan *proxycfg.ConfigSnapshot
-	var watchCancel func()
-	var proxyID structs.ServiceID
+	var (
+		cfgSnap       *proxycfg.ConfigSnapshot
+		req           *envoy.DiscoveryRequest
+		node          *envoycore.Node
+		proxyFeatures supportedProxyFeatures
+		ok            bool
+		stateCh       <-chan *proxycfg.ConfigSnapshot
+		watchCancel   func()
+		proxyID       structs.ServiceID
+	)
 
 	// need to run a small state machine to get through initial authentication.
 	var state = stateInit
 
 	// Configure handlers for each type of request
 	handlers := map[string]*xDSType{
-		EndpointType: &xDSType{
+		EndpointType: {
 			typeURL:   EndpointType,
 			resources: s.endpointsFromSnapshot,
 			stream:    stream,
 		},
-		ClusterType: &xDSType{
+		ClusterType: {
 			typeURL:   ClusterType,
 			resources: s.clustersFromSnapshot,
 			stream:    stream,
 			allowEmptyFn: func(cfgSnap *proxycfg.ConfigSnapshot) bool {
-				// Mesh gateways are allowed to inform CDS of no clusters.
-				return cfgSnap.Kind == structs.ServiceKindMeshGateway
+				// Mesh, Ingress, and Terminating gateways are allowed to inform CDS of
+				// no clusters.
+				return cfgSnap.Kind == structs.ServiceKindMeshGateway ||
+					cfgSnap.Kind == structs.ServiceKindTerminatingGateway ||
+					cfgSnap.Kind == structs.ServiceKindIngressGateway
 			},
 		},
-		RouteType: &xDSType{
+		RouteType: {
 			typeURL:   RouteType,
 			resources: routesFromSnapshot,
 			stream:    stream,
+			allowEmptyFn: func(cfgSnap *proxycfg.ConfigSnapshot) bool {
+				return cfgSnap.Kind == structs.ServiceKindIngressGateway
+			},
 		},
-		ListenerType: &xDSType{
+		ListenerType: {
 			typeURL:   ListenerType,
 			resources: s.listenersFromSnapshot,
 			stream:    stream,
+			allowEmptyFn: func(cfgSnap *proxycfg.ConfigSnapshot) bool {
+				return cfgSnap.Kind == structs.ServiceKindIngressGateway
+			},
 		},
 	}
 
@@ -245,8 +258,7 @@ func (s *Server) process(stream ADSStream, reqCh <-chan *envoy.DiscoveryRequest)
 			return status.Errorf(codes.Unauthenticated, "unauthenticated: no config snapshot")
 		}
 
-		token := tokenFromStream(stream)
-		rule, err := s.ResolveToken(token)
+		rule, err := s.ResolveToken(tokenFromContext(stream.Context()))
 
 		if acl.IsErrNotFound(err) {
 			return status.Errorf(codes.Unauthenticated, "unauthenticated: %v", err)
@@ -263,7 +275,7 @@ func (s *Server) process(stream ADSStream, reqCh <-chan *envoy.DiscoveryRequest)
 			if rule != nil && rule.ServiceWrite(cfgSnap.Proxy.DestinationServiceName, &authzContext) != acl.Allow {
 				return status.Errorf(codes.PermissionDenied, "permission denied")
 			}
-		case structs.ServiceKindMeshGateway:
+		case structs.ServiceKindMeshGateway, structs.ServiceKindTerminatingGateway, structs.ServiceKindIngressGateway:
 			cfgSnap.ProxyID.EnterpriseMeta.FillAuthzContext(&authzContext)
 			if rule != nil && rule.ServiceWrite(cfgSnap.Service, &authzContext) != acl.Allow {
 				return status.Errorf(codes.PermissionDenied, "permission denied")
@@ -296,8 +308,14 @@ func (s *Server) process(stream ADSStream, reqCh <-chan *envoy.DiscoveryRequest)
 			if req.TypeUrl == "" {
 				return status.Errorf(codes.InvalidArgument, "type URL is required for ADS")
 			}
+
+			if node == nil && req.Node != nil {
+				node = req.Node
+				proxyFeatures = determineSupportedProxyFeatures(req.Node)
+			}
+
 			if handler, ok := handlers[req.TypeUrl]; ok {
-				handler.Recv(req)
+				handler.Recv(req, node, proxyFeatures)
 			}
 		case cfgSnap = <-stateCh:
 			// We got a new config, update the version counter
@@ -367,10 +385,12 @@ func (s *Server) process(stream ADSStream, reqCh <-chan *envoy.DiscoveryRequest)
 }
 
 type xDSType struct {
-	typeURL   string
-	stream    ADSStream
-	req       *envoy.DiscoveryRequest
-	lastNonce string
+	typeURL       string
+	stream        ADSStream
+	req           *envoy.DiscoveryRequest
+	node          *envoycore.Node
+	proxyFeatures supportedProxyFeatures
+	lastNonce     string
 	// lastVersion is the version that was last sent to the proxy. It is needed
 	// because we don't want to send the same version more than once.
 	// req.VersionInfo may be an older version than the most recent once sent in
@@ -379,13 +399,21 @@ type xDSType struct {
 	// last version we sent with a Nack then req.VersionInfo will be the older
 	// version it's hanging on to.
 	lastVersion  uint64
-	resources    func(cfgSnap *proxycfg.ConfigSnapshot, token string) ([]proto.Message, error)
+	resources    func(cInfo connectionInfo, cfgSnap *proxycfg.ConfigSnapshot) ([]proto.Message, error)
 	allowEmptyFn func(cfgSnap *proxycfg.ConfigSnapshot) bool
 }
 
-func (t *xDSType) Recv(req *envoy.DiscoveryRequest) {
+// connectionInfo represents details specific to this connection
+type connectionInfo struct {
+	Token         string
+	ProxyFeatures supportedProxyFeatures
+}
+
+func (t *xDSType) Recv(req *envoy.DiscoveryRequest, node *envoycore.Node, proxyFeatures supportedProxyFeatures) {
 	if t.lastNonce == "" || t.lastNonce == req.GetResponseNonce() {
 		t.req = req
+		t.node = node
+		t.proxyFeatures = proxyFeatures
 	}
 }
 
@@ -397,7 +425,12 @@ func (t *xDSType) SendIfNew(cfgSnap *proxycfg.ConfigSnapshot, version uint64, no
 		// Already sent this version
 		return nil
 	}
-	resources, err := t.resources(cfgSnap, tokenFromStream(t.stream))
+
+	cInfo := connectionInfo{
+		Token:         tokenFromContext(t.stream.Context()),
+		ProxyFeatures: t.proxyFeatures,
+	}
+	resources, err := t.resources(cInfo, cfgSnap)
 	if err != nil {
 		return err
 	}
@@ -436,10 +469,6 @@ func (t *xDSType) SendIfNew(cfgSnap *proxycfg.ConfigSnapshot, version uint64, no
 	return nil
 }
 
-func tokenFromStream(stream ADSStream) string {
-	return tokenFromContext(stream.Context())
-}
-
 func tokenFromContext(ctx context.Context) string {
 	md, ok := metadata.FromIncomingContext(ctx)
 	if !ok {
@@ -459,8 +488,8 @@ func (s *Server) DeltaAggregatedResources(_ envoydisco.AggregatedDiscoveryServic
 
 func deniedResponse(reason string) (*envoyauthz.CheckResponse, error) {
 	return &envoyauthz.CheckResponse{
-		Status: &rpc.Status{
-			Code:    int32(rpc.PERMISSION_DENIED),
+		Status: &rpcstatus.Status{
+			Code:    int32(codes.PermissionDenied),
 			Message: "Denied: " + reason,
 		},
 	}, nil
@@ -532,8 +561,8 @@ func (s *Server) Check(ctx context.Context, r *envoyauthz.CheckRequest) (*envoya
 	s.Logger.Debug("Connect AuthZ ALLOWED", "source", r.Attributes.Source.Principal,
 		"destination", r.Attributes.Destination.Principal, "reason", reason)
 	return &envoyauthz.CheckResponse{
-		Status: &rpc.Status{
-			Code:    int32(rpc.OK),
+		Status: &rpcstatus.Status{
+			Code:    int32(codes.OK),
 			Message: "ALLOWED: " + reason,
 		},
 	}, nil
