@@ -741,7 +741,7 @@ func TestAPI_AgentService(t *testing.T) {
 		WaitTime: 100 * time.Millisecond, // Just long enough to be reliably measurable
 	}
 	start := time.Now()
-	got, qm, err = agent.Service("foo", &opts)
+	_, _, err = agent.Service("foo", &opts)
 	elapsed := time.Since(start)
 	require.NoError(err)
 	require.True(elapsed >= opts.WaitTime)
@@ -1186,15 +1186,26 @@ func TestAPI_AgentMonitor(t *testing.T) {
 		t.Fatalf("err: %v", err)
 	}
 
-	// Wait for the first log message and validate it
-	select {
-	case log := <-logCh:
-		if !(strings.Contains(log, "[INFO]") || strings.Contains(log, "[DEBUG]")) {
-			t.Fatalf("bad: %q", log)
+	retry.Run(t, func(r *retry.R) {
+		{
+			// Register a service to be sure something happens in secs
+			serviceReg := &AgentServiceRegistration{
+				Name: "redis",
+			}
+			if err := agent.ServiceRegister(serviceReg); err != nil {
+				r.Fatalf("err: %v", err)
+			}
 		}
-	case <-time.After(10 * time.Second):
-		t.Fatalf("failed to get a log message")
-	}
+		// Wait for the first log message and validate it
+		select {
+		case log := <-logCh:
+			if !(strings.Contains(log, "[INFO]") || strings.Contains(log, "[DEBUG]")) {
+				r.Fatalf("bad: %q", log)
+			}
+		case <-time.After(10 * time.Second):
+			r.Fatalf("failed to get a log message")
+		}
+	})
 }
 
 func TestAPI_AgentMonitorJSON(t *testing.T) {
@@ -1716,6 +1727,37 @@ func TestAgentService_Register_MeshGateway(t *testing.T) {
 	require.Equal(t, "bar", svc.Proxy.Config["foo"])
 }
 
+func TestAgentService_Register_TerminatingGateway(t *testing.T) {
+	t.Parallel()
+	c, s := makeClient(t)
+	defer s.Stop()
+
+	agent := c.Agent()
+
+	reg := AgentServiceRegistration{
+		Kind:    ServiceKindTerminatingGateway,
+		Name:    "terminating-gateway",
+		Address: "10.1.2.3",
+		Port:    8443,
+		Proxy: &AgentServiceConnectProxyConfig{
+			Config: map[string]interface{}{
+				"foo": "bar",
+			},
+		},
+	}
+
+	err := agent.ServiceRegister(&reg)
+	require.NoError(t, err)
+
+	svc, _, err := agent.Service("terminating-gateway", nil)
+	require.NoError(t, err)
+	require.NotNil(t, svc)
+	require.Equal(t, ServiceKindTerminatingGateway, svc.Kind)
+	require.NotNil(t, svc.Proxy)
+	require.Contains(t, svc.Proxy.Config, "foo")
+	require.Equal(t, "bar", svc.Proxy.Config["foo"])
+}
+
 func TestAgentService_ExposeChecks(t *testing.T) {
 	t.Parallel()
 	c, s := makeClient(t)
@@ -1757,3 +1799,91 @@ func TestAgentService_ExposeChecks(t *testing.T) {
 	require.True(t, svc.Proxy.Expose.Checks)
 	require.Equal(t, path, svc.Proxy.Expose.Paths[0])
 }
+
+func TestMemberACLMode(t *testing.T) {
+	type testCase struct {
+		tagValue     string
+		expectedMode MemberACLMode
+	}
+
+	cases := map[string]testCase{
+		"disabled": {
+			tagValue:     "0",
+			expectedMode: ACLModeDisabled,
+		},
+		"enabled": {
+			tagValue:     "1",
+			expectedMode: ACLModeEnabled,
+		},
+		"legacy": {
+			tagValue:     "2",
+			expectedMode: ACLModeLegacy,
+		},
+		"unknown-3": {
+			tagValue:     "3",
+			expectedMode: ACLModeUnknown,
+		},
+		"unknown-other": {
+			tagValue:     "77",
+			expectedMode: ACLModeUnknown,
+		},
+		"unknown-not-present": {
+			tagValue:     "",
+			expectedMode: ACLModeUnknown,
+		},
+	}
+
+	for name, tcase := range cases {
+		t.Run(name, func(t *testing.T) {
+			tags := map[string]string{}
+
+			if tcase.tagValue != "" {
+				tags[MemberTagKeyACLMode] = tcase.tagValue
+			}
+
+			m := AgentMember{
+				Tags: tags,
+			}
+
+			require.Equal(t, tcase.expectedMode, m.ACLMode())
+		})
+	}
+}
+
+func TestMemberIsConsulServer(t *testing.T) {
+	type testCase struct {
+		tagValue string
+		isServer bool
+	}
+
+	cases := map[string]testCase{
+		"not-present": {
+			tagValue: "",
+			isServer: false,
+		},
+		"server": {
+			tagValue: MemberTagValueRoleServer,
+			isServer: true,
+		},
+		"client": {
+			tagValue: "client",
+			isServer: false,
+		},
+	}
+
+	for name, tcase := range cases {
+		t.Run(name, func(t *testing.T) {
+			tags := map[string]string{}
+
+			if tcase.tagValue != "" {
+				tags[MemberTagKeyRole] = tcase.tagValue
+			}
+
+			m := AgentMember{
+				Tags: tags,
+			}
+
+			require.Equal(t, tcase.isServer, m.IsConsulServer())
+		})
+	}
+}
