@@ -211,7 +211,6 @@ func TestAPI_HealthChecks(t *testing.T) {
 	if err := agent.ServiceRegister(reg); err != nil {
 		t.Fatalf("err: %v", err)
 	}
-	defer agent.ServiceDeregister("foo")
 
 	retry.Run(t, func(r *retry.R) {
 		checks := HealthChecks{
@@ -223,6 +222,8 @@ func TestAPI_HealthChecks(t *testing.T) {
 				ServiceID:   "foo",
 				ServiceName: "foo",
 				ServiceTags: []string{"bar"},
+				Type:        "ttl",
+				Namespace:   defaultNamespace,
 			},
 		}
 
@@ -262,7 +263,6 @@ func TestAPI_HealthChecks_NodeMetaFilter(t *testing.T) {
 	if err := agent.ServiceRegister(reg); err != nil {
 		t.Fatalf("err: %v", err)
 	}
-	defer agent.ServiceDeregister("foo")
 
 	retry.Run(t, func(r *retry.R) {
 		checks, meta, err := health.Checks("foo", &QueryOptions{NodeMeta: meta})
@@ -272,8 +272,11 @@ func TestAPI_HealthChecks_NodeMetaFilter(t *testing.T) {
 		if meta.LastIndex == 0 {
 			r.Fatalf("bad: %v", meta)
 		}
-		if len(checks) == 0 {
-			r.Fatalf("Bad: %v", checks)
+		if len(checks) != 1 {
+			r.Fatalf("expected 1 check, got %d", len(checks))
+		}
+		if checks[0].Type != "ttl" {
+			r.Fatalf("expected type ttl, got %s", checks[0].Type)
 		}
 	})
 }
@@ -349,13 +352,18 @@ func TestAPI_HealthService_SingleTag(t *testing.T) {
 		},
 	}
 	require.NoError(t, agent.ServiceRegister(reg))
-	defer agent.ServiceDeregister("foo1")
 	retry.Run(t, func(r *retry.R) {
 		services, meta, err := health.Service("foo", "bar", true, nil)
-		require.NoError(t, err)
-		require.NotEqual(t, meta.LastIndex, 0)
-		require.Len(t, services, 1)
-		require.Equal(t, services[0].Service.ID, "foo1")
+		require.NoError(r, err)
+		require.NotEqual(r, meta.LastIndex, 0)
+		require.Len(r, services, 1)
+		require.Equal(r, services[0].Service.ID, "foo1")
+
+		for _, check := range services[0].Checks {
+			if check.CheckID == "service:foo1" && check.Type != "ttl" {
+				r.Fatalf("expected type ttl, got %s", check.Type)
+			}
+		}
 	})
 }
 func TestAPI_HealthService_MultipleTags(t *testing.T) {
@@ -379,7 +387,6 @@ func TestAPI_HealthService_MultipleTags(t *testing.T) {
 		},
 	}
 	require.NoError(t, agent.ServiceRegister(reg))
-	defer agent.ServiceDeregister("foo1")
 
 	reg2 := &AgentServiceRegistration{
 		Name: "foo",
@@ -391,25 +398,24 @@ func TestAPI_HealthService_MultipleTags(t *testing.T) {
 		},
 	}
 	require.NoError(t, agent.ServiceRegister(reg2))
-	defer agent.ServiceDeregister("foo2")
 
 	// Test searching with one tag (two results)
 	retry.Run(t, func(r *retry.R) {
 		services, meta, err := health.ServiceMultipleTags("foo", []string{"bar"}, true, nil)
 
-		require.NoError(t, err)
-		require.NotEqual(t, meta.LastIndex, 0)
-		require.Len(t, services, 2)
+		require.NoError(r, err)
+		require.NotEqual(r, meta.LastIndex, 0)
+		require.Len(r, services, 2)
 	})
 
 	// Test searching with two tags (one result)
 	retry.Run(t, func(r *retry.R) {
 		services, meta, err := health.ServiceMultipleTags("foo", []string{"bar", "v2"}, true, nil)
 
-		require.NoError(t, err)
-		require.NotEqual(t, meta.LastIndex, 0)
-		require.Len(t, services, 1)
-		require.Equal(t, services[0].Service.ID, "foo2")
+		require.NoError(r, err)
+		require.NotEqual(r, meta.LastIndex, 0)
+		require.Len(r, services, 1)
+		require.Equal(r, services[0].Service.ID, "foo2")
 	})
 }
 
@@ -421,15 +427,17 @@ func TestAPI_HealthService_NodeMetaFilter(t *testing.T) {
 	})
 	defer s.Stop()
 
+	s.WaitForSerfCheck(t)
+
 	health := c.Health()
 	retry.Run(t, func(r *retry.R) {
 		// consul service should always exist...
 		checks, meta, err := health.Service("consul", "", true, &QueryOptions{NodeMeta: meta})
-		require.NoError(t, err)
-		require.NotEqual(t, meta.LastIndex, 0)
-		require.NotEqual(t, len(checks), 0)
-		require.Equal(t, checks[0].Node.Datacenter, "dc1")
-		require.Contains(t, checks[0].Node.TaggedAddresses, "wan")
+		require.NoError(r, err)
+		require.NotEqual(r, meta.LastIndex, 0)
+		require.NotEqual(r, len(checks), 0)
+		require.Equal(r, checks[0].Node.Datacenter, "dc1")
+		require.Contains(r, checks[0].Node.TaggedAddresses, "wan")
 	})
 }
 
@@ -475,7 +483,6 @@ func TestAPI_HealthConnect(t *testing.T) {
 	}
 	err := agent.ServiceRegister(reg)
 	require.NoError(t, err)
-	defer agent.ServiceDeregister("foo")
 
 	// Register the proxy
 	proxyReg := &AgentServiceRegistration{
@@ -488,7 +495,6 @@ func TestAPI_HealthConnect(t *testing.T) {
 	}
 	err = agent.ServiceRegister(proxyReg)
 	require.NoError(t, err)
-	defer agent.ServiceDeregister("foo-proxy")
 
 	retry.Run(t, func(r *retry.R) {
 		services, meta, err := health.Connect("foo", "", true, nil)
@@ -533,6 +539,67 @@ func TestAPI_HealthConnect_Filter(t *testing.T) {
 	require.Len(t, services, 1)
 }
 
+func TestAPI_HealthIngress(t *testing.T) {
+	t.Parallel()
+	c, s := makeClient(t)
+	defer s.Stop()
+
+	agent := c.Agent()
+	health := c.Health()
+
+	s.WaitForSerfCheck(t)
+
+	// Make a service with a proxy
+	reg := &AgentServiceRegistration{
+		Name: "foo",
+		Port: 8000,
+	}
+	err := agent.ServiceRegister(reg)
+	require.NoError(t, err)
+
+	// Register the gateway
+	gatewayReg := &AgentServiceRegistration{
+		Name: "foo-gateway",
+		Port: 8001,
+		Kind: ServiceKindIngressGateway,
+	}
+	err = agent.ServiceRegister(gatewayReg)
+	require.NoError(t, err)
+
+	// Associate service and gateway
+	gatewayConfig := &IngressGatewayConfigEntry{
+		Kind: IngressGateway,
+		Name: "foo-gateway",
+		Listeners: []IngressListener{
+			{
+				Port:     2222,
+				Protocol: "tcp",
+				Services: []IngressService{
+					{
+						Name: "foo",
+					},
+				},
+			},
+		},
+	}
+	_, wm, err := c.ConfigEntries().Set(gatewayConfig, nil)
+	require.NoError(t, err)
+	require.NotNil(t, wm)
+
+	retry.Run(t, func(r *retry.R) {
+		services, meta, err := health.Ingress("foo", true, nil)
+		require.NoError(r, err)
+
+		require.NotZero(r, meta.LastIndex)
+
+		// Should be exactly 1 service - the original shouldn't show up as a connect
+		// endpoint, only it's proxy.
+		require.Len(r, services, 1)
+		require.Equal(r, services[0].Node.Datacenter, "dc1")
+		require.Equal(r, services[0].Service.Service, gatewayReg.Name)
+	})
+}
+
 func TestAPI_HealthState(t *testing.T) {
 	t.Parallel()
 	c, s := makeClient(t)
