@@ -2,6 +2,7 @@ package authmethodupdate
 
 import (
 	"encoding/json"
+	"io"
 	"io/ioutil"
 	"os"
 	"path/filepath"
@@ -16,7 +17,6 @@ import (
 	"github.com/hashicorp/consul/testrpc"
 	"github.com/hashicorp/go-uuid"
 	"github.com/mitchellh/cli"
-	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
 	// activate testing auth method
@@ -37,7 +37,7 @@ func TestAuthMethodUpdateCommand(t *testing.T) {
 	testDir := testutil.TempDir(t, "acl")
 	defer os.RemoveAll(testDir)
 
-	a := agent.NewTestAgent(t, t.Name(), `
+	a := agent.NewTestAgent(t, `
 	primary_datacenter = "dc1"
 	acl {
 		enabled = true
@@ -66,10 +66,11 @@ func TestAuthMethodUpdateCommand(t *testing.T) {
 	})
 
 	t.Run("update nonexistent method", func(t *testing.T) {
+		name := getTestName(t)
 		args := []string{
 			"-http-addr=" + a.HTTPAddr(),
 			"-token=root",
-			"-name=test",
+			"-name", name,
 		}
 
 		ui := cli.NewMockUi()
@@ -99,14 +100,17 @@ func TestAuthMethodUpdateCommand(t *testing.T) {
 		return methodName
 	}
 
-	t.Run("update all fields", func(t *testing.T) {
-		name := createAuthMethod(t)
+	finalName := createAuthMethod(t)
 
+	t.Run("update all fields", func(t *testing.T) {
+		name := finalName
 		args := []string{
 			"-http-addr=" + a.HTTPAddr(),
 			"-token=root",
 			"-name=" + name,
+			"-display-name", "updated display",
 			"-description", "updated description",
+			"-config", `{ "SessionID": "foo" }`,
 		}
 
 		ui := cli.NewMockUi()
@@ -116,13 +120,50 @@ func TestAuthMethodUpdateCommand(t *testing.T) {
 		require.Equal(t, code, 0)
 		require.Empty(t, ui.ErrorWriter.String())
 
-		method, _, err := client.ACL().AuthMethodRead(
-			name,
-			&api.QueryOptions{Token: "root"},
-		)
-		require.NoError(t, err)
-		require.NotNil(t, method)
-		require.Equal(t, "updated description", method.Description)
+		got := getTestMethod(t, client, name)
+		expect := &api.ACLAuthMethod{
+			Name:        name,
+			Type:        "testing",
+			DisplayName: "updated display",
+			Description: "updated description",
+			Config: map[string]interface{}{
+				"SessionID": "foo",
+			},
+		}
+		require.Equal(t, expect, got)
+	})
+
+	t.Run("update config field and prove no merging happens", func(t *testing.T) {
+		name := finalName
+		args := []string{
+			"-http-addr=" + a.HTTPAddr(),
+			"-token=root",
+			"-name=" + name,
+			"-display-name", "updated display",
+			"-description", "updated description",
+			"-config", `{ "Data": { "foo": "bar"} }`,
+		}
+
+		ui := cli.NewMockUi()
+		cmd := New(ui)
+
+		code := cmd.Run(args)
+		require.Equal(t, code, 0)
+		require.Empty(t, ui.ErrorWriter.String())
+
+		got := getTestMethod(t, client, name)
+		expect := &api.ACLAuthMethod{
+			Name:        name,
+			Type:        "testing",
+			DisplayName: "updated display",
+			Description: "updated description",
+			Config: map[string]interface{}{
+				"Data": map[string]interface{}{
+					"foo": "bar",
+				},
+			},
+		}
+		require.Equal(t, expect, got)
 	})
 }
 
@@ -132,7 +173,7 @@ func TestAuthMethodUpdateCommand_JSON(t *testing.T) {
 	testDir := testutil.TempDir(t, "acl")
 	defer os.RemoveAll(testDir)
 
-	a := agent.NewTestAgent(t, t.Name(), `
+	a := agent.NewTestAgent(t, `
 	primary_datacenter = "dc1"
 	acl {
 		enabled = true
@@ -187,6 +228,7 @@ func TestAuthMethodUpdateCommand_JSON(t *testing.T) {
 			"-http-addr=" + a.HTTPAddr(),
 			"-token=root",
 			"-name=" + name,
+			"-display-name", "updated display",
 			"-description", "updated description",
 			"-format=json",
 		}
@@ -195,22 +237,23 @@ func TestAuthMethodUpdateCommand_JSON(t *testing.T) {
 		cmd := New(ui)
 
 		code := cmd.Run(args)
+		output := ui.OutputWriter.String()
+
 		require.Equal(t, code, 0)
 		require.Empty(t, ui.ErrorWriter.String())
 
-		method, _, err := client.ACL().AuthMethodRead(
-			name,
-			&api.QueryOptions{Token: "root"},
-		)
-		require.NoError(t, err)
-		require.NotNil(t, method)
-		require.Equal(t, "updated description", method.Description)
-
-		output := ui.OutputWriter.String()
-
 		var jsonOutput json.RawMessage
-		err = json.Unmarshal([]byte(output), &jsonOutput)
-		assert.NoError(t, err)
+		require.NoError(t, json.Unmarshal([]byte(output), &jsonOutput))
+
+		got := getTestMethod(t, client, name)
+		expect := &api.ACLAuthMethod{
+			Name:        name,
+			Type:        "testing",
+			DisplayName: "updated display",
+			Description: "updated description",
+			Config:      map[string]interface{}{},
+		}
+		require.Equal(t, expect, got)
 	})
 }
 
@@ -220,7 +263,7 @@ func TestAuthMethodUpdateCommand_noMerge(t *testing.T) {
 	testDir := testutil.TempDir(t, "acl")
 	defer os.RemoveAll(testDir)
 
-	a := agent.NewTestAgent(t, t.Name(), `
+	a := agent.NewTestAgent(t, `
 	primary_datacenter = "dc1"
 	acl {
 		enabled = true
@@ -250,11 +293,12 @@ func TestAuthMethodUpdateCommand_noMerge(t *testing.T) {
 	})
 
 	t.Run("update nonexistent method", func(t *testing.T) {
+		name := getTestName(t)
 		args := []string{
 			"-http-addr=" + a.HTTPAddr(),
 			"-token=root",
 			"-no-merge",
-			"-name=test",
+			"-name", name,
 		}
 
 		ui := cli.NewMockUi()
@@ -292,6 +336,7 @@ func TestAuthMethodUpdateCommand_noMerge(t *testing.T) {
 			"-token=root",
 			"-no-merge",
 			"-name=" + name,
+			"-display-name", "updated display",
 			"-description", "updated description",
 		}
 
@@ -302,13 +347,14 @@ func TestAuthMethodUpdateCommand_noMerge(t *testing.T) {
 		require.Equal(t, code, 0, "err: %s", ui.ErrorWriter.String())
 		require.Empty(t, ui.ErrorWriter.String())
 
-		method, _, err := client.ACL().AuthMethodRead(
-			name,
-			&api.QueryOptions{Token: "root"},
-		)
-		require.NoError(t, err)
-		require.NotNil(t, method)
-		require.Equal(t, "updated description", method.Description)
+		got := getTestMethod(t, client, name)
+		expect := &api.ACLAuthMethod{
+			Name:        name,
+			Type:        "testing",
+			DisplayName: "updated display",
+			Description: "updated description",
+		}
+		require.Equal(t, expect, got)
 	})
 }
 
@@ -318,7 +364,7 @@ func TestAuthMethodUpdateCommand_k8s(t *testing.T) {
 	testDir := testutil.TempDir(t, "acl")
 	defer os.RemoveAll(testDir)
 
-	a := agent.NewTestAgent(t, t.Name(), `
+	a := agent.NewTestAgent(t, `
 	primary_datacenter = "dc1"
 	acl {
 		enabled = true
@@ -366,6 +412,7 @@ func TestAuthMethodUpdateCommand_k8s(t *testing.T) {
 			"-http-addr=" + a.HTTPAddr(),
 			"-token=root",
 			"-name=" + name,
+			"-display-name", "updated display",
 			"-description", "updated description",
 			"-kubernetes-host", "https://foo-new.internal:8443",
 			"-kubernetes-ca-cert", ca2.RootCert,
@@ -379,15 +426,22 @@ func TestAuthMethodUpdateCommand_k8s(t *testing.T) {
 		require.Equal(t, code, 0)
 		require.Empty(t, ui.ErrorWriter.String())
 
-		method, _, err := client.ACL().AuthMethodRead(
-			name,
-			&api.QueryOptions{Token: "root"},
-		)
-		require.NoError(t, err)
-		require.NotNil(t, method)
-		require.Equal(t, "updated description", method.Description)
+		got := getTestMethod(t, client, name)
+		expect := &api.ACLAuthMethod{
+			Name:        name,
+			Type:        "kubernetes",
+			DisplayName: "updated display",
+			Description: "updated description",
+			Config: map[string]interface{}{
+				"Host":              "https://foo-new.internal:8443",
+				"CACert":            ca2.RootCert,
+				"ServiceAccountJWT": acl.TestKubernetesJWT_B,
+			},
+		}
+		require.Equal(t, expect, got)
 
-		config, err := api.ParseKubernetesAuthMethodConfig(method.Config)
+		// also just double check our convenience parsing
+		config, err := api.ParseKubernetesAuthMethodConfig(got.Config)
 		require.NoError(t, err)
 		require.Equal(t, "https://foo-new.internal:8443", config.Host)
 		require.Equal(t, ca2.RootCert, config.CACert)
@@ -545,7 +599,7 @@ func TestAuthMethodUpdateCommand_k8s_noMerge(t *testing.T) {
 	testDir := testutil.TempDir(t, "acl")
 	defer os.RemoveAll(testDir)
 
-	a := agent.NewTestAgent(t, t.Name(), `
+	a := agent.NewTestAgent(t, `
 	primary_datacenter = "dc1"
 	acl {
 		enabled = true
@@ -726,3 +780,165 @@ func TestAuthMethodUpdateCommand_k8s_noMerge(t *testing.T) {
 		require.Equal(t, acl.TestKubernetesJWT_B, config.ServiceAccountJWT)
 	})
 }
+
+func TestAuthMethodUpdateCommand_config(t *testing.T) {
+	t.Parallel()
+	testDir := testutil.TempDir(t, "auth-method")
+	defer os.RemoveAll(testDir)
+
+	a := agent.NewTestAgent(t, `
+	primary_datacenter = "dc1"
+	acl {
+		enabled = true
+		tokens {
+			master = "root"
+		}
+	}`)
+
+	defer a.Shutdown()
+	testrpc.WaitForLeader(t, a.RPC, "dc1")
+
+	client := a.Client()
+
+	createAuthMethod := func(t *testing.T) string {
+		id, err := uuid.GenerateUUID()
+		require.NoError(t, err)
+
+		methodName := "test" + id
+
+		_, _, err = client.ACL().AuthMethodCreate(
+			&api.ACLAuthMethod{
+				Name:        methodName,
+				Type:        "testing",
+				Description: "test",
+				Config: map[string]interface{}{
+					"SessionID": "big",
+				},
+			},
+			&api.WriteOptions{Token: "root"},
+		)
+		require.NoError(t, err)
+
+		return methodName
+	}
+
+	readUpdate := func(t *testing.T, methodName string) {
+
+		method, _, err := client.ACL().AuthMethodRead(
+			methodName,
+			&api.QueryOptions{Token: "root"},
+		)
+		require.NoError(t, err)
+		require.NotNil(t, method)
+		require.Equal(t, "update", method.Config["SessionID"])
+	}
+
+	t.Run("config file", func(t *testing.T) {
+		methodName := createAuthMethod(t)
+		configFile := filepath.Join(testDir, "config.json")
+		jsonConfig := `{"SessionID":"update"}`
+		require.NoError(t, ioutil.WriteFile(configFile, []byte(jsonConfig), 0644))
+
+		args := []string{
+			"-http-addr=" + a.HTTPAddr(),
+			"-token=root",
+			"-name=" + methodName,
+			"-no-merge=true",
+			"-config=@" + configFile,
+		}
+		ui := cli.NewMockUi()
+		cmd := New(ui)
+		code := cmd.Run(args)
+		require.Equal(t, 0, code)
+		require.Empty(t, ui.ErrorWriter.String())
+		readUpdate(t, methodName)
+	})
+
+	t.Run("config stdin", func(t *testing.T) {
+		methodName := createAuthMethod(t)
+		ui := cli.NewMockUi()
+		cmd := New(ui)
+		stdinR, stdinW := io.Pipe()
+		cmd.testStdin = stdinR
+
+		go func() {
+			stdinW.Write([]byte(`{"SessionID":"update"}`))
+			stdinW.Close()
+		}()
+		args := []string{
+			"-http-addr=" + a.HTTPAddr(),
+			"-token=root",
+			"-name=" + methodName,
+			"-no-merge=true",
+			"-config=-",
+		}
+
+		code := cmd.Run(args)
+		require.Equal(t, 0, code)
+		require.Empty(t, ui.ErrorWriter.String())
+		readUpdate(t, methodName)
+	})
+
+	t.Run("config string", func(t *testing.T) {
+		methodName := createAuthMethod(t)
+		args := []string{
+			"-http-addr=" + a.HTTPAddr(),
+			"-token=root",
+			"-name=" + methodName,
+			"-no-merge=true",
+			"-config=" + `{"SessionID":"update"}`,
+		}
+		ui := cli.NewMockUi()
+		cmd := New(ui)
+		code := cmd.Run(args)
+		require.Equal(t, 0, code)
+		require.Empty(t, ui.ErrorWriter.String())
+		readUpdate(t, methodName)
+	})
+	t.Run("config with no merge", func(t *testing.T) {
+		methodName := createAuthMethod(t)
+		args := []string{
+			"-http-addr=" + a.HTTPAddr(),
+			"-token=root",
+			"-name=" + methodName,
+			"-no-merge=false",
+			"-config=" + `{"SessionID":"update"}`,
+		}
+		ui := cli.NewMockUi()
+		cmd := New(ui)
+		code := cmd.Run(args)
+		require.Equal(t, 0, code)
+		require.Empty(t, ui.ErrorWriter.String())
+		readUpdate(t, methodName)
+	})
+}
+
+func getTestMethod(t *testing.T, client *api.Client, methodName string) *api.ACLAuthMethod {
+	t.Helper()
+
+	method, _, err := client.ACL().AuthMethodRead(
+		methodName,
+		&api.QueryOptions{Token: "root"},
+	)
+	require.NoError(t, err)
+	require.NotNil(t, method)
+
+	// zero these out since we don't really care
+	method.CreateIndex = 0
+	method.ModifyIndex = 0
+
+	if method.Namespace == "default" {
+		method.Namespace = ""
+	}
+
+	return method
+}
+
+func getTestName(t *testing.T) string {
+	t.Helper()
+
+	id, err := uuid.GenerateUUID()
+	require.NoError(t, err)
+	return "test-" + id
+
+}
