@@ -3,9 +3,10 @@ package bindingrulecreate
 import (
 	"flag"
 	"fmt"
+	"strings"
 
 	"github.com/hashicorp/consul/api"
-	"github.com/hashicorp/consul/command/acl"
+	"github.com/hashicorp/consul/command/acl/bindingrule"
 	"github.com/hashicorp/consul/command/flags"
 	"github.com/mitchellh/cli"
 )
@@ -29,6 +30,7 @@ type cmd struct {
 	bindName       string
 
 	showMeta bool
+	format   string
 }
 
 func (c *cmd) init() {
@@ -75,6 +77,12 @@ func (c *cmd) init() {
 		"Name to bind on match. Can use ${var} interpolation. "+
 			"This flag is required.",
 	)
+	c.flags.StringVar(
+		&c.format,
+		"format",
+		bindingrule.PrettyFormat,
+		fmt.Sprintf("Output format {%s}", strings.Join(bindingrule.GetSupportedFormats(), "|")),
+	)
 
 	c.http = &flags.HTTPFlags{}
 	flags.Merge(c.flags, c.http.ClientFlags())
@@ -122,7 +130,21 @@ func (c *cmd) Run(args []string) int {
 		return 1
 	}
 
-	acl.PrintBindingRule(rule, c.UI, c.showMeta)
+	formatter, err := bindingrule.NewFormatter(c.format, c.showMeta)
+	if err != nil {
+		c.UI.Error(err.Error())
+		return 1
+	}
+
+	out, err := formatter.FormatBindingRule(rule)
+	if err != nil {
+		c.UI.Error(err.Error())
+		return 1
+	}
+	if out != "" {
+		c.UI.Info(out)
+	}
+
 	return 0
 }
 
