@@ -3,8 +3,9 @@ package bindingrulelist
 import (
 	"flag"
 	"fmt"
+	"strings"
 
-	"github.com/hashicorp/consul/command/acl"
+	"github.com/hashicorp/consul/command/acl/bindingrule"
 	"github.com/hashicorp/consul/command/flags"
 	"github.com/mitchellh/cli"
 )
@@ -24,6 +25,7 @@ type cmd struct {
 	authMethodName string
 
 	showMeta bool
+	format   string
 }
 
 func (c *cmd) init() {
@@ -44,6 +46,13 @@ func (c *cmd) init() {
 		"Only show rules linked to the auth method with the given name.",
 	)
 
+	c.flags.StringVar(
+		&c.format,
+		"format",
+		bindingrule.PrettyFormat,
+		fmt.Sprintf("Output format {%s}", strings.Join(bindingrule.GetSupportedFormats(), "|")),
+	)
+
 	c.http = &flags.HTTPFlags{}
 	flags.Merge(c.flags, c.http.ClientFlags())
 	flags.Merge(c.flags, c.http.ServerFlags())
@@ -68,8 +77,19 @@ func (c *cmd) Run(args []string) int {
 		return 1
 	}
 
-	for _, rule := range rules {
-		acl.PrintBindingRuleListEntry(rule, c.UI, c.showMeta)
+	formatter, err := bindingrule.NewFormatter(c.format, c.showMeta)
+	if err != nil {
+		c.UI.Error(err.Error())
+		return 1
+	}
+
+	out, err := formatter.FormatBindingRuleList(rules)
+	if err != nil {
+		c.UI.Error(err.Error())
+		return 1
+	}
+	if out != "" {
+		c.UI.Info(out)
 	}
 
 	return 0
