@@ -3,9 +3,11 @@ package bindingruleupdate
 import (
 	"flag"
 	"fmt"
+	"strings"
 
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/command/acl"
+	"github.com/hashicorp/consul/command/acl/bindingrule"
 	"github.com/hashicorp/consul/command/flags"
 	"github.com/mitchellh/cli"
 )
@@ -31,6 +33,7 @@ type cmd struct {
 
 	noMerge  bool
 	showMeta bool
+	format   string
 }
 
 func (c *cmd) init() {
@@ -88,6 +91,13 @@ func (c *cmd) init() {
 			"with the exception of the binding rule ID which is immutable.",
 	)
 
+	c.flags.StringVar(
+		&c.format,
+		"format",
+		bindingrule.PrettyFormat,
+		fmt.Sprintf("Output format {%s}", strings.Join(bindingrule.GetSupportedFormats(), "|")),
+	)
+
 	c.http = &flags.HTTPFlags{}
 	flags.Merge(c.flags, c.http.ClientFlags())
 	flags.Merge(c.flags, c.http.ServerFlags())
@@ -171,8 +181,21 @@ func (c *cmd) Run(args []string) int {
 		return 1
 	}
 
-	c.UI.Info(fmt.Sprintf("Binding rule updated successfully"))
-	acl.PrintBindingRule(rule, c.UI, c.showMeta)
+	formatter, err := bindingrule.NewFormatter(c.format, c.showMeta)
+	if err != nil {
+		c.UI.Error(err.Error())
+		return 1
+	}
+
+	out, err := formatter.FormatBindingRule(rule)
+	if err != nil {
+		c.UI.Error(err.Error())
+		return 1
+	}
+	if out != "" {
+		c.UI.Info(out)
+	}
+
 	return 0
 }
 
