@@ -4,10 +4,12 @@ import (
 	"flag"
 	"fmt"
 	"io"
+	"strings"
 
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/api"
 	aclhelpers "github.com/hashicorp/consul/command/acl"
+	"github.com/hashicorp/consul/command/acl/policy"
 	"github.com/hashicorp/consul/command/flags"
 	"github.com/hashicorp/consul/command/helpers"
 	"github.com/mitchellh/cli"
@@ -33,6 +35,7 @@ type cmd struct {
 	fromToken     string
 	tokenIsSecret bool
 	showMeta      bool
+	format        string
 
 	testStdin io.Reader
 }
@@ -53,6 +56,12 @@ func (c *cmd) init() {
 		"Similar to the -rules option the token to use can be loaded from stdin or from a file")
 	c.flags.BoolVar(&c.tokenIsSecret, "token-secret", false, "Indicates the token provided with "+
 		"-from-token is a SecretID and not an AccessorID")
+	c.flags.StringVar(
+		&c.format,
+		"format",
+		policy.PrettyFormat,
+		fmt.Sprintf("Output format {%s}", strings.Join(policy.GetSupportedFormats(), "|")),
+	)
 
 	c.http = &flags.HTTPFlags{}
 	flags.Merge(c.flags, c.http.ClientFlags())
@@ -114,13 +123,26 @@ func (c *cmd) Run(args []string) int {
 		Rules:       rules,
 	}
 
-	policy, _, err := client.ACL().PolicyCreate(newPolicy, nil)
+	p, _, err := client.ACL().PolicyCreate(newPolicy, nil)
 	if err != nil {
 		c.UI.Error(fmt.Sprintf("Failed to create new policy: %v", err))
 		return 1
 	}
 
-	aclhelpers.PrintPolicy(policy, c.UI, c.showMeta)
+	formatter, err := policy.NewFormatter(c.format, c.showMeta)
+	if err != nil {
+		c.UI.Error(err.Error())
+		return 1
+	}
+	out, err := formatter.FormatPolicy(p)
+	if err != nil {
+		c.UI.Error(err.Error())
+		return 1
+	}
+	if out != "" {
+		c.UI.Info(out)
+	}
+
 	return 0
 }
 
