@@ -3,8 +3,9 @@ package rolelist
 import (
 	"flag"
 	"fmt"
+	"strings"
 
-	"github.com/hashicorp/consul/command/acl"
+	"github.com/hashicorp/consul/command/acl/role"
 	"github.com/hashicorp/consul/command/flags"
 	"github.com/mitchellh/cli"
 )
@@ -22,12 +23,19 @@ type cmd struct {
 	help  string
 
 	showMeta bool
+	format   string
 }
 
 func (c *cmd) init() {
 	c.flags = flag.NewFlagSet("", flag.ContinueOnError)
 	c.flags.BoolVar(&c.showMeta, "meta", false, "Indicates that policy metadata such "+
 		"as the content hash and raft indices should be shown for each entry")
+	c.flags.StringVar(
+		&c.format,
+		"format",
+		role.PrettyFormat,
+		fmt.Sprintf("Output format {%s}", strings.Join(role.GetSupportedFormats(), "|")),
+	)
 
 	c.http = &flags.HTTPFlags{}
 	flags.Merge(c.flags, c.http.ClientFlags())
@@ -53,8 +61,18 @@ func (c *cmd) Run(args []string) int {
 		return 1
 	}
 
-	for _, role := range roles {
-		acl.PrintRoleListEntry(role, c.UI, c.showMeta)
+	formatter, err := role.NewFormatter(c.format, c.showMeta)
+	if err != nil {
+		c.UI.Error(err.Error())
+		return 1
+	}
+	out, err := formatter.FormatRoleList(roles)
+	if err != nil {
+		c.UI.Error(err.Error())
+		return 1
+	}
+	if out != "" {
+		c.UI.Info(out)
 	}
 
 	return 0
