@@ -66,7 +66,7 @@ func TestTokenCloneCommand_Pretty(t *testing.T) {
 	testDir := testutil.TempDir(t, "acl")
 	defer os.RemoveAll(testDir)
 
-	a := agent.NewTestAgent(t, t.Name(), `
+	a := agent.NewTestAgent(t, `
    primary_datacenter = "dc1"
    acl {
       enabled = true
@@ -173,7 +173,7 @@ func TestTokenCloneCommand_JSON(t *testing.T) {
 	testDir := testutil.TempDir(t, "acl")
 	defer os.RemoveAll(testDir)
 
-	a := agent.NewTestAgent(t, t.Name(), `
+	a := agent.NewTestAgent(t, `
    primary_datacenter = "dc1"
    acl {
       enabled = true
