@@ -24,13 +24,13 @@ func TestTokenCreateCommand_noTabs(t *testing.T) {
 
 func TestTokenCreateCommand_Pretty(t *testing.T) {
 	t.Parallel()
-	require := require.New(t)
 
 	testDir := testutil.TempDir(t, "acl")
 	defer os.RemoveAll(testDir)
 
-	a := agent.NewTestAgent(t, t.Name(), `
+	a := agent.NewTestAgent(t, `
 	primary_datacenter = "dc1"
+	node_name = "test-node"
 	acl {
 		enabled = true
 		tokens {
@@ -41,9 +41,6 @@ func TestTokenCreateCommand_Pretty(t *testing.T) {
 	defer a.Shutdown()
 	testrpc.WaitForLeader(t, a.RPC, "dc1")
 
-	ui := cli.NewMockUi()
-	cmd := New(ui)
-
 	// Create a policy
 	client := a.Client()
 
@@ -51,66 +48,75 @@ func TestTokenCreateCommand_Pretty(t *testing.T) {
 		&api.ACLPolicy{Name: "test-policy"},
 		&api.WriteOptions{Token: "root"},
 	)
-	require.NoError(err)
+	require.NoError(t, err)
+
+	run := func(t *testing.T, args []string) *api.ACLToken {
+		ui := cli.NewMockUi()
+		cmd := New(ui)
+
+		code := cmd.Run(append(args, "-format=json"))
+		require.Equal(t, 0, code)
+		require.Empty(t, ui.ErrorWriter.String())
+
+		var token api.ACLToken
+		require.NoError(t, json.Unmarshal(ui.OutputWriter.Bytes(), &token))
+		return &token
+	}
 
 	// create with policy by name
-	{
-		args := []string{
+	t.Run("policy-name", func(t *testing.T) {
+		_ = run(t, []string{
 			"-http-addr=" + a.HTTPAddr(),
 			"-token=root",
 			"-policy-name=" + policy.Name,
 			"-description=test token",
-		}
-
-		code := cmd.Run(args)
-		require.Equal(code, 0)
-		require.Empty(ui.ErrorWriter.String())
-	}
+		})
+	})
 
 	// create with policy by id
-	{
-		args := []string{
+	t.Run("policy-id", func(t *testing.T) {
+		_ = run(t, []string{
 			"-http-addr=" + a.HTTPAddr(),
 			"-token=root",
 			"-policy-id=" + policy.ID,
 			"-description=test token",
-		}
+		})
+	})
 
-		code := cmd.Run(args)
-		require.Empty(ui.ErrorWriter.String())
-		require.Equal(code, 0)
-	}
+	// create with a node identity
+	t.Run("node-identity", func(t *testing.T) {
+		token := run(t, []string{
+			"-http-addr=" + a.HTTPAddr(),
+			"-token=root",
+			"-node-identity=" + a.Config.NodeName + ":" + a.Config.Datacenter,
+		})
+
+		conf := api.DefaultConfig()
+		conf.Address = a.HTTPAddr()
+		conf.Token = token.SecretID
+		client, err := api.NewClient(conf)
+		require.NoError(t, err)
+
+		nodes, _, err := client.Catalog().Nodes(nil)
+		require.NoError(t, err)
+		require.Len(t, nodes, 1)
+		require.Equal(t, a.Config.NodeName, nodes[0].Node)
+	})
 
 	// create with accessor and secret
-	{
-		args := []string{
+	t.Run("predefined-ids", func(t *testing.T) {
+		token := run(t, []string{
 			"-http-addr=" + a.HTTPAddr(),
 			"-token=root",
 			"-policy-id=" + policy.ID,
 			"-description=test token",
 			"-accessor=3d852bb8-5153-4388-a3ca-8ca78661889f",
 			"-secret=3a69a8d8-c4d4-485d-9b19-b5b61648ea0c",
-		}
+		})
 
-		code := cmd.Run(args)
-		require.Empty(ui.ErrorWriter.String())
-		require.Equal(code, 0)
-
-		conf := api.DefaultConfig()
-		conf.Address = a.HTTPAddr()
-		conf.Token = "root"
-
-		// going to use the API client to grab the token - we could potentially try to grab the values
-		// out of the command output but this seems easier.
-		client, err := api.NewClient(conf)
-		require.NoError(err)
-		require.NotNil(client)
-
-		token, _, err := client.ACL().TokenRead("3d852bb8-5153-4388-a3ca-8ca78661889f", nil)
-		require.NoError(err)
-		require.Equal("3d852bb8-5153-4388-a3ca-8ca78661889f", token.AccessorID)
-		require.Equal("3a69a8d8-c4d4-485d-9b19-b5b61648ea0c", token.SecretID)
-	}
+		require.Equal(t, "3d852bb8-5153-4388-a3ca-8ca78661889f", token.AccessorID)
+		require.Equal(t, "3a69a8d8-c4d4-485d-9b19-b5b61648ea0c", token.SecretID)
+	})
 }
 
 func TestTokenCreateCommand_JSON(t *testing.T) {
@@ -120,7 +126,7 @@ func TestTokenCreateCommand_JSON(t *testing.T) {
 	testDir := testutil.TempDir(t, "acl")
 	defer os.RemoveAll(testDir)
 
-	a := agent.NewTestAgent(t, t.Name(), `
+	a := agent.NewTestAgent(t, `
 	primary_datacenter = "dc1"
 	acl {
 		enabled = true
