@@ -6,8 +6,6 @@ import (
 	"strings"
 	"testing"
 
-	"github.com/stretchr/testify/require"
-
 	"github.com/hashicorp/consul/agent"
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/sdk/testutil"
@@ -15,6 +13,7 @@ import (
 	"github.com/hashicorp/consul/testrpc"
 	"github.com/mitchellh/cli"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 )
 
 func TestTokenUpdateCommand_noTabs(t *testing.T) {
@@ -27,14 +26,11 @@ func TestTokenUpdateCommand_noTabs(t *testing.T) {
 
 func TestTokenUpdateCommand(t *testing.T) {
 	t.Parallel()
-	assert := assert.New(t)
-	// Alias because we need to access require package in Retry below
-	req := require.New(t)
 
 	testDir := testutil.TempDir(t, "acl")
 	defer os.RemoveAll(testDir)
 
-	a := agent.NewTestAgent(t, t.Name(), `
+	a := agent.NewTestAgent(t, `
 	primary_datacenter = "dc1"
 	acl {
 		enabled = true
@@ -46,8 +42,6 @@ func TestTokenUpdateCommand(t *testing.T) {
 	defer a.Shutdown()
 	testrpc.WaitForLeader(t, a.RPC, "dc1")
 
-	ui := cli.NewMockUi()
-
 	// Create a policy
 	client := a.Client()
 
@@ -55,16 +49,17 @@ func TestTokenUpdateCommand(t *testing.T) {
 		&api.ACLPolicy{Name: "test-policy"},
 		&api.WriteOptions{Token: "root"},
 	)
-	req.NoError(err)
+	require.NoError(t, err)
 
 	// create a token
 	token, _, err := client.ACL().TokenCreate(
 		&api.ACLToken{Description: "test"},
 		&api.WriteOptions{Token: "root"},
 	)
-	req.NoError(err)
+	require.NoError(t, err)
 
 	// create a legacy token
+	// nolint: staticcheck // we have to use the deprecated API to create a legacy token
 	legacyTokenSecretID, _, err := client.ACL().Create(&api.ACLEntry{
 		Name:  "Legacy token",
 		Type:  "client",
@@ -72,79 +67,100 @@ func TestTokenUpdateCommand(t *testing.T) {
 	},
 		&api.WriteOptions{Token: "root"},
 	)
-	req.NoError(err)
+	require.NoError(t, err)
 
 	// We fetch the legacy token later to give server time to async background
 	// upgrade it.
 
-	// update with policy by name
-	{
+	run := func(t *testing.T, args []string) *api.ACLToken {
+		ui := cli.NewMockUi()
 		cmd := New(ui)
-		args := []string{
+
+		code := cmd.Run(append(args, "-format=json"))
+		require.Equal(t, 0, code)
+		require.Empty(t, ui.ErrorWriter.String())
+
+		var token api.ACLToken
+		require.NoError(t, json.Unmarshal(ui.OutputWriter.Bytes(), &token))
+		return &token
+	}
+
+	// update with node identity
+	t.Run("node-identity", func(t *testing.T) {
+		token := run(t, []string{
+			"-http-addr=" + a.HTTPAddr(),
+			"-id=" + token.AccessorID,
+			"-token=root",
+			"-node-identity=foo:bar",
+			"-description=test token",
+		})
+
+		require.Len(t, token.NodeIdentities, 1)
+		require.Equal(t, "foo", token.NodeIdentities[0].NodeName)
+		require.Equal(t, "bar", token.NodeIdentities[0].Datacenter)
+	})
+
+	t.Run("node-identity-merge", func(t *testing.T) {
+		token := run(t, []string{
+			"-http-addr=" + a.HTTPAddr(),
+			"-id=" + token.AccessorID,
+			"-token=root",
+			"-node-identity=bar:baz",
+			"-description=test token",
+			"-merge-node-identities",
+		})
+
+		require.Len(t, token.NodeIdentities, 2)
+		expected := []*api.ACLNodeIdentity{
+			&api.ACLNodeIdentity{
+				NodeName:   "foo",
+				Datacenter: "bar",
+			},
+			&api.ACLNodeIdentity{
+				NodeName:   "bar",
+				Datacenter: "baz",
+			},
+		}
+		require.ElementsMatch(t, expected, token.NodeIdentities)
+	})
+
+	// update with policy by name
+	t.Run("policy-name", func(t *testing.T) {
+		token := run(t, []string{
 			"-http-addr=" + a.HTTPAddr(),
 			"-id=" + token.AccessorID,
 			"-token=root",
 			"-policy-name=" + policy.Name,
 			"-description=test token",
-		}
+		})
 
-		code := cmd.Run(args)
-		assert.Equal(code, 0)
-		assert.Empty(ui.ErrorWriter.String())
-
-		token, _, err := client.ACL().TokenRead(
-			token.AccessorID,
-			&api.QueryOptions{Token: "root"},
-		)
-		assert.NoError(err)
-		assert.NotNil(token)
-	}
+		require.Len(t, token.Policies, 1)
+	})
 
 	// update with policy by id
-	{
-		cmd := New(ui)
-		args := []string{
+	t.Run("policy-id", func(t *testing.T) {
+		token := run(t, []string{
 			"-http-addr=" + a.HTTPAddr(),
 			"-id=" + token.AccessorID,
 			"-token=root",
 			"-policy-id=" + policy.ID,
 			"-description=test token",
-		}
+		})
 
-		code := cmd.Run(args)
-		assert.Equal(code, 0)
-		assert.Empty(ui.ErrorWriter.String())
-
-		token, _, err := client.ACL().TokenRead(
-			token.AccessorID,
-			&api.QueryOptions{Token: "root"},
-		)
-		assert.NoError(err)
-		assert.NotNil(token)
-	}
+		require.Len(t, token.Policies, 1)
+	})
 
 	// update with no description shouldn't delete the current description
-	{
-		cmd := New(ui)
-		args := []string{
+	t.Run("merge-description", func(t *testing.T) {
+		token := run(t, []string{
 			"-http-addr=" + a.HTTPAddr(),
 			"-id=" + token.AccessorID,
 			"-token=root",
 			"-policy-name=" + policy.Name,
-		}
+		})
 
-		code := cmd.Run(args)
-		assert.Equal(code, 0)
-		assert.Empty(ui.ErrorWriter.String())
-
-		token, _, err := client.ACL().TokenRead(
-			token.AccessorID,
-			&api.QueryOptions{Token: "root"},
-		)
-		assert.NoError(err)
-		assert.NotNil(token)
-		assert.Equal("test token", token.Description)
-	}
+		require.Equal(t, "test token", token.Description)
+	})
 
 	// Need legacy token now, hopefully server had time to generate an accessor ID
 	// in the background but wait for it if not.
@@ -153,39 +169,28 @@ func TestTokenUpdateCommand(t *testing.T) {
 		// Fetch the legacy token via new API so we can use it's accessor ID
 		legacyToken, _, err = client.ACL().TokenReadSelf(
 			&api.QueryOptions{Token: legacyTokenSecretID})
-		r.Check(err)
+		require.NoError(r, err)
 		require.NotEmpty(r, legacyToken.AccessorID)
 	})
 
 	// upgrade legacy token should replace rules and leave token in a "new" state!
-	{
-		cmd := New(ui)
-		args := []string{
+	t.Run("legacy-upgrade", func(t *testing.T) {
+		token := run(t, []string{
 			"-http-addr=" + a.HTTPAddr(),
 			"-id=" + legacyToken.AccessorID,
 			"-token=root",
 			"-policy-name=" + policy.Name,
 			"-upgrade-legacy",
-		}
+		})
 
-		code := cmd.Run(args)
-		assert.Equal(code, 0)
-		assert.Empty(ui.ErrorWriter.String())
-
-		gotToken, _, err := client.ACL().TokenRead(
-			legacyToken.AccessorID,
-			&api.QueryOptions{Token: "root"},
-		)
-		assert.NoError(err)
-		assert.NotNil(gotToken)
 		// Description shouldn't change
-		assert.Equal("Legacy token", gotToken.Description)
-		assert.Len(gotToken.Policies, 1)
+		require.Equal(t, "Legacy token", token.Description)
+		require.Len(t, token.Policies, 1)
 		// Rules should now be empty meaning this is no longer a legacy token
-		assert.Empty(gotToken.Rules)
+		require.Empty(t, token.Rules)
 		// Secret should not have changes
-		assert.Equal(legacyToken.SecretID, gotToken.SecretID)
-	}
+		require.Equal(t, legacyToken.SecretID, token.SecretID)
+	})
 }
 
 func TestTokenUpdateCommand_JSON(t *testing.T) {
@@ -197,7 +202,7 @@ func TestTokenUpdateCommand_JSON(t *testing.T) {
 	testDir := testutil.TempDir(t, "acl")
 	defer os.RemoveAll(testDir)
 
-	a := agent.NewTestAgent(t, t.Name(), `
+	a := agent.NewTestAgent(t, `
 	primary_datacenter = "dc1"
 	acl {
 		enabled = true
