@@ -11,7 +11,6 @@ import (
 	"github.com/hashicorp/consul/testrpc"
 
 	"github.com/hashicorp/consul/agent"
-	"github.com/hashicorp/consul/agent/config"
 	"github.com/hashicorp/consul/sdk/testutil"
 	"github.com/hashicorp/consul/sdk/testutil/retry"
 	"github.com/mitchellh/cli"
@@ -34,7 +33,7 @@ func TestConfigFail(t *testing.T) {
 		},
 		{
 			args: []string{"agent", "-server", "-bind=10.0.0.1", "-datacenter=foo", "some-other-arg"},
-			out:  "==> config: Unknown extra arguments: [some-other-arg]\n",
+			out:  "==> Unexpected extra arguments: [some-other-arg]\n",
 		},
 		{
 			args: []string{"agent", "-server", "-bind=10.0.0.1"},
@@ -81,10 +80,10 @@ func TestConfigFail(t *testing.T) {
 }
 
 func TestRetryJoin(t *testing.T) {
-	a := agent.NewTestAgent(t, t.Name(), "")
+	a := agent.NewTestAgent(t, "")
 	defer a.Shutdown()
 
-	b := agent.NewTestAgent(t, t.Name(), `
+	b := agent.NewTestAgent(t, `
 		retry_join = ["`+a.Config.SerfBindAddrLAN.String()+`"]
 		retry_join_wan = ["`+a.Config.SerfBindAddrWAN.String()+`"]
 		retry_interval = "100ms"
@@ -205,67 +204,3 @@ func TestBadDataDirPermissions(t *testing.T) {
 		t.Fatalf("expected permission denied error, got: %s", out)
 	}
 }
-
-func TestReloadLoggerFail(t *testing.T) {
-	a := agent.NewTestAgent(t, t.Name(), "")
-	defer a.Shutdown()
-
-	ui := cli.NewMockUi()
-	cmd := New(ui, "", "", "", "", nil)
-
-	bindAddr := a.Config.BindAddr.String()
-	cmd.flagArgs.Config.BindAddr = &bindAddr
-	cmd.flagArgs.Config.DataDir = &a.Config.DataDir
-
-	cmd.logger = testutil.Logger(t)
-
-	newLogLevel := "BLAH"
-	cmd.flagArgs.Config.LogLevel = &newLogLevel
-
-	oldCfg := config.RuntimeConfig{
-		LogLevel: "INFO",
-	}
-	cfg, err := cmd.handleReload(a.Agent, &oldCfg)
-	if err == nil {
-		t.Fatal("Should fail with bad log level")
-	}
-
-	if !strings.Contains(err.Error(), "Invalid log level") {
-		t.Fatalf("expected invalid log level error, got: %s", err)
-	}
-	if cfg.LogLevel != "INFO" {
-		t.Fatalf("expected log level to stay the same, got: %s", cfg.LogLevel)
-	}
-}
-
-func TestReloadLoggerSuccess(t *testing.T) {
-	a := agent.NewTestAgent(t, t.Name(), "")
-	defer a.Shutdown()
-
-	ui := cli.NewMockUi()
-	cmd := New(ui, "", "", "", "", nil)
-
-	bindAddr := a.Config.BindAddr.String()
-	cmd.flagArgs.Config.BindAddr = &bindAddr
-	cmd.flagArgs.Config.DataDir = &a.Config.DataDir
-
-	cmd.logger = testutil.Logger(t)
-
-	newLogLevel := "ERROR"
-	cmd.flagArgs.Config.LogLevel = &newLogLevel
-
-	oldCfg := config.RuntimeConfig{
-		LogLevel: "INFO",
-	}
-	cfg, err := cmd.handleReload(a.Agent, &oldCfg)
-	if err != nil {
-		t.Fatalf("unexpected error: %s", err)
-	}
-
-	if cfg.LogLevel != "ERROR" {
-		t.Fatalf("expected log level to change to 'ERROR', got: %s", cfg.LogLevel)
-	}
-	if cmd.logger.IsWarn() || !cmd.logger.IsError() {
-		t.Fatal("expected logger level to change to 'ERROR'")
-	}
-}
