@@ -147,16 +147,16 @@ func printNodes(nodes []*api.Node, detailed bool) (string, error) {
 		result = simpleNodes(nodes)
 	}
 
-	return columnize.SimpleFormat(result), nil
+	return columnize.Format(result, &columnize.Config{Delim: string([]byte{0x1f})}), nil
 }
 
 func detailedNodes(nodes []*api.Node) []string {
 	result := make([]string, 0, len(nodes)+1)
-	header := "Node|ID|Address|DC|TaggedAddresses|Meta"
+	header := "Node\x1fID\x1fAddress\x1fDC\x1fTaggedAddresses\x1fMeta"
 	result = append(result, header)
 
 	for _, node := range nodes {
-		result = append(result, fmt.Sprintf("%s|%s|%s|%s|%s|%s",
+		result = append(result, fmt.Sprintf("%s\x1f%s\x1f%s\x1f%s\x1f%s\x1f%s",
 			node.Node, node.ID, node.Address, node.Datacenter,
 			mapToKV(node.TaggedAddresses, ", "), mapToKV(node.Meta, ", ")))
 	}
@@ -166,7 +166,7 @@ func detailedNodes(nodes []*api.Node) []string {
 
 func simpleNodes(nodes []*api.Node) []string {
 	result := make([]string, 0, len(nodes)+1)
-	header := "Node|ID|Address|DC"
+	header := "Node\x1fID\x1fAddress\x1fDC"
 	result = append(result, header)
 
 	for _, node := range nodes {
@@ -176,7 +176,7 @@ func simpleNodes(nodes []*api.Node) []string {
 		if idx > 0 {
 			id = id[0:idx]
 		}
-		result = append(result, fmt.Sprintf("%s|%s|%s|%s",
+		result = append(result, fmt.Sprintf("%s\x1f%s\x1f%s\x1f%s",
 			node.Node, id, node.Address, node.Datacenter))
 	}
 
