@@ -34,7 +34,7 @@ func TestCatalogListServicesCommand_Validation(t *testing.T) {
 
 func TestCatalogListServicesCommand(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
 
