@@ -19,7 +19,7 @@ func TestConnectCAGetConfigCommand_noTabs(t *testing.T) {
 
 func TestConnectCAGetConfigCommand(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
 
