@@ -24,7 +24,7 @@ func TestConnectCASetConfigCommand_noTabs(t *testing.T) {
 func TestConnectCASetConfigCommand(t *testing.T) {
 	t.Parallel()
 	require := require.New(t)
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 
 	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
