@@ -59,7 +59,7 @@ func TestKVGetCommand_Validation(t *testing.T) {
 
 func TestKVGetCommand(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 	client := a.Client()
 
@@ -93,7 +93,7 @@ func TestKVGetCommand(t *testing.T) {
 
 func TestKVGetCommand_Base64(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 	client := a.Client()
 
@@ -128,7 +128,7 @@ func TestKVGetCommand_Base64(t *testing.T) {
 
 func TestKVGetCommand_Missing(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 
 	ui := cli.NewMockUi()
@@ -147,7 +147,7 @@ func TestKVGetCommand_Missing(t *testing.T) {
 
 func TestKVGetCommand_Empty(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 	client := a.Client()
 
@@ -176,7 +176,7 @@ func TestKVGetCommand_Empty(t *testing.T) {
 
 func TestKVGetCommand_Detailed(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 	client := a.Client()
 
@@ -220,7 +220,7 @@ func TestKVGetCommand_Detailed(t *testing.T) {
 
 func TestKVGetCommand_Keys(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 	client := a.Client()
 
@@ -255,7 +255,7 @@ func TestKVGetCommand_Keys(t *testing.T) {
 
 func TestKVGetCommand_Recurse(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 	client := a.Client()
 
@@ -295,7 +295,7 @@ func TestKVGetCommand_Recurse(t *testing.T) {
 
 func TestKVGetCommand_RecurseBase64(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 	client := a.Client()
 
@@ -336,7 +336,7 @@ func TestKVGetCommand_RecurseBase64(t *testing.T) {
 
 func TestKVGetCommand_DetailedBase64(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 	client := a.Client()
 
