@@ -41,7 +41,7 @@ func TestMaintCommand_ConflictingArgs(t *testing.T) {
 
 func TestMaintCommand_NoArgs(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 
 	// Register the service and put it into maintenance mode
@@ -90,7 +90,7 @@ func TestMaintCommand_NoArgs(t *testing.T) {
 
 func TestMaintCommand_EnableNodeMaintenance(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 
 	ui := cli.NewMockUi()
@@ -114,7 +114,7 @@ func TestMaintCommand_EnableNodeMaintenance(t *testing.T) {
 
 func TestMaintCommand_DisableNodeMaintenance(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 
 	ui := cli.NewMockUi()
@@ -137,7 +137,7 @@ func TestMaintCommand_DisableNodeMaintenance(t *testing.T) {
 
 func TestMaintCommand_EnableServiceMaintenance(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 
 	// Register the service
@@ -171,7 +171,7 @@ func TestMaintCommand_EnableServiceMaintenance(t *testing.T) {
 
 func TestMaintCommand_DisableServiceMaintenance(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 
 	// Register the service
@@ -204,7 +204,7 @@ func TestMaintCommand_DisableServiceMaintenance(t *testing.T) {
 
 func TestMaintCommand_ServiceMaintenance_NoService(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 
 	ui := cli.NewMockUi()
