@@ -17,15 +17,10 @@ func TestReloadCommand_noTabs(t *testing.T) {
 
 func TestReloadCommand(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 
 	// Setup a dummy response to errCh to simulate a successful reload
-	go func() {
-		errCh := <-a.ReloadCh()
-		errCh <- nil
-	}()
-
 	ui := cli.NewMockUi()
 	c := New(ui)
 	args := []string{"-http-addr=" + a.HTTPAddr()}
