@@ -70,7 +70,7 @@ func TestSnapshotSaveCommand_Validation(t *testing.T) {
 
 func TestSnapshotSaveCommand(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 	client := a.Client()
 
@@ -104,7 +104,7 @@ func TestSnapshotSaveCommand(t *testing.T) {
 
 func TestSnapshotSaveCommand_TruncatedStream(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t, t.Name(), ``)
+	a := agent.NewTestAgent(t, ``)
 	defer a.Shutdown()
 	client := a.Client()
 
@@ -132,7 +132,7 @@ func TestSnapshotSaveCommand_TruncatedStream(t *testing.T) {
 	var fakeResult atomic.Value
 
 	// Run a fake webserver to pretend to be the snapshot API.
-	fakeAddr, cleanup := lib.StartTestServer(http.HandlerFunc(func(w http.ResponseWriter, req *http.Request) {
+	fakeAddr := lib.StartTestServer(t, http.HandlerFunc(func(w http.ResponseWriter, req *http.Request) {
 		if req.URL.Path != "/v1/snapshot" {
 			w.WriteHeader(http.StatusNotFound)
 			return
@@ -151,7 +151,6 @@ func TestSnapshotSaveCommand_TruncatedStream(t *testing.T) {
 		data := raw.([]byte)
 		_, _ = w.Write(data)
 	}))
-	defer cleanup()
 
 	// Wait until the server is actually listening.
 	retry.Run(t, func(r *retry.R) {
