@@ -8,12 +8,12 @@ import (
 	"log"
 	"net"
 	"net/http"
-	"os"
 	"sync/atomic"
 
 	"github.com/hashicorp/consul/agent/connect"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/sdk/freeport"
+	"github.com/hashicorp/go-hclog"
 	testing "github.com/mitchellh/go-testing-interface"
 )
 
@@ -22,8 +22,9 @@ func TestService(t testing.T, service string, ca *structs.CARoot) *Service {
 	t.Helper()
 
 	// Don't need to talk to client since we are setting TLSConfig locally
+	logger := hclog.New(&hclog.LoggerOptions{})
 	svc, err := NewDevServiceWithTLSConfig(service,
-		log.New(os.Stderr, "", log.LstdFlags), TestTLSConfig(t, service, ca))
+		logger, TestTLSConfig(t, service, ca))
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -94,22 +95,24 @@ type TestServer struct {
 	// Listening is closed when the listener is run.
 	Listening chan struct{}
 
-	l        net.Listener
-	stopFlag int32
-	stopChan chan struct{}
+	l             net.Listener
+	returnPortsFn func()
+	stopFlag      int32
+	stopChan      chan struct{}
 }
 
 // NewTestServer returns a TestServer. It should be closed when test is
 // complete.
 func NewTestServer(t testing.T, service string, ca *structs.CARoot) *TestServer {
-	ports := freeport.GetT(t, 1)
+	ports := freeport.MustTake(1)
 	return &TestServer{
-		Service:   service,
-		CA:        ca,
-		stopChan:  make(chan struct{}),
-		TLSCfg:    TestTLSConfig(t, service, ca),
-		Addr:      fmt.Sprintf("127.0.0.1:%d", ports[0]),
-		Listening: make(chan struct{}),
+		Service:       service,
+		CA:            ca,
+		stopChan:      make(chan struct{}),
+		TLSCfg:        TestTLSConfig(t, service, ca),
+		Addr:          fmt.Sprintf("127.0.0.1:%d", ports[0]),
+		Listening:     make(chan struct{}),
+		returnPortsFn: func() { freeport.Return(ports) },
 	}
 }
 
@@ -186,6 +189,10 @@ func (s *TestServer) Close() error {
 		if s.l != nil {
 			s.l.Close()
 		}
+		if s.returnPortsFn != nil {
+			s.returnPortsFn()
+			s.returnPortsFn = nil
+		}
 		close(s.stopChan)
 	}
 	return nil
