@@ -5,6 +5,7 @@ import (
 
 	"github.com/hashicorp/consul/ipaddr"
 	"github.com/hashicorp/consul/sdk/freeport"
+	"github.com/mitchellh/go-testing-interface"
 )
 
 // StartTestServer fires up a web server on a random unused port to serve the
@@ -16,15 +17,20 @@ import (
 // `sdk/freeport` so you can have one part of the test try to use a port and
 // _know_ nothing is listening. If you simply assumed unbound ports were free
 // you'd end up with test cross-talk and weirdness.
-func StartTestServer(handler http.Handler) (string, func()) {
+func StartTestServer(t testing.T, handler http.Handler) string {
 	ports := freeport.MustTake(1)
+	t.Cleanup(func() {
+		freeport.Return(ports)
+	})
+
 	addr := ipaddr.FormatAddressPort("127.0.0.1", ports[0])
 
 	server := &http.Server{Addr: addr, Handler: handler}
+	t.Cleanup(func() {
+		server.Close()
+	})
+
 	go server.ListenAndServe()
 
-	return addr, func() {
-		server.Close()
-		freeport.Return(ports)
-	}
+	return addr
 }
