@@ -3,11 +3,11 @@
 package sentinel
 
 import (
-	"log"
+	"github.com/hashicorp/go-hclog"
 )
 
 // New returns a new instance of the Sentinel code engine. This is only available
 // in Consul Enterprise so this version always returns nil.
-func New(logger *log.Logger) Evaluator {
+func New(logger hclog.Logger) Evaluator {
 	return nil
 }
