@@ -17,6 +17,12 @@ import (
 	"github.com/hashicorp/go-hclog"
 )
 
+// ALPNWrapper is a function that is used to wrap a non-TLS connection and
+// returns an appropriate TLS connection or error. This taks a datacenter and
+// node name as argument to configure the desired SNI value and the desired
+// next proto for configuring ALPN.
+type ALPNWrapper func(dc, nodeName, alpnProto string, conn net.Conn) (net.Conn, error)
+
 // DCWrapper is a function that is used to wrap a non-TLS connection
 // and returns an appropriate TLS connection or error. This takes
 // a datacenter as an argument.
@@ -121,9 +127,9 @@ type Config struct {
 	// and key).
 	EnableAgentTLSForChecks bool
 
-	// AutoEncryptTLS opts the agent into provisioning agent
+	// AutoTLS opts the agent into provisioning agent
 	// TLS certificates.
-	AutoEncryptTLS bool
+	AutoTLS bool
 }
 
 func tlsVersions() []string {
@@ -153,14 +159,14 @@ func SpecificDC(dc string, tlsWrap DCWrapper) Wrapper {
 	}
 }
 
-type autoEncrypt struct {
+type autoTLS struct {
 	manualCAPems         []string
 	connectCAPems        []string
 	cert                 *tls.Certificate
 	verifyServerHostname bool
 }
 
-func (a *autoEncrypt) caPems() []string {
+func (a *autoTLS) caPems() []string {
 	return append(a.manualCAPems, a.connectCAPems...)
 }
 
@@ -174,7 +180,7 @@ type manual struct {
 type Configurator struct {
 	sync.RWMutex
 	base                 *Config
-	autoEncrypt          *autoEncrypt
+	autoTLS              *autoTLS
 	manual               *manual
 	peerDatacenterUseTLS map[string]bool
 
@@ -195,7 +201,7 @@ func NewConfigurator(config Config, logger hclog.Logger) (*Configurator, error)
 	c := &Configurator{
 		logger:               logger.Named(logging.TLSUtil),
 		manual:               &manual{},
-		autoEncrypt:          &autoEncrypt{},
+		autoTLS:              &autoTLS{},
 		peerDatacenterUseTLS: map[string]bool{},
 	}
 	err := c.Update(config)
@@ -209,7 +215,7 @@ func NewConfigurator(config Config, logger hclog.Logger) (*Configurator, error)
 func (c *Configurator) CAPems() []string {
 	c.RLock()
 	defer c.RUnlock()
-	return append(c.manual.caPems, c.autoEncrypt.caPems()...)
+	return append(c.manual.caPems, c.autoTLS.caPems()...)
 }
 
 // ManualCAPems returns the currently loaded CAs in PEM format.
@@ -232,11 +238,11 @@ func (c *Configurator) Update(config Config) error {
 	if err != nil {
 		return err
 	}
-	pems, err := loadCAs(config.CAFile, config.CAPath)
+	pems, err := LoadCAs(config.CAFile, config.CAPath)
 	if err != nil {
 		return err
 	}
-	pool, err := pool(append(pems, c.autoEncrypt.caPems()...))
+	pool, err := pool(append(pems, c.autoTLS.caPems()...))
 	if err != nil {
 		return err
 	}
@@ -251,17 +257,17 @@ func (c *Configurator) Update(config Config) error {
 	return nil
 }
 
-// UpdateAutoEncryptCA updates the autoEncrypt.caPems. This is supposed to be called
+// UpdateAutoTLSCA updates the autoEncrypt.caPems. This is supposed to be called
 // from the server in order to be able to accept TLS connections with TLS
 // certificates.
 // Or it is being called on the client side when CA changes are detected.
-func (c *Configurator) UpdateAutoEncryptCA(connectCAPems []string) error {
+func (c *Configurator) UpdateAutoTLSCA(connectCAPems []string) error {
 	c.Lock()
 	// order of defers matters because log acquires a RLock()
 	defer c.log("UpdateAutoEncryptCA")
 	defer c.Unlock()
 
-	pool, err := pool(append(c.manual.caPems, append(c.autoEncrypt.manualCAPems, connectCAPems...)...))
+	pool, err := pool(append(c.manual.caPems, append(c.autoTLS.manualCAPems, connectCAPems...)...))
 	if err != nil {
 		c.RUnlock()
 		return err
@@ -270,14 +276,14 @@ func (c *Configurator) UpdateAutoEncryptCA(connectCAPems []string) error {
 		c.RUnlock()
 		return err
 	}
-	c.autoEncrypt.connectCAPems = connectCAPems
+	c.autoTLS.connectCAPems = connectCAPems
 	c.caPool = pool
 	c.version++
 	return nil
 }
 
-// UpdateAutoEncryptCert
-func (c *Configurator) UpdateAutoEncryptCert(pub, priv string) error {
+// UpdateAutoTLSCert
+func (c *Configurator) UpdateAutoTLSCert(pub, priv string) error {
 	// order of defers matters because log acquires a RLock()
 	defer c.log("UpdateAutoEncryptCert")
 	cert, err := tls.X509KeyPair([]byte(pub), []byte(priv))
@@ -288,14 +294,14 @@ func (c *Configurator) UpdateAutoEncryptCert(pub, priv string) error {
 	c.Lock()
 	defer c.Unlock()
 
-	c.autoEncrypt.cert = &cert
+	c.autoTLS.cert = &cert
 	c.version++
 	return nil
 }
 
-// UpdateAutoEncrypt sets everything under autoEncrypt. This is being called on the
-// client when it received its cert from AutoEncrypt endpoint.
-func (c *Configurator) UpdateAutoEncrypt(manualCAPems, connectCAPems []string, pub, priv string, verifyServerHostname bool) error {
+// UpdateAutoTLS sets everything under autoEncrypt. This is being called on the
+// client when it received its cert from AutoEncrypt/AutoConfig endpoints.
+func (c *Configurator) UpdateAutoTLS(manualCAPems, connectCAPems []string, pub, priv string, verifyServerHostname bool) error {
 	// order of defers matters because log acquires a RLock()
 	defer c.log("UpdateAutoEncrypt")
 	cert, err := tls.X509KeyPair([]byte(pub), []byte(priv))
@@ -310,11 +316,11 @@ func (c *Configurator) UpdateAutoEncrypt(manualCAPems, connectCAPems []string, p
 	if err != nil {
 		return err
 	}
-	c.autoEncrypt.manualCAPems = manualCAPems
-	c.autoEncrypt.connectCAPems = connectCAPems
-	c.autoEncrypt.cert = &cert
+	c.autoTLS.manualCAPems = manualCAPems
+	c.autoTLS.connectCAPems = connectCAPems
+	c.autoTLS.cert = &cert
 	c.caPool = pool
-	c.autoEncrypt.verifyServerHostname = verifyServerHostname
+	c.autoTLS.verifyServerHostname = verifyServerHostname
 	c.version++
 	return nil
 }
@@ -369,20 +375,19 @@ func (c *Configurator) check(config Config, pool *x509.CertPool, cert *tls.Certi
 
 	// Ensure we have a CA and cert if VerifyIncoming is set
 	if config.anyVerifyIncoming() {
-		autoEncryptMsg := " AutoEncrypt only secures the connection between client and server and doesn't affect incoming connections on the client."
 		if pool == nil {
-			errMsg := "VerifyIncoming set, and no CA certificate provided!"
-			if config.AutoEncryptTLS {
-				errMsg += autoEncryptMsg
-			}
-			return fmt.Errorf(errMsg)
+			// both auto-config and auto-encrypt require verifying the connection from the client to the server for secure
+			// operation. In order to be able to verify the servers certificate we must have some CA certs already provided.
+			// Therefore, even though both of those features can push down extra CA certificates which could be used to
+			// verify incoming connections, we still must consider it an error if none are provided in the initial configuration
+			// as those features cannot be successfully enabled without providing CA certificates to use those features.
+			return fmt.Errorf("VerifyIncoming set but no CA certificates were provided")
 		}
-		if cert == nil {
-			errMsg := "VerifyIncoming set, and no Cert/Key pair provided!"
-			if config.AutoEncryptTLS {
-				errMsg += autoEncryptMsg
-			}
-			return fmt.Errorf(errMsg)
+
+		// We will use the auto_encrypt/auto_config cert for TLS in the incoming APIs when available. Therefore the check
+		// here will ensure that either we enabled one of those two features or a certificate and key were provided manually
+		if cert == nil && !config.AutoTLS {
+			return fmt.Errorf("VerifyIncoming requires either a Cert and Key pair in the configuration file, or auto_encrypt/auto_config be enabled")
 		}
 	}
 	return nil
@@ -415,7 +420,7 @@ func loadKeyPair(certFile, keyFile string) (*tls.Certificate, error) {
 	return &cert, nil
 }
 
-func loadCAs(caFile, caPath string) ([]string, error) {
+func LoadCAs(caFile, caPath string) ([]string, error) {
 	if caFile == "" && caPath == "" {
 		return nil, nil
 	}
@@ -494,11 +499,17 @@ func (c *Configurator) commonTLSConfig(verifyIncoming bool) *tls.Config {
 	// to a server requesting a certificate. Return the autoEncrypt certificate
 	// if possible, otherwise default to the manually provisioned one.
 	tlsConfig.GetClientCertificate = func(*tls.CertificateRequestInfo) (*tls.Certificate, error) {
-		cert := c.autoEncrypt.cert
+		cert := c.autoTLS.cert
 		if cert == nil {
 			cert = c.manual.cert
 		}
 
+		if cert == nil {
+			// the return value MUST not be nil but an empty certificate will be
+			// treated the same as having no client certificate
+			cert = &tls.Certificate{}
+		}
+
 		return cert, nil
 	}
 
@@ -524,7 +535,7 @@ func (c *Configurator) Cert() *tls.Certificate {
 	defer c.RUnlock()
 	cert := c.manual.cert
 	if cert == nil {
-		cert = c.autoEncrypt.cert
+		cert = c.autoTLS.cert
 	}
 	return cert
 }
@@ -542,7 +553,7 @@ func (c *Configurator) outgoingRPCTLSDisabled() bool {
 	defer c.RUnlock()
 
 	// if AutoEncrypt enabled, always use TLS
-	if c.base.AutoEncryptTLS {
+	if c.base.AutoTLS {
 		return false
 	}
 
@@ -554,6 +565,17 @@ func (c *Configurator) outgoingRPCTLSDisabled() bool {
 	return true
 }
 
+func (c *Configurator) MutualTLSCapable() bool {
+	return c.mutualTLSCapable()
+}
+
+// This function acquires a read lock because it reads from the config.
+func (c *Configurator) mutualTLSCapable() bool {
+	c.RLock()
+	defer c.RUnlock()
+	return c.caPool != nil && (c.autoTLS.cert != nil || c.manual.cert != nil)
+}
+
 // This function acquires a read lock because it reads from the config.
 func (c *Configurator) verifyOutgoing() bool {
 	c.RLock()
@@ -561,13 +583,24 @@ func (c *Configurator) verifyOutgoing() bool {
 
 	// If AutoEncryptTLS is enabled and there is a CA, then verify
 	// outgoing.
-	if c.base.AutoEncryptTLS && c.caPool != nil {
+	if c.base.AutoTLS && c.caPool != nil {
 		return true
 	}
 
 	return c.base.VerifyOutgoing
 }
 
+func (c *Configurator) ServerSNI(dc, nodeName string) string {
+	// Strip the trailing '.' from the domain if any
+	domain := strings.TrimSuffix(c.domain(), ".")
+
+	if nodeName == "" || nodeName == "*" {
+		return "server." + dc + "." + domain
+	}
+
+	return nodeName + ".server." + dc + "." + domain
+}
+
 // This function acquires a read lock because it reads from the config.
 func (c *Configurator) domain() string {
 	c.RLock()
@@ -610,7 +643,7 @@ func (c *Configurator) serverNameOrNodeName() string {
 func (c *Configurator) VerifyServerHostname() bool {
 	c.RLock()
 	defer c.RUnlock()
-	return c.base.VerifyServerHostname || c.autoEncrypt.verifyServerHostname
+	return c.base.VerifyServerHostname || c.autoTLS.verifyServerHostname
 }
 
 // IncomingGRPCConfig generates a *tls.Config for incoming GRPC connections.
@@ -639,6 +672,22 @@ func (c *Configurator) IncomingRPCConfig() *tls.Config {
 	return config
 }
 
+// IncomingALPNRPCConfig generates a *tls.Config for incoming RPC connections
+// directly using TLS with ALPN instead of the older byte-prefixed protocol.
+func (c *Configurator) IncomingALPNRPCConfig(alpnProtos []string) *tls.Config {
+	c.log("IncomingALPNRPCConfig")
+	// Since the ALPN-RPC variation is indirectly exposed to the internet via
+	// mesh gateways we force mTLS and full server name verification.
+	config := c.commonTLSConfig(true)
+	config.InsecureSkipVerify = false
+
+	config.GetConfigForClient = func(*tls.ClientHelloInfo) (*tls.Config, error) {
+		return c.IncomingALPNRPCConfig(alpnProtos), nil
+	}
+	config.NextProtos = alpnProtos
+	return config
+}
+
 // IncomingInsecureRPCConfig means that it doesn't verify incoming even thought
 // it might have been configured. This is only supposed to be used by the
 // servers for the insecure RPC server. At the time of writing only the
@@ -694,6 +743,24 @@ func (c *Configurator) OutgoingRPCConfig() *tls.Config {
 	return c.commonTLSConfig(false)
 }
 
+// OutgoingALPNRPCConfig generates a *tls.Config for outgoing RPC connections
+// directly using TLS with ALPN instead of the older byte-prefixed protocol.
+// If there is a CA or VerifyOutgoing is set, a *tls.Config will be provided,
+// otherwise we assume that no TLS should be used which completely disables the
+// ALPN variation.
+func (c *Configurator) OutgoingALPNRPCConfig() *tls.Config {
+	c.log("OutgoingALPNRPCConfig")
+	if !c.mutualTLSCapable() {
+		return nil // ultimately this will hard-fail as TLS is required
+	}
+
+	// Since the ALPN-RPC variation is indirectly exposed to the internet via
+	// mesh gateways we force mTLS and full server name verification.
+	config := c.commonTLSConfig(true)
+	config.InsecureSkipVerify = false
+	return config
+}
+
 // OutgoingRPCWrapper wraps the result of OutgoingRPCConfig in a DCWrapper. It
 // decides if verify server hostname should be used.
 func (c *Configurator) OutgoingRPCWrapper() DCWrapper {
@@ -712,12 +779,25 @@ func (c *Configurator) UseTLS(dc string) bool {
 	return !c.outgoingRPCTLSDisabled() && c.getAreaForPeerDatacenterUseTLS(dc)
 }
 
+// OutgoingALPNRPCWrapper wraps the result of OutgoingALPNRPCConfig in an
+// ALPNWrapper. It configures all of the negotiation plumbing.
+func (c *Configurator) OutgoingALPNRPCWrapper() ALPNWrapper {
+	c.log("OutgoingALPNRPCWrapper")
+	if !c.mutualTLSCapable() {
+		return nil
+	}
+
+	return func(dc, nodeName, alpnProto string, conn net.Conn) (net.Conn, error) {
+		return c.wrapALPNTLSClient(dc, nodeName, alpnProto, conn)
+	}
+}
+
 // AutoEncryptCertNotAfter returns NotAfter from the auto_encrypt cert. In case
 // there is no cert, it will return a time in the past.
 func (c *Configurator) AutoEncryptCertNotAfter() time.Time {
 	c.RLock()
 	defer c.RUnlock()
-	tlsCert := c.autoEncrypt.cert
+	tlsCert := c.autoTLS.cert
 	if tlsCert == nil || tlsCert.Certificate == nil {
 		return time.Now().AddDate(0, 0, -1)
 	}
@@ -738,7 +818,7 @@ func (c *Configurator) log(name string) {
 	if c.logger != nil {
 		c.RLock()
 		defer c.RUnlock()
-		c.logger.Debug(name, "version", c.version)
+		c.logger.Trace(name, "version", c.version)
 	}
 }
 
@@ -808,6 +888,48 @@ func (c *Configurator) wrapTLSClient(dc string, conn net.Conn) (net.Conn, error)
 	return tlsConn, err
 }
 
+// Wrap a net.Conn into a client tls connection suitable for secure ALPN-RPC,
+// performing any additional verification as needed.
+func (c *Configurator) wrapALPNTLSClient(dc, nodeName, alpnProto string, conn net.Conn) (net.Conn, error) {
+	if dc == "" {
+		return nil, fmt.Errorf("cannot dial using ALPN-RPC without a target datacenter")
+	} else if nodeName == "" {
+		return nil, fmt.Errorf("cannot dial using ALPN-RPC without a target node")
+	} else if alpnProto == "" {
+		return nil, fmt.Errorf("cannot dial using ALPN-RPC without a target alpn protocol")
+	}
+
+	config := c.OutgoingALPNRPCConfig()
+	if config == nil {
+		return nil, fmt.Errorf("cannot dial via a mesh gateway when outgoing TLS is disabled")
+	}
+
+	// Since the ALPN-RPC variation is indirectly exposed to the internet via
+	// mesh gateways we force mTLS and full hostname validation (forcing
+	// verify_server_hostname and verify_outgoing to be effectively true).
+
+	config.ServerName = c.ServerSNI(dc, nodeName)
+	config.NextProtos = []string{alpnProto}
+
+	tlsConn := tls.Client(conn, config)
+
+	// NOTE: For this handshake to succeed the server must have key material
+	// for either "<nodename>.server.<datacenter>.<domain>" or
+	// "*.server.<datacenter>.<domain>" in addition to the
+	// "server.<datacenter>.<domain>" required for standard TLS'd RPC.
+	if err := tlsConn.Handshake(); err != nil {
+		tlsConn.Close()
+		return nil, err
+	}
+
+	if cs := tlsConn.ConnectionState(); !cs.NegotiatedProtocolIsMutual {
+		tlsConn.Close()
+		return nil, fmt.Errorf("could not negotiate ALPN protocol %q with %q", alpnProto, config.ServerName)
+	}
+
+	return tlsConn, nil
+}
+
 // ParseCiphers parse ciphersuites from the comma-separated string into
 // recognized slice
 func ParseCiphers(cipherStr string) ([]uint16, error) {
@@ -819,29 +941,18 @@ func ParseCiphers(cipherStr string) ([]uint16, error) {
 	}
 	ciphers := strings.Split(cipherStr, ",")
 
+	// Note: this needs to be kept up to date with the cipherMap in CipherString
 	cipherMap := map[string]uint16{
-		"TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305":    tls.TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305,
-		"TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305":  tls.TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305,
-		"TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256":   tls.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
-		"TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256": tls.TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256,
-		"TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384":   tls.TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384,
-		"TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384": tls.TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384,
-		"TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256":   tls.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256,
-		"TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA":      tls.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA,
-		"TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256": tls.TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256,
 		"TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA":    tls.TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA,
-		"TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA":      tls.TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA,
+		"TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256": tls.TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256,
+		"TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256": tls.TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256,
 		"TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA":    tls.TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA,
-		"TLS_RSA_WITH_AES_128_GCM_SHA256":         tls.TLS_RSA_WITH_AES_128_GCM_SHA256,
-		"TLS_RSA_WITH_AES_256_GCM_SHA384":         tls.TLS_RSA_WITH_AES_256_GCM_SHA384,
-		"TLS_RSA_WITH_AES_128_CBC_SHA256":         tls.TLS_RSA_WITH_AES_128_CBC_SHA256,
-		"TLS_RSA_WITH_AES_128_CBC_SHA":            tls.TLS_RSA_WITH_AES_128_CBC_SHA,
-		"TLS_RSA_WITH_AES_256_CBC_SHA":            tls.TLS_RSA_WITH_AES_256_CBC_SHA,
-		"TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA":     tls.TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA,
-		"TLS_RSA_WITH_3DES_EDE_CBC_SHA":           tls.TLS_RSA_WITH_3DES_EDE_CBC_SHA,
-		"TLS_RSA_WITH_RC4_128_SHA":                tls.TLS_RSA_WITH_RC4_128_SHA,
-		"TLS_ECDHE_RSA_WITH_RC4_128_SHA":          tls.TLS_ECDHE_RSA_WITH_RC4_128_SHA,
-		"TLS_ECDHE_ECDSA_WITH_RC4_128_SHA":        tls.TLS_ECDHE_ECDSA_WITH_RC4_128_SHA,
+		"TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384": tls.TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384,
+		"TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA":      tls.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA,
+		"TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256":   tls.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256,
+		"TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256":   tls.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
+		"TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA":      tls.TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA,
+		"TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384":   tls.TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384,
 	}
 	for _, cipher := range ciphers {
 		if v, ok := cipherMap[cipher]; ok {
@@ -853,3 +964,31 @@ func ParseCiphers(cipherStr string) ([]uint16, error) {
 
 	return suites, nil
 }
+
+// CipherString performs the inverse operation of ParseCiphers
+func CipherString(ciphers []uint16) (string, error) {
+	// Note: this needs to be kept up to date with the cipherMap in ParseCiphers
+	cipherMap := map[uint16]string{
+		tls.TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA:    "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA",
+		tls.TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256: "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256",
+		tls.TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256: "TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256",
+		tls.TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA:    "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA",
+		tls.TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384: "TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384",
+		tls.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA:      "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA",
+		tls.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256:   "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256",
+		tls.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256:   "TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256",
+		tls.TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA:      "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA",
+		tls.TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384:   "TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384",
+	}
+
+	cipherStrings := make([]string, len(ciphers))
+	for i, cipher := range ciphers {
+		if v, ok := cipherMap[cipher]; ok {
+			cipherStrings[i] = v
+		} else {
+			return "", fmt.Errorf("unsupported cipher %d", cipher)
+		}
+	}
+
+	return strings.Join(cipherStrings, ","), nil
+}
