@@ -7,3 +7,7 @@ type AreaID string
 // This represents the existing WAN area that's built in to Consul. Consul
 // Enterprise generalizes areas, which are represented with UUIDs.
 const AreaWAN AreaID = "wan"
+
+// This represents the existing LAN area that's built in to Consul. Consul
+// Enterprise generalizes areas, which are represented with UUIDs.
+const AreaLAN AreaID = "lan"
