@@ -1,7 +1,7 @@
 import Adapter from './application';
 
 export default Adapter.extend({
-  requestForFindAll: function(request) {
+  requestForQuery: function(request) {
     return request`
       GET /v1/catalog/datacenters
     `;
