@@ -6,11 +6,20 @@ import { SLUG_KEY } from 'consul-ui/models/intention';
 
 // TODO: Update to use this.formatDatacenter()
 export default Adapter.extend({
-  requestForQuery: function(request, { dc, index, id }) {
+  requestForQuery: function(request, { dc, filter, index, uri }) {
     return request`
       GET /v1/connect/intentions?${{ dc }}
+      X-Request-ID: ${uri}${
+      typeof filter !== 'undefined'
+        ? `
+      X-Range: ${filter}`
+        : ``
+    }
 
-      ${{ index }}
+      ${{
+        index,
+        filter,
+      }}
     `;
   },
   requestForQueryRecord: function(request, { dc, index, id }) {
@@ -19,6 +28,7 @@ export default Adapter.extend({
     }
     return request`
       GET /v1/connect/intentions/${id}?${{ dc }}
+      Cache-Control: no-store
 
       ${{ index }}
     `;
