@@ -1,19 +1,21 @@
 import Adapter from './application';
 // TODO: Update to use this.formatDatacenter()
 export default Adapter.extend({
-  requestForQuery: function(request, { dc, index, id }) {
+  requestForQuery: function(request, { dc, index, id, uri }) {
     return request`
       GET /v1/internal/ui/nodes?${{ dc }}
+      X-Request-ID: ${uri}
 
       ${{ index }}
     `;
   },
-  requestForQueryRecord: function(request, { dc, index, id }) {
+  requestForQueryRecord: function(request, { dc, index, id, uri }) {
     if (typeof id === 'undefined') {
       throw new Error('You must specify an id');
     }
     return request`
       GET /v1/internal/ui/node/${id}?${{ dc }}
+      X-Request-ID: ${uri}
 
       ${{ index }}
     `;
