@@ -1,11 +1,10 @@
 import Controller from '@ember/controller';
 import { inject as service } from '@ember/service';
-import { get } from '@ember/object';
 export default Controller.extend({
   builder: service('form'),
   init: function() {
     this._super(...arguments);
-    this.form = get(this, 'builder').form('policy');
+    this.form = this.builder.form('policy');
   },
   setProperties: function(model) {
     // essentially this replaces the data with changesets
