@@ -7,7 +7,7 @@ export function serviceExternalSource(params, hash) {
     source = get(params[0], 'Meta.external-source');
   }
   const prefix = typeof hash.prefix === 'undefined' ? '' : hash.prefix;
-  if (source) {
+  if (source && ['kubernetes', 'terraform', 'nomad', 'consul', 'aws'].includes(source)) {
     return `${prefix}${source}`;
   }
   return;
