@@ -17,10 +17,12 @@ export default Model.extend({
     defaultValue: '',
   }),
   // frontend only for ordering where CreateIndex can't be used
-  CreateTime: attr('date'),
+  CreateTime: attr('date', { defaultValue: 0 }),
   //
   Datacenter: attr('string'),
   Namespace: attr('string'),
+  SyncTime: attr('number'),
+  meta: attr(),
   Datacenters: attr(),
   CreateIndex: attr('number'),
   ModifyIndex: attr('number'),
