@@ -1,7 +1,6 @@
 import Route from '@ember/routing/route';
 import { inject as service } from '@ember/service';
 import { hash } from 'rsvp';
-import { get } from '@ember/object';
 
 import WithAclActions from 'consul-ui/mixins/acl/with-actions';
 
@@ -11,7 +10,7 @@ export default Route.extend(WithAclActions, {
   model: function(params) {
     return hash({
       isLoading: false,
-      item: get(this, 'repo').findBySlug(params.id, this.modelFor('dc').dc.Name),
+      item: this.repo.findBySlug(params.id, this.modelFor('dc').dc.Name),
       types: ['management', 'client'],
     });
   },
