@@ -7,7 +7,8 @@ import WithPolicyActions from 'consul-ui/mixins/policy/with-actions';
 export default Route.extend(WithPolicyActions, {
   repo: service('repository/policy'),
   queryParams: {
-    s: {
+    sortBy: 'sort',
+    search: {
       as: 'filter',
       replace: true,
     },
