@@ -7,7 +7,8 @@ import WithRoleActions from 'consul-ui/mixins/role/with-actions';
 export default Route.extend(WithRoleActions, {
   repo: service('repository/role'),
   queryParams: {
-    s: {
+    sortBy: 'sort',
+    search: {
       as: 'filter',
       replace: true,
     },
