@@ -1,7 +1,6 @@
 import SingleRoute from 'consul-ui/routing/single';
 import { inject as service } from '@ember/service';
 import { hash } from 'rsvp';
-import { get } from '@ember/object';
 
 import WithTokenActions from 'consul-ui/mixins/token/with-actions';
 
@@ -13,7 +12,7 @@ export default SingleRoute.extend(WithTokenActions, {
       return hash({
         ...model,
         ...{
-          token: get(this, 'settings').findBySlug('token'),
+          token: this.settings.findBySlug('token'),
         },
       });
     });
