@@ -7,7 +7,8 @@ export default Route.extend(WithTokenActions, {
   repo: service('repository/token'),
   settings: service('settings'),
   queryParams: {
-    s: {
+    sortBy: 'sort',
+    search: {
       as: 'filter',
       replace: true,
     },
