@@ -4,16 +4,18 @@ import { hash } from 'rsvp';
 
 import WithNspaceActions from 'consul-ui/mixins/nspace/with-actions';
 export default Route.extend(WithNspaceActions, {
+  data: service('data-source/service'),
   repo: service('repository/nspace'),
   queryParams: {
-    s: {
+    sortBy: 'sort',
+    search: {
       as: 'filter',
       replace: true,
     },
   },
   model: function(params) {
     return hash({
-      items: this.repo.findAll(),
+      items: this.data.source(uri => uri`/*/*/namespaces`),
       isLoading: false,
     });
   },
