@@ -7,7 +7,7 @@ export default Route.extend({
   repo: service('repository/dc'),
   model: function(params) {
     return hash({
-      item: get(this, 'repo').getActive(),
+      item: this.repo.getActive(),
     });
   },
   afterModel: function({ item }, transition) {
