@@ -9,32 +9,33 @@ export default Route.extend({
   dcRepo: service('repository/dc'),
   nspacesRepo: service('repository/nspace/disabled'),
   model: function(params) {
+    const app = this.modelFor('application');
     return hash({
       item: this.repo.findAll(),
-      dcs: this.dcRepo.findAll(),
-      nspaces: this.nspacesRepo.findAll(),
+      dc: this.dcRepo.getActive(undefined, app.dcs),
       nspace: this.nspacesRepo.getActive(),
     }).then(model => {
       if (typeof get(model.item, 'client.blocking') === 'undefined') {
         set(model, 'item.client', { blocking: true });
       }
-      return hash({
-        ...model,
-        ...{
-          dc: this.dcRepo.getActive(null, model.dcs),
-        },
-      });
+      return model;
     });
   },
   setupController: function(controller, model) {
     controller.setProperties(model);
   },
   actions: {
-    update: function(item) {
-      if (!get(item, 'client.blocking')) {
-        this.client.abort();
+    update: function(slug, item) {
+      switch (slug) {
+        case 'client':
+          if (!get(item, 'client.blocking')) {
+            this.client.abort();
+          }
+          break;
       }
-      this.repo.persist(item);
+      this.repo.persist({
+        [slug]: item,
+      });
     },
   },
 });
