@@ -1,5 +1,4 @@
 import Route from '@ember/routing/route';
-import { get } from '@ember/object';
 import { assert } from '@ember/debug';
 import { Promise, hash } from 'rsvp';
 export default Route.extend({
@@ -8,21 +7,28 @@ export default Route.extend({
     return transition.targetName.split('.').pop() === 'create';
   },
   model: function(params, transition) {
-    const repo = get(this, 'repo');
+    const repo = this.repo;
     assert(
       "`repo` is undefined, please define RepositoryService using `repo: service('repositoryName')`",
       typeof repo !== 'undefined'
     );
     const dc = this.modelFor('dc').dc.Name;
+    const nspace = this.modelFor('nspace').nspace.substr(1);
     const create = this.isCreate(...arguments);
     return hash({
       isLoading: false,
       dc: dc,
+      nspace: nspace,
       create: create,
       ...repo.status({
         item: create
-          ? Promise.resolve(repo.create({ Datacenter: dc }))
-          : repo.findBySlug(params.id, dc),
+          ? Promise.resolve(
+              repo.create({
+                Datacenter: dc,
+                Namespace: nspace,
+              })
+            )
+          : repo.findBySlug(params.id, dc, nspace),
       }),
     });
   },
