@@ -4,6 +4,9 @@ export default function(filterable) {
     const sLower = s.toLowerCase();
     return (
       get(item, 'Node')
+        .toLowerCase()
+        .indexOf(sLower) !== -1 ||
+      get(item, 'Address')
         .toLowerCase()
         .indexOf(sLower) !== -1
     );
