@@ -31,6 +31,11 @@ export default Service.extend({
     inViewportCallbacks = new WeakMap();
     $_ = getComponentFactory(getOwner(this));
   },
+  willDestroy: function() {
+    this._super(...arguments);
+    inViewportCallbacks = null;
+    $_ = null;
+  },
   document: function() {
     return this.doc;
   },
@@ -46,6 +51,24 @@ export default Service.extend({
   sibling: sibling,
   isOutside: isOutside,
   normalizeEvent: normalizeEvent,
+  setEventTargetProperty: function(e, property, cb) {
+    const target = e.target;
+    return new Proxy(e, {
+      get: function(obj, prop, receiver) {
+        if (prop === 'target') {
+          return new Proxy(target, {
+            get: function(obj, prop, receiver) {
+              if (prop === property) {
+                return cb(e.target[property]);
+              }
+              return target[prop];
+            },
+          });
+        }
+        return Reflect.get(...arguments);
+      },
+    });
+  },
   listeners: createListeners,
   root: function() {
     return this.doc.documentElement;
@@ -93,7 +116,7 @@ export default Service.extend({
   },
   isInViewport: function($el, cb, threshold = 0) {
     inViewportCallbacks.set($el, cb);
-    const observer = new IntersectionObserver(
+    let observer = new IntersectionObserver(
       (entries, observer) => {
         entries.map(item => {
           const cb = inViewportCallbacks.get(item.target);
@@ -109,6 +132,13 @@ export default Service.extend({
     );
     observer.observe($el); // eslint-disable-line ember/no-observers
     // observer.unobserve($el);
-    return () => observer.disconnect(); // eslint-disable-line ember/no-observers
+    return () => {
+      observer.unobserve($el); // eslint-disable-line ember/no-observers
+      if (inViewportCallbacks) {
+        inViewportCallbacks.delete($el);
+      }
+      observer.disconnect(); // eslint-disable-line ember/no-observers
+      observer = null;
+    };
   },
 });
