@@ -2,7 +2,6 @@ import RepositoryService from 'consul-ui/services/repository';
 import { inject as service } from '@ember/service';
 import { get } from '@ember/object';
 import Error from '@ember/error';
-import { Promise } from 'rsvp';
 
 const modelName = 'dc';
 export default RepositoryService.extend({
@@ -11,10 +10,7 @@ export default RepositoryService.extend({
     return modelName;
   },
   findAll: function() {
-    return this.store.findAll(this.getModelName()).then(function(items) {
-      // TODO: Move to view/template
-      return items.sortBy('Name');
-    });
+    return this.store.query(this.getModelName(), {});
   },
   findBySlug: function(name, items) {
     if (name != null) {
