@@ -1,7 +1,14 @@
 import RepositoryService from 'consul-ui/services/repository';
+import { PRIMARY_KEY, SLUG_KEY } from 'consul-ui/models/nspace';
 
 const modelName = 'nspace';
 export default RepositoryService.extend({
+  getPrimaryKey: function() {
+    return PRIMARY_KEY;
+  },
+  getSlugKey: function() {
+    return SLUG_KEY;
+  },
   getModelName: function() {
     return modelName;
   },
@@ -9,6 +16,7 @@ export default RepositoryService.extend({
     const query = {};
     if (typeof configuration.cursor !== 'undefined') {
       query.index = configuration.cursor;
+      query.uri = configuration.uri;
     }
     return this.store.query(this.getModelName(), query);
   },
