@@ -1,38 +1,22 @@
 import { inject as service } from '@ember/service';
 import { get } from '@ember/object';
 
+import { restartWhenAvailable } from 'consul-ui/services/client/http';
 import LazyProxyService from 'consul-ui/services/lazy-proxy';
 
 import { cache as createCache, BlockingEventSource } from 'consul-ui/utils/dom/event-source';
 
 const createProxy = function(repo, find, settings, cache, serialize = JSON.stringify) {
   const client = this.client;
-  const store = this.store;
   // custom createEvent, here used to reconcile the ember-data store for each tick
-  const createEvent = function(result, configuration) {
+  const createEvent = function(result = {}, configuration) {
     const event = {
       type: 'message',
       data: result,
     };
-    const meta = get(event.data || {}, 'meta') || {};
-    if (typeof meta.date !== 'undefined') {
-      // unload anything older than our current sync date/time
-      const checkNspace = meta.nspace !== '';
-      store.peekAll(repo.getModelName()).forEach(function(item) {
-        const dc = get(item, 'Datacenter');
-        if (dc === meta.dc) {
-          if (checkNspace) {
-            const nspace = get(item, 'Namespace');
-            if (nspace !== meta.namespace) {
-              return;
-            }
-          }
-          const date = get(item, 'SyncTime');
-          if (typeof date !== 'undefined' && date != meta.date) {
-            store.unloadRecord(item);
-          }
-        }
-      });
+    const meta = get(event, 'data.meta') || {};
+    if (typeof meta.range === 'undefined') {
+      repo.reconcile(meta);
     }
     return event;
   };
@@ -52,24 +36,13 @@ const createProxy = function(repo, find, settings, cache, serialize = JSON.strin
         // original find... with configuration now added
         return repo[find](...args)
           .then(res => {
-            if (!configuration.settings.enabled) {
-              // blocking isn't enabled, immediately close
+            if (!configuration.settings.enabled || typeof res === 'undefined') {
+              // blocking isn't enabled, or we got no data, immediately close
               this.close();
             }
             return res;
           })
-          .catch(function(e) {
-            // setup the aborted connection restarting
-            // this should happen here to avoid cache deletion
-            const status = get(e, 'errors.firstObject.status');
-            if (status === '0') {
-              // Any '0' errors (abort) should possibly try again, depending upon the circumstances
-              // whenAvailable returns a Promise that resolves when the client is available
-              // again
-              return client.whenAvailable(e);
-            }
-            throw e;
-          });
+          .catch(restartWhenAvailable(client));
       },
       {
         key: key,
@@ -83,18 +56,28 @@ const createProxy = function(repo, find, settings, cache, serialize = JSON.strin
   };
 };
 let cache = null;
+let cacheMap = null;
 export default LazyProxyService.extend({
-  store: service('store'),
   settings: service('settings'),
-  wait: service('timeout'),
   client: service('client/http'),
   init: function() {
     this._super(...arguments);
     if (cache === null) {
-      cache = createCache({});
+      this.resetCache();
     }
   },
+  resetCache: function() {
+    Object.entries(cacheMap || {}).forEach(function([key, item]) {
+      item.close();
+    });
+    cacheMap = {};
+    cache = createCache(cacheMap);
+  },
   willDestroy: function() {
+    Object.entries(cacheMap || {}).forEach(function([key, item]) {
+      item.close();
+    });
+    cacheMap = null;
     cache = null;
   },
   shouldProxy: function(content, method) {
