@@ -1,4 +1,4 @@
-import base64js from 'npm:base64-js';
+import base64js from 'base64-js';
 export default function(str, encoding = 'utf-8') {
   // decode
   const bytes = base64js.toByteArray(str);
