@@ -1,9 +1,10 @@
 import { get } from '@ember/object';
-import { Promise } from 'rsvp';
 
 const pause = 2000;
 // native EventSource retry is ~3s wait
-export const create5xxBackoff = function(ms = 3000, P = Promise, wait = setTimeout) {
+// any specified errors here will mean that the blocking query will attempt
+// a reconnection every 3s until it reconnects to Consul
+export const createErrorBackoff = function(ms = 3000, P = Promise, wait = setTimeout) {
   // This expects an ember-data like error
   return function(err) {
     const status = get(err, 'errors.firstObject.status');
@@ -11,6 +12,11 @@ export const create5xxBackoff = function(ms = 3000, P = Promise, wait = setTimeo
       switch (true) {
         // Any '5xx' (not 500) errors should back off and try again
         case status.indexOf('5') === 0 && status.length === 3 && status !== '500':
+        // fallsthrough
+        case status === '0':
+          // TODO: Move this to the view layer so we can show a connection error
+          // and reconnection success to the user
+          // Any 0 aborted connections should back off and try again
           return new P(function(resolve) {
             wait(function() {
               resolve(err);
@@ -52,9 +58,9 @@ const defaultCreateEvent = function(result, configuration) {
  * Wraps an EventSource with functionality to add native EventSource-like functionality
  *
  * @param {Class} [CallableEventSource] - CallableEventSource Class
- * @param {Function} [backoff] - Default backoff function for all instances, defaults to create5xxBackoff
+ * @param {Function} [backoff] - Default backoff function for all instances, defaults to createErrorBackoff
  */
-export default function(EventSource, backoff = create5xxBackoff()) {
+export default function(EventSource, backoff = createErrorBackoff()) {
   /**
    * An EventSource implementation to add native EventSource-like functionality with just callbacks (`cursor` and 5xx backoff)
    *
@@ -97,10 +103,13 @@ export default function(EventSource, backoff = create5xxBackoff()) {
               // pick off the `cursor` from the meta and add it to configuration
               // along with cursor validation
               configuration.cursor = validateCursor(meta.cursor, configuration.cursor);
+              configuration.cacheControl = meta.cacheControl;
             }
-            this.currentEvent = event;
-            this.dispatchEvent(this.currentEvent);
-            const throttledResolve = throttle(configuration, this.currentEvent, this.previousEvent);
+            if ((configuration.cacheControl || '').indexOf('no-store') === -1) {
+              this.currentEvent = event;
+            }
+            this.dispatchEvent(event);
+            const throttledResolve = throttle(configuration, event, this.previousEvent);
             this.previousEvent = this.currentEvent;
             return throttledResolve(result);
           });
