@@ -5,6 +5,12 @@ import Slots from '../mixins/slots';
 const YieldSlotComponent = Component.extend({
   layout,
   tagName: '',
+  _name: computed('name', function() {
+    return this.name;
+  }),
+  _blockParams: computed('params', function() {
+    return this.params;
+  }),
   _parentView: computed(function() {
     return this.nearestOfType(Slots);
   }),
@@ -13,8 +19,4 @@ const YieldSlotComponent = Component.extend({
   }),
 });
 
-YieldSlotComponent.reopenClass({
-  positionalParams: ['_name', '_blockParams'],
-});
-
 export default YieldSlotComponent;
