@@ -1,6 +1,8 @@
 import { module, test } from 'qunit';
 import { setupTest } from 'ember-qunit';
+import getNspaceRunner from 'consul-ui/tests/helpers/get-nspace-runner';
 
+const nspaceRunner = getNspaceRunner('discovery-chain');
 module('Integration | Adapter | discovery-chain', function(hooks) {
   setupTest(hooks);
   const dc = 'dc-1';
@@ -9,11 +11,11 @@ module('Integration | Adapter | discovery-chain', function(hooks) {
     const adapter = this.owner.lookup('adapter:discovery-chain');
     const client = this.owner.lookup('service:client/http');
     const expected = `GET /v1/discovery-chain/${id}?dc=${dc}`;
-    const actual = adapter.requestForQueryRecord(client.url, {
+    const actual = adapter.requestForQueryRecord(client.requestParams.bind(client), {
       dc: dc,
       id: id,
     });
-    assert.equal(actual, expected);
+    assert.equal(`${actual.method} ${actual.url}`, expected);
   });
   test("requestForQueryRecord throws if you don't specify an id", function(assert) {
     const adapter = this.owner.lookup('adapter:discovery-chain');
@@ -24,4 +26,25 @@ module('Integration | Adapter | discovery-chain', function(hooks) {
       });
     });
   });
+  test('requestForQueryRecord returns the correct body', function(assert) {
+    return nspaceRunner(
+      (adapter, serializer, client) => {
+        return adapter.requestForQueryRecord(client.body, {
+          id: id,
+          dc: dc,
+          ns: 'team-1',
+          index: 1,
+        });
+      },
+      {
+        index: 1,
+        ns: 'team-1',
+      },
+      {
+        index: 1,
+      },
+      this,
+      assert
+    );
+  });
 });
