@@ -1,6 +1,6 @@
 import { module, test } from 'qunit';
 import { setupRenderingTest } from 'ember-qunit';
-import { render, find } from '@ember/test-helpers';
+import { render } from '@ember/test-helpers';
 import hbs from 'htmlbars-inline-precompile';
 
 module('Integration | Component | service identity', function(hooks) {
@@ -12,23 +12,13 @@ module('Integration | Component | service identity', function(hooks) {
 
     await render(hbs`{{service-identity}}`);
 
-    assert.ok(
-      find('*')
-        .textContent.trim()
-        .indexOf('service_prefix') !== -1,
-      ''
-    );
+    assert.ok(this.element.textContent.trim().indexOf('service_prefix') !== -1);
 
     // Template block usage:
     await render(hbs`
       {{#service-identity}}{{/service-identity}}
     `);
 
-    assert.ok(
-      find('*')
-        .textContent.trim()
-        .indexOf('service_prefix') !== -1,
-      ''
-    );
+    assert.ok(this.element.textContent.trim().indexOf('service_prefix') !== -1);
   });
 });
