@@ -3,7 +3,7 @@ import { setupRenderingTest } from 'ember-qunit';
 import { render } from '@ember/test-helpers';
 import hbs from 'htmlbars-inline-precompile';
 
-module('helper:default', function(hooks) {
+module('Integration | Helper | default', function(hooks) {
   setupRenderingTest(hooks);
 
   // Replace this with your real tests.
@@ -12,13 +12,13 @@ module('helper:default', function(hooks) {
 
     await render(hbs`{{default inputValue}}`);
 
-    assert.dom('*').hasText('1234');
+    assert.equal(this.element.textContent.trim(), '1234');
   });
   test('it renders the default value', async function(assert) {
     this.set('inputValue', '');
 
     await render(hbs`{{default inputValue '1234'}}`);
 
-    assert.dom('*').hasText('1234');
+    assert.equal(this.element.textContent.trim(), '1234');
   });
 });
