@@ -2,7 +2,7 @@ import { module, skip } from 'qunit';
 import { setupRenderingTest } from 'ember-qunit';
 import hbs from 'htmlbars-inline-precompile';
 
-module('helper:is-href', function(hooks) {
+module('Integration | Helper | is-href', function(hooks) {
   setupRenderingTest(hooks);
 
   // Replace this with your real tests.
@@ -11,6 +11,6 @@ module('helper:is-href', function(hooks) {
 
     this.render(hbs`{{is-href inputValue}}`);
 
-    assert.dom('*').hasText('1234');
+    assert.equal(this.element.textContent.trim(), '1234');
   });
 });
