@@ -3,7 +3,7 @@ import { setupRenderingTest } from 'ember-qunit';
 import { render } from '@ember/test-helpers';
 import hbs from 'htmlbars-inline-precompile';
 
-module('helper:last', function(hooks) {
+module('Integration | Helper | last', function(hooks) {
   setupRenderingTest(hooks);
 
   // Replace this with your real tests.
@@ -12,6 +12,6 @@ module('helper:last', function(hooks) {
 
     await render(hbs`{{last inputValue}}`);
 
-    assert.dom('*').hasText('/');
+    assert.equal(this.element.textContent.trim(), '/');
   });
 });
