@@ -3,7 +3,7 @@ import { setupRenderingTest } from 'ember-qunit';
 import { render } from '@ember/test-helpers';
 import hbs from 'htmlbars-inline-precompile';
 
-module('helper:service/external-source', function(hooks) {
+module('Integration | Helper | service/external-source', function(hooks) {
   setupRenderingTest(hooks);
 
   // Replace this with your real tests.
@@ -12,13 +12,13 @@ module('helper:service/external-source', function(hooks) {
 
     await render(hbs`{{service/external-source inputValue}}`);
 
-    assert.dom('*').hasText('consul');
+    assert.equal(this.element.textContent.trim(), 'consul');
   });
   test('it renders prefixed', async function(assert) {
     this.set('inputValue', { Meta: { 'external-source': 'consul' } });
 
     await render(hbs`{{service/external-source inputValue prefix='external-source-'}}`);
 
-    assert.dom('*').hasText('external-source-consul');
+    assert.equal(this.element.textContent.trim(), 'external-source-consul');
   });
 });
