@@ -3,7 +3,7 @@ import { setupRenderingTest } from 'ember-qunit';
 import { render } from '@ember/test-helpers';
 import hbs from 'htmlbars-inline-precompile';
 
-module('helper:split', function(hooks) {
+module('Integration | Helper | split', function(hooks) {
   setupRenderingTest(hooks);
 
   // Replace this with your real tests.
@@ -12,6 +12,6 @@ module('helper:split', function(hooks) {
 
     await render(hbs`{{split inputValue}}`);
 
-    assert.dom('*').hasText('a,string,split,by,a,comma');
+    assert.equal(this.element.textContent.trim(), 'a,string,split,by,a,comma');
   });
 });
