@@ -1,4 +1,5 @@
 import { moduleFor, test } from 'ember-qunit';
+import { get } from '@ember/object';
 import repo from 'consul-ui/tests/helpers/repo';
 import { createPolicies } from 'consul-ui/tests/helpers/normalizers';
 
@@ -7,11 +8,15 @@ moduleFor(`service:repository/${NAME}`, `Integration | Service | ${NAME}`, {
   // Specify the other units that are required for this test.
   integration: true,
 });
+const now = new Date().getTime();
 const dc = 'dc-1';
 const id = 'role-name';
 const undefinedNspace = 'default';
 [undefinedNspace, 'team-1', undefined].forEach(nspace => {
   test(`findByDatacenter returns the correct data for list endpoint when nspace is ${nspace}`, function(assert) {
+    get(this.subject(), 'store').serializerFor(NAME).timestamp = function() {
+      return now;
+    };
     return repo(
       'Role',
       'findAllByDatacenter',
@@ -33,6 +38,7 @@ const undefinedNspace = 'default';
           expected(function(payload) {
             return payload.map(item =>
               Object.assign({}, item, {
+                SyncTime: now,
                 Datacenter: dc,
                 Namespace: item.Namespace || undefinedNspace,
                 uid: `["${item.Namespace || undefinedNspace}","${dc}","${item.ID}"]`,
