@@ -1,33 +1,50 @@
 import {
-  create,
+  create as createPage,
   clickable,
   is,
   attribute,
   collection,
   text,
   isPresent,
+  triggerable,
 } from 'ember-cli-page-object';
+
 import { alias } from 'ember-cli-page-object/macros';
 import { visitable } from 'consul-ui/tests/lib/page-object/visitable';
+
+// utils
 import createDeletable from 'consul-ui/tests/lib/page-object/createDeletable';
 import createSubmitable from 'consul-ui/tests/lib/page-object/createSubmitable';
 import createCreatable from 'consul-ui/tests/lib/page-object/createCreatable';
 import createCancelable from 'consul-ui/tests/lib/page-object/createCancelable';
 
-import page from 'consul-ui/tests/pages/components/page';
-import radiogroup from 'consul-ui/tests/lib/page-object/radiogroup';
-import freetextFilter from 'consul-ui/tests/pages/components/freetext-filter';
-import catalogFilter from 'consul-ui/tests/pages/components/catalog-filter';
-import aclFilter from 'consul-ui/tests/pages/components/acl-filter';
-import intentionFilter from 'consul-ui/tests/pages/components/intention-filter';
-import tokenListFactory from 'consul-ui/tests/pages/components/token-list';
-import policyFormFactory from 'consul-ui/tests/pages/components/policy-form';
-import policySelectorFactory from 'consul-ui/tests/pages/components/policy-selector';
-import roleFormFactory from 'consul-ui/tests/pages/components/role-form';
-import roleSelectorFactory from 'consul-ui/tests/pages/components/role-selector';
-// TODO: should this specifically be modal or form?
-// should all forms be forms?
+// components
+import pageFactory from 'consul-ui/components/hashicorp-consul/pageobject';
 
+import radiogroup from 'consul-ui/components/radio-group/pageobject';
+import tabgroup from 'consul-ui/components/tab-nav/pageobject';
+import authFormFactory from 'consul-ui/components/auth-form/pageobject';
+import freetextFilterFactory from 'consul-ui/components/freetext-filter/pageobject';
+
+import searchBarFactory from 'consul-ui/components/search-bar/pageobject';
+
+import policyFormFactory from 'consul-ui/components/policy-form/pageobject';
+import policySelectorFactory from 'consul-ui/components/policy-selector/pageobject';
+import roleFormFactory from 'consul-ui/components/role-form/pageobject';
+import roleSelectorFactory from 'consul-ui/components/role-selector/pageobject';
+
+import popoverSelectFactory from 'consul-ui/components/popover-select/pageobject';
+import morePopoverMenuFactory from 'consul-ui/components/more-popover-menu/pageobject';
+
+import tokenListFactory from 'consul-ui/components/token-list/pageobject';
+import consulTokenListFactory from 'consul-ui/components/consul-token-list/pageobject';
+import consulRoleListFactory from 'consul-ui/components/consul-role-list/pageobject';
+import consulPolicyListFactory from 'consul-ui/components/consul-policy-list/pageobject';
+import consulIntentionListFactory from 'consul-ui/components/consul-intention-list/pageobject';
+import consulNspaceListFactory from 'consul-ui/components/consul-nspace-list/pageobject';
+import consulKvListFactory from 'consul-ui/components/consul-kv-list/pageobject';
+
+// pages
 import index from 'consul-ui/tests/pages/index';
 import dcs from 'consul-ui/tests/pages/dc';
 import settings from 'consul-ui/tests/pages/settings';
@@ -51,74 +68,120 @@ import intention from 'consul-ui/tests/pages/dc/intentions/edit';
 import nspaces from 'consul-ui/tests/pages/dc/nspaces/index';
 import nspace from 'consul-ui/tests/pages/dc/nspaces/edit';
 
+// utils
 const deletable = createDeletable(clickable);
 const submitable = createSubmitable(clickable, is);
 const creatable = createCreatable(clickable, is);
 const cancelable = createCancelable(clickable, is);
 
+// components
 const tokenList = tokenListFactory(clickable, attribute, collection, deletable);
-
+const authForm = authFormFactory(submitable, clickable, attribute);
+const freetextFilter = freetextFilterFactory(triggerable);
+const catalogToolbar = searchBarFactory(freetextFilter);
+const catalogFilter = searchBarFactory(freetextFilter, () =>
+  radiogroup('status', ['', 'passing', 'warning', 'critical'])
+);
+const aclFilter = searchBarFactory(freetextFilter, () =>
+  radiogroup('type', ['', 'management', 'client'])
+);
 const policyForm = policyFormFactory(submitable, cancelable, radiogroup, text);
 const policySelector = policySelectorFactory(clickable, deletable, collection, alias, policyForm);
-
 const roleForm = roleFormFactory(submitable, cancelable, policySelector);
 const roleSelector = roleSelectorFactory(clickable, deletable, collection, alias, roleForm);
 
+const morePopoverMenu = morePopoverMenuFactory(clickable);
+const popoverSelect = popoverSelectFactory(clickable, collection);
+
+const consulIntentionList = consulIntentionListFactory(collection, clickable, attribute, deletable);
+const consulNspaceList = consulNspaceListFactory(
+  collection,
+  clickable,
+  attribute,
+  text,
+  morePopoverMenu
+);
+const consulKvList = consulKvListFactory(collection, clickable, attribute, deletable);
+const consulTokenList = consulTokenListFactory(
+  collection,
+  clickable,
+  attribute,
+  text,
+  morePopoverMenu
+);
+const consulRoleList = consulRoleListFactory(
+  collection,
+  clickable,
+  attribute,
+  text,
+  morePopoverMenu
+);
+const consulPolicyList = consulPolicyListFactory(
+  collection,
+  clickable,
+  attribute,
+  text,
+  morePopoverMenu
+);
+
+const page = pageFactory(clickable, attribute, is, authForm);
+
+// pages
+const create = function(appView) {
+  appView = {
+    ...page(),
+    ...appView,
+  };
+  return createPage(appView);
+};
 export default {
   index: create(index(visitable, collection)),
   dcs: create(dcs(visitable, clickable, attribute, collection)),
   services: create(
-    services(visitable, clickable, attribute, collection, page, catalogFilter, radiogroup)
+    services(
+      visitable,
+      clickable,
+      text,
+      attribute,
+      isPresent,
+      collection,
+      popoverSelect,
+      radiogroup
+    )
   ),
-  service: create(service(visitable, attribute, collection, text, catalogFilter, radiogroup)),
-  instance: create(instance(visitable, attribute, collection, text, radiogroup)),
+  service: create(
+    service(
+      visitable,
+      clickable,
+      attribute,
+      collection,
+      text,
+      consulIntentionList,
+      catalogToolbar,
+      tabgroup
+    )
+  ),
+  instance: create(instance(visitable, attribute, collection, text, tabgroup)),
   nodes: create(nodes(visitable, clickable, attribute, collection, catalogFilter)),
-  node: create(node(visitable, deletable, clickable, attribute, collection, radiogroup)),
-  kvs: create(kvs(visitable, deletable, creatable, clickable, attribute, collection)),
+  node: create(node(visitable, deletable, clickable, attribute, collection, tabgroup, text)),
+  kvs: create(kvs(visitable, creatable, consulKvList)),
   kv: create(kv(visitable, attribute, submitable, deletable, cancelable, clickable)),
   acls: create(acls(visitable, deletable, creatable, clickable, attribute, collection, aclFilter)),
   acl: create(acl(visitable, submitable, deletable, cancelable, clickable)),
-  policies: create(
-    policies(
-      visitable,
-      deletable,
-      creatable,
-      clickable,
-      attribute,
-      collection,
-      text,
-      freetextFilter
-    )
-  ),
+  policies: create(policies(visitable, creatable, consulPolicyList, popoverSelect)),
   policy: create(policy(visitable, submitable, deletable, cancelable, clickable, tokenList)),
-  roles: create(
-    roles(visitable, deletable, creatable, clickable, attribute, collection, text, freetextFilter)
-  ),
+  roles: create(roles(visitable, creatable, consulRoleList, popoverSelect)),
   // TODO: This needs a policyList
   role: create(role(visitable, submitable, deletable, cancelable, policySelector, tokenList)),
-  tokens: create(
-    tokens(
-      visitable,
-      submitable,
-      deletable,
-      creatable,
-      clickable,
-      attribute,
-      collection,
-      text,
-      freetextFilter
-    )
-  ),
+  tokens: create(tokens(visitable, creatable, text, consulTokenList, popoverSelect)),
   token: create(
     token(visitable, submitable, deletable, cancelable, clickable, policySelector, roleSelector)
   ),
   intentions: create(
-    intentions(visitable, deletable, creatable, clickable, attribute, collection, intentionFilter)
+    intentions(visitable, creatable, clickable, consulIntentionList, popoverSelect)
   ),
   intention: create(intention(visitable, submitable, deletable, cancelable)),
-  nspaces: create(
-    nspaces(visitable, deletable, creatable, clickable, attribute, collection, text, freetextFilter)
-  ),
+  nspaces: create(nspaces(visitable, creatable, consulNspaceList, popoverSelect)),
   nspace: create(
     nspace(visitable, submitable, deletable, cancelable, policySelector, roleSelector)
   ),
