@@ -1,9 +1,9 @@
 export default function(visitable, submitable, deletable, cancelable, clickable, tokenList) {
   return {
     visit: visitable(['/:dc/acls/policies/:policy', '/:dc/acls/policies/create']),
-    ...submitable({}, 'form > div'),
-    ...cancelable({}, 'form > div'),
-    ...deletable({}, 'form > div'),
+    ...submitable({}, 'main form > div'),
+    ...cancelable({}, 'main form > div'),
+    ...deletable({}, 'main form > div'),
     tokens: tokenList(),
     validDatacenters: clickable('[name="policy[isScoped]"]'),
     datacenter: clickable('[name="policy[Datacenters]"]'),
