@@ -4,6 +4,7 @@ export default function(visitable, submitable, deletable, cancelable) {
       deletable({
         visit: visitable(['/:dc/intentions/:intention', '/:dc/intentions/create']),
       })
-    )
+    ),
+    'main'
   );
 }
